//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "sgraphdoc.h"
#include "sgraphview.h"
#include "AnimatePlotDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnimatePlotDialog dialog


CAnimatePlotDialog::CAnimatePlotDialog(CView *pView)
{
	// Save a pointer to the parent class
	m_pView = pView;

	//{{AFX_DATA_INIT(CAnimatePlotDialog)
	m_plot_num = _T("");
	m_info = _T("");
	//}}AFX_DATA_INIT
}

BOOL CAnimatePlotDialog::Create()
{
	return CDialog::Create(CAnimatePlotDialog::IDD);
}

void CAnimatePlotDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAnimatePlotDialog)
	DDX_Control(pDX, IDC_PLOT_SLIDER, m_plot_slider);
	DDX_Text(pDX, IDC_PLOT_NUM_TEXT, m_plot_num);
	DDV_MaxChars(pDX, m_plot_num, 45);
	DDX_Text(pDX, IDC_INFO_TEXT, m_info);
	DDV_MaxChars(pDX, m_info, 45);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAnimatePlotDialog, CDialog)
	//{{AFX_MSG_MAP(CAnimatePlotDialog)
	ON_WM_HSCROLL()
	ON_MESSAGE(WM_ANIMATE_UPDATE, OnUpdateSlider)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAnimatePlotDialog message handlers

void CAnimatePlotDialog::OnCancel() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_ANIMATE_CLOSE);
	}
}

void CAnimatePlotDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(m_pView != NULL){
		CSgraphDoc* pDoc = ((CSgraphView *)m_pView)->GetDocument();
		ASSERT_VALID(pDoc);
		
		int cur;

		pDoc->graph[pDoc->selected_graph].curr_animate_plot = cur =
			m_plot_slider.GetPos();

		m_plot_num.Format("Plot: %d", cur);

		m_info.Format("%s", 
			pDoc->graph[pDoc->selected_graph].plot[cur].info);

		m_pView->PostMessage(WM_ANIMATE_ADJUST);

		UpdateData(FALSE);
	}
	
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

int CAnimatePlotDialog::GetCurrPlot()
{
	return m_plot_slider.GetPos();
}

BOOL CAnimatePlotDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	OnUpdateSlider(0, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

LRESULT CAnimatePlotDialog::OnUpdateSlider(WPARAM wParam, LPARAM lParam)
{
	if(m_pView != NULL){
		CSgraphDoc* pDoc = ((CSgraphView *)m_pView)->GetDocument();
		ASSERT_VALID(pDoc);

		if(pDoc->graph[pDoc->selected_graph].animate_plots){

			// Make sure that there is at least one plot to view
			if(pDoc->graph[pDoc->selected_graph].plot.Length() == 0){
				m_plot_slider.SetPos(0);
				m_plot_num.Format("Animation has been disabled!");
				m_info.Format("please load at least one plot");
				m_plot_slider.EnableWindow(FALSE);
			}
			else{
				m_plot_slider.EnableWindow(TRUE);

				if(pDoc->graph[pDoc->selected_graph].curr_animate_plot < 0)
					pDoc->graph[pDoc->selected_graph].curr_animate_plot = 0;
				
				int cur = pDoc->graph[pDoc->selected_graph].curr_animate_plot;
				int plot_len = pDoc->graph[pDoc->selected_graph].plot.Length();

				// Make sure that, if the user has deleted any plots, we don't
				// overrun an array boundary
				if(cur >= plot_len){
					pDoc->graph[pDoc->selected_graph].curr_animate_plot = 0;
					cur = 0;
				}

				// Set the range and position of the slider
				m_plot_slider.SetRange(0, plot_len - 1);
				m_plot_slider.SetPos(cur);

				m_plot_num.Format("Plot: %d", cur);

				m_info.Format("%s", 
					pDoc->graph[pDoc->selected_graph].plot[cur].info);
			}
		}
		else{
			m_plot_slider.SetPos(0);
			m_plot_num.Format("Animation has been disabled!");
			m_info.Format("(enable via the plot options dialog)");
			m_plot_slider.EnableWindow(FALSE);
		}
	}
	else{
		m_plot_slider.SetPos(0);
		m_plot_num.Format("Animation has been disabled!");
		m_info.Format("(enable via the plot options dialog)");
		m_plot_slider.EnableWindow(FALSE);
	}

	UpdateData(FALSE);

	return 0L;
}
