//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "sgraph.h"
#include "sgraphDoc.h"
#include "AxisDialog.h"
#include "TickMarkDialog.h"
#include "TickLabelDialog.h"
#include "GridStyleDialog.h"
#include "PrestoUtil.h"
#include "color.h"
#include "PrestoColor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAxisDialog dialog


CAxisDialog::CAxisDialog(CSgraphDoc* pDoc, int axis, 
						 CWnd* pParent /*=NULL*/)
	: CDialog(CAxisDialog::IDD, pParent)
{
	m_pDoc = pDoc;
	m_graph = &(pDoc->graph[pDoc->selected_graph]);

	selected_axis = axis;

	// Intialize the controls in the OnInit() function.
	//{{AFX_DATA_INIT(CAxisDialog)
	m_label = _T("");
	m_major_tick = _T("");
	m_minor_tick = _T("");
	//}}AFX_DATA_INIT
}


void CAxisDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAxisDialog)
	DDX_Control(pDX, IDC_AXIS_COLOR, m_axis_color);
	DDX_Control(pDX, IDC_TICK_GRID, m_grid);
	DDX_Control(pDX, IDC_TICK_MARKS, m_tick_marks);
	DDX_Control(pDX, IDC_TICK_LABELS, m_tick_labels);
	DDX_Control(pDX, IDC_AXIS, m_axis);
	DDX_Control(pDX, IDC_APPLY, m_apply);
	DDX_Text(pDX, IDC_AXIS_LABEL, m_label);
	DDX_Text(pDX, IDC_MAJOR_TICK, m_major_tick);
	DDX_Text(pDX, IDC_MINOR_TICK, m_minor_tick);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAxisDialog, CDialog)
	//{{AFX_MSG_MAP(CAxisDialog)
	ON_BN_CLICKED(IDC_TICK_MARK_BUTTON, OnTickMarkButton)
	ON_BN_CLICKED(IDC_TICK_LABEL_BUTTON, OnTickLabelButton)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_CBN_SELCHANGE(IDC_AXIS, OnSelchangeAxis)
	ON_BN_CLICKED(IDC_GRID_STYLE_BUTTON, OnGridStyleButton)
	ON_CBN_SELCHANGE(IDC_AXIS_COLOR, OnSelchangeAxisColor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAxisDialog message handlers
BOOL CAxisDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	SAxisObject *normal_ptr, *opposite_ptr;

	if(selected_axis == XAxis){
		normal_ptr = &(m_graph->bottomAxis);
		opposite_ptr = &(m_graph->topAxis);

		m_label = m_graph->bottomAxisLable.GetText();
	}
	else{
		normal_ptr = &(m_graph->leftAxis);
		opposite_ptr = &(m_graph->rightAxis);

		m_label = m_graph->leftAxisLable.GetText();
	}

	m_tick_marks.SetCheck(opposite_ptr->isTickDrawn ||
			normal_ptr->isTickDrawn);

	m_tick_labels.SetCheck(opposite_ptr->isTickLableDrawn || 
		normal_ptr->isTickLableDrawn);

	m_grid.SetCheck(opposite_ptr->isGridDrawn ||
			normal_ptr->isGridDrawn);

	m_major_tick.Format("%.6g", normal_ptr->MajorTickSpace());
	m_minor_tick.Format("%.6g", normal_ptr->MinorTickSpace());
	
	// Set the axis color
	int tmp_color = ColorToIndex(normal_ptr->axis_color);

	if(tmp_color < 0){
		// Custom color
		custom_axis_color = -tmp_color;
		m_axis_color.SetCurSel(S_CLR_CUSTOM);
	}
	else{
		custom_axis_color = -1;
		m_axis_color.SetCurSel(tmp_color);
	}

	// Set the selected axis combo box
	m_axis.SetCurSel(selected_axis);
	m_apply.SetCurSel(0 /* Current Axis */);

	UpdateData(FALSE /* Initialize dialog */);

	return TRUE;
}

// Apply any changes that the user may have made
void CAxisDialog::OnOK() 
{
	OnApply();

	CDialog::OnOK();
}

void CAxisDialog::OnTickMarkButton() 
{
	CTickMarkDialog tmDlg(m_pDoc, m_axis.GetCurSel());

	if(tmDlg.DoModal() == IDOK){
		m_pDoc->UpdateAllViews(NULL);
		m_pDoc->SetModifiedFlag();

		// Check the check box on the dialog
		m_tick_marks.SetCheck(TRUE);
	}
}

void CAxisDialog::OnTickLabelButton() 
{
	CTickLabelDialog tlDlg(m_pDoc, m_axis.GetCurSel());

	if(tlDlg.DoModal() == IDOK){
		m_pDoc->UpdateAllViews(NULL);
		m_pDoc->SetModifiedFlag();

		// Check the  check box on the dialog
		m_tick_labels.SetCheck(TRUE);
	}
}

void CAxisDialog::OnGridStyleButton() 
{
	CGridStyleDialog gsDlg(m_pDoc, m_axis.GetCurSel());

	if(gsDlg.DoModal() == IDOK){
		m_pDoc->UpdateAllViews(NULL);
		m_pDoc->SetModifiedFlag();

		// Check the check box on the dialog
		m_grid.SetCheck(TRUE);
	}
}


void CAxisDialog::OnApply()
{
	float major_tick_space, minor_tick_space;

	UpdateData(TRUE /* Retrieve data from dialog */);

	// Make sure that the user has entered valid numbers for the 
	// major and minor tick spaceing.
	if(StringToFloat(m_major_tick, major_tick_space) == FALSE){
		AfxMessageBox("Please enter a valid number for the major "
			"tick spacing.");
		return;
	}

	if(major_tick_space <= 0.0f){
		AfxMessageBox("Please enter a positive number for the major "
			"tick spacing.");
		return;
	}

	if(StringToFloat(m_minor_tick, minor_tick_space) == FALSE){
		AfxMessageBox("Please enter a valid number for the minor "
			"tick spacing.");
		return;
	}

	if(minor_tick_space <= 0.0f){
		AfxMessageBox("Please enter a positive number for the minor "
			"tick spacing.");
		return;
	}

	// What graphs and axes do we apply the changes to?
	int apply = m_apply.GetCurSel();

	int tick_location, label_location;
	// Get the selected axis
	selected_axis = m_axis.GetCurSel();

	SAxisObject *ptr1, *ptr2;

	// The four different possibilities for the apply combo box
	if(apply == 0 /* Current Graph */){
		if(selected_axis == XAxis){
			ptr1 = &(m_graph->bottomAxis);
			
			ptr2 = &(m_graph->topAxis);

			m_graph->bottomAxisLable.SetText(m_label);

			tick_location = m_graph->x_tick_location;
			label_location = m_graph->x_tick_label_location;
		}
		else{
			ptr1 = &(m_graph->leftAxis);
			
			ptr2 = &(m_graph->rightAxis);

			m_graph->leftAxisLable.SetText(m_label);

			tick_location = m_graph->y_tick_location;
			label_location = m_graph->y_tick_label_location;
		}
		
		ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_NORMAL) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_NORMAL) || 
			(tick_location == SGraph::TICK_MARK_BOTH));
		
		ptr1->MajorTickSpace(major_tick_space, TRUE);
		ptr1->MinorTickSpace(minor_tick_space, TRUE);

		ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_OPPOSITE) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
			(tick_location == SGraph::TICK_MARK_BOTH));

		ptr2->MajorTickSpace(major_tick_space, TRUE);
		ptr2->MinorTickSpace(minor_tick_space, TRUE);

		ptr1->isGridDrawn = m_grid.GetCheck();

		if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
			ptr1->axis_color = ptr2->axis_color = 
				(COLORREF)(custom_axis_color);
		}
		else{
			ptr1->axis_color = ptr2->axis_color = 
				IndexToColor(m_axis_color.GetCurSel());
		}

		// Set the sticky flag
		ptr1->isStickyTickSpace = TRUE;
		ptr2->isStickyTickSpace = TRUE;
	}
	
	if(apply == 1 /* All Axes, Current Graph */){
		ptr1 = &(m_graph->bottomAxis);
			
		ptr2 = &(m_graph->topAxis);

		m_graph->bottomAxisLable.SetText(m_label);
		
		tick_location = m_graph->x_tick_location;
		label_location = m_graph->x_tick_label_location;

		ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_NORMAL) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_NORMAL) || 
			(tick_location == SGraph::TICK_MARK_BOTH));
		
		ptr1->MajorTickSpace(major_tick_space, TRUE);
		ptr1->MinorTickSpace(minor_tick_space, TRUE);

		ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_OPPOSITE) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
			(tick_location == SGraph::TICK_MARK_BOTH));

		ptr2->MajorTickSpace(major_tick_space, TRUE);
		ptr2->MinorTickSpace(minor_tick_space, TRUE);

		if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
			ptr1->axis_color = ptr2->axis_color = 
				(COLORREF)(custom_axis_color);
		}
		else{
			ptr1->axis_color = ptr2->axis_color = 
				IndexToColor(m_axis_color.GetCurSel());
		}

		// Set the sticky flag
		ptr1->isStickyTickSpace = TRUE;
		ptr2->isStickyTickSpace = TRUE;

		ptr1->isGridDrawn = m_grid.GetCheck();

		ptr1 = &(m_graph->leftAxis);
			
		ptr2 = &(m_graph->rightAxis);

		m_graph->leftAxisLable.SetText(m_label);
		
		tick_location = m_graph->y_tick_location;
		label_location = m_graph->y_tick_label_location;

		ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_NORMAL) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_NORMAL) || 
			(tick_location == SGraph::TICK_MARK_BOTH));
		
		ptr1->MajorTickSpace(major_tick_space, TRUE);
		ptr1->MinorTickSpace(minor_tick_space, TRUE);
		
		ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
			((label_location == SGraph::TICK_MARK_OPPOSITE) || 
			(label_location == SGraph::TICK_MARK_BOTH));
		ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
			((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
			(tick_location == SGraph::TICK_MARK_BOTH));

		ptr2->MajorTickSpace(major_tick_space, TRUE);
		ptr2->MinorTickSpace(minor_tick_space, TRUE);

		ptr1->isGridDrawn = m_grid.GetCheck();

		if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
			ptr1->axis_color = ptr2->axis_color = 
				(COLORREF)(custom_axis_color);
		}
		else{
			ptr1->axis_color = ptr2->axis_color = 
				IndexToColor(m_axis_color.GetCurSel());
		}

		// Set the sticky flag
		ptr1->isStickyTickSpace = TRUE;
		ptr2->isStickyTickSpace = TRUE;
	}

	if(apply == 2 /* Current Axis, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++){

			if(selected_axis == XAxis){
				ptr1 = &(m_pDoc->graph[i].bottomAxis);
			
				ptr2 = &(m_pDoc->graph[i].topAxis);

				m_pDoc->graph[i].bottomAxisLable.SetText(m_label);

				tick_location = m_pDoc->graph[m_pDoc->selected_graph].x_tick_location;
				label_location = m_pDoc->graph[m_pDoc->selected_graph].x_tick_label_location;
			}
			else{
				ptr1 = &(m_pDoc->graph[i].leftAxis);
			
				ptr2 = &(m_pDoc->graph[i].rightAxis);

				m_pDoc->graph[i].leftAxisLable.SetText(m_label);

				tick_location = m_pDoc->graph[m_pDoc->selected_graph].y_tick_location;
				label_location = m_pDoc->graph[m_pDoc->selected_graph].y_tick_label_location;
			}

			ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_NORMAL) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_NORMAL) || 
				(tick_location == SGraph::TICK_MARK_BOTH));
		
			ptr1->MajorTickSpace(major_tick_space, TRUE);
			ptr1->MinorTickSpace(minor_tick_space, TRUE);
			
			ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_OPPOSITE) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
				(tick_location == SGraph::TICK_MARK_BOTH));

			ptr2->MajorTickSpace(major_tick_space, TRUE);
			ptr2->MinorTickSpace(minor_tick_space, TRUE);

			ptr1->isGridDrawn = m_grid.GetCheck();

			if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
				ptr1->axis_color = ptr2->axis_color = 
					(COLORREF)(custom_axis_color);
			}
			else{
				ptr1->axis_color = ptr2->axis_color = 
					IndexToColor(m_axis_color.GetCurSel());
			}
			
			// Set the sticky flag
			ptr1->isStickyTickSpace = TRUE;
			ptr2->isStickyTickSpace = TRUE;
		}
	}

	if(apply == 3 /* All Axes, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++){
			
			ptr1 = &(m_pDoc->graph[i].bottomAxis);
			
			ptr2 = &(m_pDoc->graph[i].topAxis);

			m_pDoc->graph[i].bottomAxisLable.SetText(m_label);
			
			tick_location = m_pDoc->graph[m_pDoc->selected_graph].x_tick_location;
			label_location = m_pDoc->graph[m_pDoc->selected_graph].x_tick_label_location;

			ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_NORMAL) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_NORMAL) || 
				(tick_location == SGraph::TICK_MARK_BOTH));

			ptr1->MajorTickSpace(major_tick_space, TRUE);
			ptr1->MinorTickSpace(minor_tick_space, TRUE);
			
			ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_OPPOSITE) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
				(tick_location == SGraph::TICK_MARK_BOTH));

			ptr2->MajorTickSpace(major_tick_space, TRUE);
			ptr2->MinorTickSpace(minor_tick_space, TRUE);

			// Set the sticky flag
			ptr1->isStickyTickSpace = TRUE;
			ptr2->isStickyTickSpace = TRUE;

			ptr1->isGridDrawn = m_grid.GetCheck();

			ptr1 = &(m_pDoc->graph[i].leftAxis);
		
			ptr2 = &(m_pDoc->graph[i].rightAxis);

			m_pDoc->graph[i].leftAxisLable.SetText(m_label);
			
			tick_location = m_pDoc->graph[m_pDoc->selected_graph].y_tick_location;
			label_location = m_pDoc->graph[m_pDoc->selected_graph].y_tick_label_location;

			ptr1->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_NORMAL) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr1->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_NORMAL) || 
				(tick_location == SGraph::TICK_MARK_BOTH));
		
			ptr1->MajorTickSpace(major_tick_space, TRUE);
			ptr1->MinorTickSpace(minor_tick_space, TRUE);
			
			ptr2->isTickLableDrawn = m_tick_labels.GetCheck() && 
				((label_location == SGraph::TICK_MARK_OPPOSITE) || 
				(label_location == SGraph::TICK_MARK_BOTH));
			ptr2->isTickDrawn = m_tick_marks.GetCheck() && 
				((tick_location == SGraph::TICK_MARK_OPPOSITE) || 
				(tick_location == SGraph::TICK_MARK_BOTH));

			ptr2->MajorTickSpace(major_tick_space, TRUE);
			ptr2->MinorTickSpace(minor_tick_space, TRUE);

			ptr1->isGridDrawn = m_grid.GetCheck();

			if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
				ptr1->axis_color = ptr2->axis_color = 
					(COLORREF)(custom_axis_color);
			}
			else{
				ptr1->axis_color = ptr2->axis_color = 
					IndexToColor(m_axis_color.GetCurSel());
			}

			// Set the sticky flag
			ptr1->isStickyTickSpace = TRUE;
			ptr2->isStickyTickSpace = TRUE;
		}
	}

	m_pDoc->UpdateAllViews(NULL);
	m_pDoc->SetModifiedFlag();
}

void CAxisDialog::OnSelchangeAxis()
{
	SAxisObject *normal_ptr, *opposite_ptr;

	// Get the selected axis
	selected_axis = m_axis.GetCurSel();

	if(selected_axis == XAxis){
		normal_ptr = &(m_graph->bottomAxis);
		opposite_ptr = &(m_graph->topAxis);

		m_label = m_graph->bottomAxisLable.GetText();
	}
	else{
		normal_ptr = &(m_graph->leftAxis);
		opposite_ptr = &(m_graph->rightAxis);

		m_label = m_graph->leftAxisLable.GetText();
	}

	m_tick_marks.SetCheck(opposite_ptr->isTickDrawn ||
			normal_ptr->isTickDrawn);

	m_tick_labels.SetCheck(opposite_ptr->isTickLableDrawn || 
		normal_ptr->isTickLableDrawn);

	m_grid.SetCheck(opposite_ptr->isGridDrawn || 
		normal_ptr->isGridDrawn);

	m_major_tick.Format("%.6g", normal_ptr->MajorTickSpace());
	m_minor_tick.Format("%.6g", normal_ptr->MinorTickSpace());

	// Set the axis color
	int tmp_color = ColorToIndex(normal_ptr->axis_color);

	if(tmp_color < 0){
		// Custom color
		custom_axis_color = -tmp_color;
		m_axis_color.SetCurSel(S_CLR_CUSTOM);
	}
	else{
		custom_axis_color = -1;
		m_axis_color.SetCurSel(tmp_color);
	}

	UpdateData(FALSE /* Initialize dialog */);
}

void CAxisDialog::OnSelchangeAxisColor() 
{
	// Allow the user to specify a custom line color
	if(m_axis_color.GetCurSel() == S_CLR_CUSTOM){
		COLORREF tmp_color;

		if((custom_axis_color == NO_COLOR) || (custom_axis_color > 0))
			tmp_color = custom_axis_color;
		else
			tmp_color = (COLORREF)(0);

		CPrestoColor pcDlg(tmp_color);

		if(pcDlg.DoModal() == IDOK)
			custom_axis_color = (int)(pcDlg.GetColor());
	}
}
