//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "CommandLine.h"
#include "Excepts.h"
#include "io.h"

// Helper function(s)
CString GetWildcardPathInfo(LPCTSTR lpszParam);

// Current flags and arguments that are recognized:
// filename									Open filename
// -nxy		filename	[filename2 ...]		Import nxy data from filename
// -xy		filename	[filename2 ...]		Import xy data from filename
// -xydx	filename	[filename2 ...]		Import xydx data from filename
// -xydy	filename	[filename2 ...]		Import xydy data from filename
// -xydxdy	filename	[filename2 ...]		Import xydxdy data from filename
// -wav		filename	[filename2 ...]		Import audio data from filename
// -pipe	pipe_addr						Listen to the pipe specified by pipe_addr
// -logx									Use a log scale for the x axis
// -logy									Use a log scale for the y axis
// -logxy									Use a log scale for the x and y axes
// -help									List the allowed command line options
// -?										List the allowed command line options
void CSGraphCommandLine::ParseParam(LPCTSTR lpszParam, 
									BOOL bFlag, BOOL bLast)
{
	// Take care of the base class
	// ParseParam(lpszParam, bFlag, bLast);
	
	// Do we expect that the current lpszParam contains
	// a filename for data import?
	static BOOL read_file_name = FALSE;
	
	// Have we read a flag?
	if(bFlag){
		if(read_file_name){
			m_importXY = FALSE;
			m_importNXY = FALSE;
			m_importXYDX = FALSE;
			m_importXYDY = FALSE;
			m_importXYDXDY = FALSE;
			m_importWAV = FALSE;

			throw IO_Error("A filename was not found on the "
				"command line", __FILE__, __LINE__);
		}
		
		if(strcmp(lpszParam, "xy") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importXY = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importXY = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "nxy") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importNXY = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importNXY = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "xydx") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importXYDX = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importXYDX = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "xydy") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importXYDY = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importXYDY = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "xydxdy") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importXYDXDY = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importXYDXDY = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "wav") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_importWAV = FALSE;
				throw IO_Error("A filename was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_importWAV = TRUE;
				read_file_name = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "pipe") == 0){
			// The next item we parse must be the filename
			if(bLast){
				m_pipe = FALSE;
				throw IO_Error("A pipe address was not found on the "
					"command line", __FILE__, __LINE__);
			}
			else{
				m_pipe = TRUE;

				return;
			}
		}

		if(strcmp(lpszParam, "?") == 0){
			m_displayCommandLine = TRUE;

			return;
		}

		if(strcmp(lpszParam, "help") == 0){
			m_displayCommandLine = TRUE;

			return;
		}

		if(strcmp(lpszParam, "logx") == 0){
			m_logX = TRUE;

			return;
		}

		if(strcmp(lpszParam, "logy") == 0){
			m_logY = TRUE;

			return;
		}

		if(strcmp(lpszParam, "logxy") == 0){
			m_logX = TRUE;
			m_logY = TRUE;

			return;
		}
		// If we get here, an uknown flag was encountered on the
		// command line.
		throw IO_Error("A unknown flag was found on the "
					"command line", __FILE__, __LINE__);
	}
	else{
		if(read_file_name){
			
			// Does this name contain a wild card?
			if(strchr(lpszParam, '*') != NULL){
				// This string contains a wild card, save the file name(s).
				struct _finddata_t fileinfo;
				long fileHandle;

				fileHandle = _findfirst(lpszParam, &fileinfo);

				if(fileHandle == -1){
					throw IO_Error("The wildcard failed to match a file name", 
						__FILE__, __LINE__);
				}
				
				CString pathInfo = GetWildcardPathInfo(lpszParam);

				// Iterate over all file names that match the wildcard pattern while
				// making sure to append any path information.
				while(_findnext(fileHandle, &fileinfo) == 0){
					m_import_filename.Push(pathInfo + CString(fileinfo.name));
				}

				_findclose(fileHandle);
			}
			else{
				// Save the file name
				m_import_filename.Push(CString(lpszParam));
			}

			return;
		}
		else{
			if(m_pipe)
				m_pipe_addr = lpszParam;
			else{
				// The user has specified a sgraph file to open
				// (NOT import).
				m_strFileName = lpszParam;
				m_nShellCommand = CCommandLineInfo::FileOpen;
			}
		}
	}

	if(m_pipe){
		// If we're reading data from a pipe,
		// turn off all other options
		m_importXY = FALSE;
		m_importNXY = FALSE;
		m_importXYDX = FALSE;
		m_importXYDY = FALSE;
		m_importXYDXDY = FALSE;
		m_importWAV = FALSE;
		m_displayCommandLine = FALSE;
		m_logX = FALSE;
		m_logY = FALSE;

		return;
	}
}

CString GetWildcardPathInfo(LPCTSTR lpszParam)
{
	CString pathinfo = _T("");

	// First identify the location of the wildcard character
	char *ptr = strchr(lpszParam, '*');

	if(ptr == NULL)
		return pathinfo;

	int len = 0;

	// Now search backwards for the location of the first '\' character
	while(ptr != lpszParam){
		if(*ptr == '\\'){
			len = ptr - lpszParam + 1;
			char *tmpString = new char [len + 1];
			strncpy(tmpString, lpszParam, len);
			tmpString[len] = '\0';

			pathinfo = tmpString;

			return pathinfo;
		}

		ptr -= 1;
	}
	
	return pathinfo;
}
