//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "DerivativeDialog.h"
#include "sgraphDoc.h"
#include "color.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Local function definitions
float DiscreteDeriv3(DataPoint *xy, int a, int b, int c);

/////////////////////////////////////////////////////////////////////////////
// CDerivativeDialog dialog


CDerivativeDialog::CDerivativeDialog(CDocument* pDoc, 
									 CWnd* pParent /*=NULL*/)
	: CDialog(CDerivativeDialog::IDD, pParent)
{
	m_pDoc = pDoc;
	
	// By default, choose a centered discrete derivative
	deriv_loc = IDC_RADIO_CENTER;
	
	// By default, choose a three point derivative.
	deriv_num_point = THREE_PT_DERIV;

	//{{AFX_DATA_INIT(CDerivativeDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDerivativeDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDerivativeDialog)
	DDX_Control(pDX, IDC_NUM_POINT, m_num_point);
	DDX_Control(pDX, IDC_GRAPH, m_graph);
	DDX_Control(pDX, IDC_DIFF_LIST, m_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDerivativeDialog, CDialog)
	//{{AFX_MSG_MAP(CDerivativeDialog)
	ON_LBN_SELCHANGE(IDC_DIFF_LIST, OnSelectChange)
	ON_BN_CLICKED(IDC_RADIO_CENTER, OnRadioCenter)
	ON_BN_CLICKED(IDC_RADIO_LEFT, OnRadioLeft)
	ON_BN_CLICKED(IDC_RADIO_RIGHT, OnRadioRight)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDerivativeDialog message handlers
BOOL CDerivativeDialog::OnInitDialog()
{
	int i;
	
	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_list, m_pDoc);

	// Set the derivative location (i.e. centered, left or right)
	// radio buttons
	switch(deriv_loc){
		case IDC_RADIO_CENTER:
			CheckRadioButton(IDC_RADIO_CENTER,		// First button
							 IDC_RADIO_RIGHT,		// Last bottom	
							 IDC_RADIO_CENTER);		// Button to check
			break;
		case IDC_RADIO_LEFT:
			CheckRadioButton(IDC_RADIO_CENTER,		// First button
							 IDC_RADIO_RIGHT,		// Last bottom	
							 IDC_RADIO_LEFT);		// Button to check
			break;
		case IDC_RADIO_RIGHT:
			CheckRadioButton(IDC_RADIO_CENTER,		// First button
							 IDC_RADIO_RIGHT,		// Last bottom	
							 IDC_RADIO_RIGHT);		// Button to check
			break;
	};

	// Fill the "num point" combo box with the appropriate choices
	// (i.e. three point derivative or five point derivative).
	m_num_point.SetCurSel(deriv_num_point);

	// Fill the graph combo box with various graph choices
	m_graph.AddString("Current");

	for(i = 0;i < ((CSgraphDoc *)m_pDoc)->num_graph;i++)
		if(((CSgraphDoc *)m_pDoc)->graph[i].isDrawn){
			CString counter;
			counter.Format("%d", i);
			m_graph.AddString(counter);
		}
	
	// Make the "Current" string the combo box default
	m_graph.SetCurSel(0);

	return TRUE;
}

void CDerivativeDialog::OnOK() 
{
	int i, plot_count;
	int destination_graph;

	// This could take a while for large data sets
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);
	
	// Read in all of the variables from the dialog box
	UpdateData();

	// Has the user selected a plot(s)?
	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected)
			plot_count ++;

	if(plot_count == 0){
		// Restore the original cursor
		::SetCursor(prevCursor);
		AfxMessageBox("No Plots have been selected!");
		return;
	}
	
	// Get the number of points to be used in the derivative
	deriv_num_point = m_num_point.GetCurSel();

	///////////////////////////////////////////
	// Perform some sanity checks on the inputs
	///////////////////////////////////////////
	if(deriv_num_point == FIVE_PT_DERIV){
		// Restore the original cursor
		::SetCursor(prevCursor);

		// Not currently supported!
		AfxMessageBox("The five point discrete derivative is currently "
			"unsupported by this version. Sorry!");
		return;
	}

	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected){
			switch(deriv_num_point){
				case THREE_PT_DERIV:
					if(m_ptr->plot[i].data.Length() < 3){
						// Restore the original cursor
						::SetCursor(prevCursor);

						AfxMessageBox("At least three points are needed "
							 "to compute a three point derivative.");
						return;
					}
					break;
				case FIVE_PT_DERIV:
					if(m_ptr->plot[i].data.Length() < 5){
						// Restore the original cursor
						::SetCursor(prevCursor);

						AfxMessageBox("At least five points are needed "
							 "to compute a five point derivative.");
						return;
					}
					break;
			};
		}

	// Get the destination graph
	if((destination_graph = m_graph.GetCurSel()) == 0)
		// The current graph
		destination_graph = ((CSgraphDoc *)m_pDoc)->selected_graph;
	else
		destination_graph --;

	SGraph *d_graph = &(((CSgraphDoc *)m_pDoc)->graph[destination_graph]);

	// Compute the histogram(s)
	for(i = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected){
			SPlot newPlot;

			Derivative(newPlot, m_ptr->plot[i], i);

			d_graph->plot.Push(newPlot);

			// Are we inserting a plot into a log or log-log graph?
			if(((d_graph->x_scaleType == SCALE_LOG) && !newPlot.canXLog) ||
				((d_graph->y_scaleType == SCALE_LOG) && !newPlot.canYLog)){
				// Restore the original cursor
				::SetCursor(prevCursor);

				AfxMessageBox("Warning! Can't use a log axis with negative or zero "
					"values!");
				
				int x_scale = d_graph->x_scaleType;
				int y_scale = d_graph->y_scaleType;

				if(!newPlot.canXLog)
					x_scale = SCALE_LINEAR;
				
				if(!newPlot.canYLog)
					y_scale = SCALE_LINEAR;

				d_graph->SetGraphType(x_scale, y_scale);
			}
		}

	// Update the plot range and update the document
	d_graph->SetPlotRange();
	d_graph->SetBoundingBox();

	((CSgraphDoc *)m_pDoc)->SetModifiedFlag();

	// Restore the original cursor
	::SetCursor(prevCursor);

	CDialog::OnOK();
}

void CDerivativeDialog::OnSelectChange() 
{
	PlotListBox::OnSelectChange();
}

void CDerivativeDialog::OnRadioCenter() 
{
	deriv_loc = IDC_RADIO_CENTER;
}

void CDerivativeDialog::OnRadioLeft() 
{
	deriv_loc = IDC_RADIO_LEFT;
}

void CDerivativeDialog::OnRadioRight() 
{
	deriv_loc = IDC_RADIO_RIGHT;
}


// Compute a three point discrete derivative. Note that left, centered or right
// handed derivatives are just permutations of the indicies.
// centered == DiscreteDeriv3(xy, 0, 1, 2);
// left == DiscreteDeriv3(xy, 1, 2, 0);
// right == DiscreteDeriv3(xy, 2, 0, 1);

float DiscreteDeriv3(DataPoint *xy, int a, int b, int c)
{
	return xy[a].y*(xy[b].x - xy[c].x)/((xy[a].x - xy[b].x)*
			(xy[a].x - xy[c].x)) + xy[b].y*(1.0f/(xy[b].x - xy[a].x) + 
			1.0f/(xy[b].x - xy[c].x)) + xy[c].y*(xy[b].x - xy[a].x)/(
			(xy[c].x - xy[a].x)*(xy[c].x - xy[b].x));
}

// Compute the discrete derivative of a plot (and return the result as a
// new plot).
SPlot &CDerivativeDialog::Derivative(SPlot &deriv, SPlot &plot, int id)
{
	stackIterator<DataPoint> next(plot.data);
	DataPoint *xy;
	DataPoint value;

	// Assume derivative data is all positive until we read a number <= 0.0
	deriv.canXLog = TRUE;
	deriv.canYLog = TRUE;

	switch(deriv_num_point){
		case THREE_PT_DERIV:
			xy = new DataPoint[3];
			
			// Fill the xy array
			next(xy[0]);
			next(xy[1]);

			while(next(xy[2])){
				// Is the derivative defined?
				if((xy[0].x != xy[1].x) && (xy[0].x != xy[2].x) && (xy[1].x != xy[2].x)){
					switch(deriv_loc){
						case IDC_RADIO_CENTER:
							value.x = xy[1].x;
							value.y = DiscreteDeriv3(xy, 0, 1, 2);
							break;
						case IDC_RADIO_LEFT:
							value.x = xy[0].x;
							value.y = DiscreteDeriv3(xy, 1, 2, 0);
							break;
						case IDC_RADIO_RIGHT:
							value.x = xy[2].x;
							value.y = DiscreteDeriv3(xy, 2, 0, 1);
							break;
					};

					if(value.x <= 0.0)
						deriv.canXLog = FALSE;
				
					if(value.y <= 0.0)
						deriv.canYLog = FALSE;

					deriv.data.Push(value);
				}

				// Move the "window" ahead on point
				xy[0] = xy[1];
				xy[1] = xy[2];
			}
			
			break;
	};

	delete [] xy;
	
	////////////////////////////////////
	// Build the deriv plot header info
	////////////////////////////////////
	// Give the plot a name
	deriv.info.Format("Derivative of Set %d", id);

	// Set the plot style
	deriv.pointType = SPlot::POINT_NONE;

	deriv.lineColor = IndexToColor((ColorToIndex(plot.lineColor) + 1) % 
						S_CLR_NONE);

	// Exclude white from the list of colors!
	if(ColorToIndex(deriv.lineColor) == S_CLR_WHITE)
		deriv.lineColor = IndexToColor(S_CLR_WHITE + 1);
	
	deriv.lineType = (plot.lineType + 1)%SPlot::LINE_NONE;

	return deriv;
}
