//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#ifndef EXCEPTS_H__
#define EXCEPTS_H__

#include <stdio.h>

//	Base class for ALL exceptions  
//	along with some derived classes 
//
// 1/24/00 Excepts.Cpp has been merged into this file (now there is
// only Excepts.h).
//
//	2/13/00	Added the #include <stdio.h> to prevent compiler error

class Error     { 
private: 
	const char *msg; 
	char *filename; 
	int linenum;
public: 
	Error()
	{
		msg = NULL; 
		filename = NULL; 
		linenum = 0; 
	};
	
	Error(const char *m,char *fn = NULL,int ln = 0):msg(m),filename(fn),linenum(ln)
	{
		// Nothing to do here.
	};
	
	~Error()
	{
		linenum = 0;
	};

	// Write the error to the debugging console and return the error message 
	const char *message()
	{
		if ( msg != NULL ){
#ifdef _WINDOWS
			TRACE0(msg);
#endif // _WINDOWS
			return msg; 
		}
		else {
#ifdef _WINDOWS
			TRACE0("Excepts error thrown\n");
#endif // _WINDOWS
			return "Excepts error thrown"; 
		}
	};
	
	// Return full report with filename and line number
	char *report()
	{
		// A buffer for the error message. It is dangerous to have a
		// fixed length buffer for storing the error message!
		static char str[1024];
        
		if ( linenum == 0 ) {      
			if ( filename == NULL ){
#ifdef _WINDOWS
				TRACE0("Excepts error thrown\n");
#endif // _WINDOWS
				return "Excepts error thrown"; 
			}
			else
				sprintf(str,"%s in %s\n",msg,filename); 
		} 
		else {  
			if ( filename == NULL ){
#ifdef _WINDOWS
				TRACE0("Excepts error thrown\n");
#endif //  _WINDOWS
				return "Excepts error thrown"; 
			}
			else 
				sprintf(str,"%s in %s at line %d\n", msg, filename, linenum);
		}

#ifdef _WINDOWS
		TRACE0(str);
#endif // _WINDOWS

		return str;
	};
};


class IO_Error : public Error   // Input/Output Error
{	
public:
	IO_Error();
	IO_Error(char *m,char *fn = NULL,int ln = 0)
		: Error(m,fn,ln){  } ; 

 }; 

class MP_Error  : public Error  // Message Passing Error 
{ 
public:
	MP_Error();
	MP_Error(char *m,char *fn = NULL,int ln = 0)
		: Error(m,fn,ln){  } ; 
}; 

class Mem_Error : public Error  // Memory Error 
{
public:
	Mem_Error();
	Mem_Error(char *m,char *fn = NULL,int ln = 0)
		: Error(m,fn,ln){  } ; 
};


class Bounds_Error : public Error {
public:
	Bounds_Error();
	Bounds_Error(char *m,char *fn = NULL,int ln = 0)
		: Error(m,fn,ln){  } ; 
}; 

#endif 
