//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "IntegrateDialog.h"
#include "sgraphDoc.h"
#include "TextOutputDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIntegrateDialog dialog

// Local function definition
CString Integrate(SPlot &plot, float lower, float upper);
CString Integrate(SPlot &plot);
int DataPtrCmp(const void *elem1, const void *elem2 );

CIntegrateDialog::CIntegrateDialog(CDocument* pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CIntegrateDialog::IDD, pParent)
{
	// By default, use automatic integration limits
	limits = IDC_AUTO_RADIO;
	m_pDoc = pDoc;
	
	//{{AFX_DATA_INIT(CIntegrateDialog)
	m_lower = 0.0f;
	m_upper = 0.0f;
	//}}AFX_DATA_INIT
}


void CIntegrateDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIntegrateDialog)
	DDX_Control(pDX, IDC_INTEGRATE_LIST, m_list);
	DDX_Text(pDX, IDC_LOWER_EDIT, m_lower);
	DDX_Text(pDX, IDC_UPPER_EDIT, m_upper);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CIntegrateDialog, CDialog)
	//{{AFX_MSG_MAP(CIntegrateDialog)
	ON_BN_CLICKED(IDC_AUTO_RADIO, OnAutoRadio)
	ON_BN_CLICKED(IDC_USER_RADIO, OnUserRadio)
	ON_LBN_SELCHANGE(IDC_INTEGRATE_LIST, OnSelchangeIntegrateList)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIntegrateDialog message handlers

// CDerivativeDialog message handlers
BOOL CIntegrateDialog::OnInitDialog()
{	
	CDialog::OnInitDialog();

	// Initialize the list box
	PlotListBox::OnInitDialog(m_list, m_pDoc);

	// Set the derivative location (i.e. centered, left or right)
	// radio buttons
	switch(limits){
		case IDC_AUTO_RADIO:
			CheckRadioButton(IDC_AUTO_RADIO,		// First button
							 IDC_USER_RADIO,		// Last bottom	
							 IDC_AUTO_RADIO);		// Button to check

			// Deactivate the limit edit boxes
			GetDlgItem(IDC_LOWER_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_UPPER_EDIT)->EnableWindow(FALSE);

			break;
		case IDC_USER_RADIO:
			CheckRadioButton(IDC_AUTO_RADIO,		// First button
							 IDC_USER_RADIO,		// Last bottom	
							 IDC_USER_RADIO);		// Button to check
			break;
	};

	UpdateData(FALSE);

	return TRUE;
}

void CIntegrateDialog::OnAutoRadio() 
{
	limits = IDC_AUTO_RADIO;

	// Deactivate the limit edit boxes
	GetDlgItem(IDC_LOWER_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_UPPER_EDIT)->EnableWindow(FALSE);
	
}

void CIntegrateDialog::OnUserRadio() 
{
	limits = IDC_USER_RADIO;

	// Deactivate the limit edit boxes
	GetDlgItem(IDC_LOWER_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_UPPER_EDIT)->EnableWindow(TRUE);
	
}

void CIntegrateDialog::OnSelchangeIntegrateList() 
{
	PlotListBox::OnSelectChange();
}

void CIntegrateDialog::OnOK() 
{
	int i, plot_count;

	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);
	
	// Read in all of the variables from the dialog box
	if(!UpdateData())
		return;

	// Has the user selected a plot(s)?
	for(i = 0, plot_count = 0;i < m_ptr->plot.Length();i++)
		if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected)
			plot_count ++;

	if(plot_count == 0){
		AfxMessageBox("No Plots have been selected!");
		return;
	}
	
	CString buffer;

	if(limits == IDC_AUTO_RADIO){
		for(i = 0;i < m_ptr->plot.Length();i++)
			if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected)
				buffer = buffer + Integrate(m_ptr->plot[i]);

	}
	else{ // limits == IDC_USER_RADIO
		for(i = 0;i < m_ptr->plot.Length();i++)
			if(m_ptr->plot[i].isDrawn && m_ptr->plot[i].isSelected)
				buffer = buffer + Integrate(m_ptr->plot[i],
					m_lower, m_upper);
	}

	CTextOutputDialog toDlg("Integration results", buffer);

	toDlg.DoModal();

	// The ok button has been changed to an "Apply" button.
	// Don't get rid of the dialog until the user selects cancel.
	// CDialog::OnOK();
}

CString Integrate(SPlot &plot, float lower, float upper)
{
	CString buffer;
	CString tmp;

	tmp.Format("Integration limits [%g, %g]\r\n", lower, upper);

	buffer = plot.info + 
		"\r\n" + 
		tmp;
	
	if(plot.data.Length() == 0){
		buffer = buffer + "No points in plot!\r\n\r\n";
		return buffer;
	}

	if(plot.data.Length() == 1){
		buffer = buffer + "1 point in plot, integral is zero.\r\n\r\n";
		return buffer;
	}

	DataPoint **ptr_array;
	int len = plot.data.Length();

	ptr_array = new DataPoint* [len];

	stackIterator<DataPoint> next(plot.data);
	DataPoint xy;
	int i;
	double sum = 0.0;

	for(i = 0;i < len;i++){
		ptr_array[i] = &(next.current()->value);
		next(xy);
	}	

	// Sort the array of pointers
	qsort(ptr_array, len, sizeof(DataPoint *), DataPtrCmp);

	float flipLim;

	if(lower > upper){
		flipLim = lower;
		lower = upper;
		upper = flipLim;

		flipLim = -1.0f;
	}
	else
		flipLim = 1.0f;

	for(i = 0;i < len - 1;i++){
		if((ptr_array[i]->x >= lower) && (ptr_array[i+1]->x <= upper))
			sum += 0.5*(ptr_array[i+1]->x - ptr_array[i]->x)*
				(ptr_array[i+1]->y + ptr_array[i]->y);
	}	
		
	sum *= flipLim;

	delete [] ptr_array;

	tmp.Format("%g", sum);

	buffer = buffer + "Integral = " + tmp + "\r\n\r\n";

	return buffer;
}

CString Integrate(SPlot &plot)
{
	CString buffer;
	CString tmp;

	buffer = plot.info + 
		"\r\n";
	
	if(plot.data.Length() == 0){
		buffer = buffer + "No points in plot!\r\n\r\n";
		return buffer;
	}

	if(plot.data.Length() == 1){
		buffer = buffer + "1 point in plot, integral is zero.\r\n\r\n";
		return buffer;
	}

	DataPoint **ptr_array;
	int len = plot.data.Length();

	ptr_array = new DataPoint* [len];

	stackIterator<DataPoint> next(plot.data);
	DataPoint xy;
	int i;
	double sum = 0.0;

	for(i = 0;i < len;i++){
		ptr_array[i] = &(next.current()->value);
		next(xy);
	}	

	// Sort the array of pointers
	qsort(ptr_array, len, sizeof(DataPoint *), DataPtrCmp);

	tmp.Format("Integration limits [%g, %g]\r\n", 
		ptr_array[0]->x, 
		ptr_array[len - 1]->x);

	buffer = buffer + tmp;

	for(i = 0;i < len - 1;i++)
		sum += 0.5*(ptr_array[i+1]->x - ptr_array[i]->x)*
			(ptr_array[i+1]->y + ptr_array[i]->y);

	delete [] ptr_array;

	tmp.Format("%g", sum);

	buffer = buffer + "Integral = " + tmp + "\r\n\r\n";

	return buffer;
}

int DataPtrCmp(const void *elem1, const void *elem2 )
{
	if((*((DataPoint**)(elem1)))->x > (*((DataPoint**)(elem2)))->x)
		return 1;

	if((*((DataPoint**)(elem1)))->x < (*((DataPoint**)(elem2)))->x)
		return -1;

	// elem1 == elem2
	return 0;
}