//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "MainFrm.h"
#include "sgraphdoc.h"
#include "sgraphview.h"
#include "PrestoLink/PrestoLink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SIZING()
	ON_WM_GETMINMAXINFO()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_STOP_PIPE, RefreshTheView)
	ON_MESSAGE(WM_PIPE_STATUS, PipeStatus)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,	// <--- Display general info here
	ID_SEPARATOR,	// <--- Display the (x,y) coordiantes here
	ID_SEPARATOR	// <--- Display Graph ID here
	//ID_INDICATOR_CAPS,
	//ID_INDICATOR_NUM,
	//ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	isWYSIWYG = FALSE;
	aspectWYSIWYG = 1.0f;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// X,Y coordiante info
	m_wndStatusBar.SetPaneInfo(XY_PANE, m_wndStatusBar.GetItemID(XY_PANE),
		SBPS_NORMAL, 200);

	m_wndStatusBar.SetPaneInfo(GRAPH_PANE, m_wndStatusBar.GetItemID(XY_PANE),
		SBPS_NORMAL, 25);

	// Rendering and pipe reading info
	m_wndStatusBar.SetPaneInfo(INFO_PANE, m_wndStatusBar.GetItemID(INFO_PANE),
		SBPS_STRETCH, 0);

	// Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

LRESULT CMainFrame::RefreshTheView(WPARAM wParam, LPARAM lParam)
{
	// Added this message handle to provide a thread safe way of
	// redrawing the screen!
	((CSgraphView *)GetActiveView())->StopPipe();

	// Erase the status bar
	m_wndStatusBar.SetPaneText(INFO_PANE, "Ready");

	return 0L;
}

// Update the status bar
LRESULT CMainFrame::PipeStatus(WPARAM wParam, LPARAM lParam)
{
	CString buffer;

	switch(wParam){
		case PP_OPEN_PIPE:
			m_wndStatusBar.SetPaneText(INFO_PANE, "Waiting for data ...");
			break;
		case PP_CLOSE_PIPE:
			m_wndStatusBar.SetPaneText(INFO_PANE, "Closing pipe");
			break;
		case PP_NEW_PLOT:
			buffer.Format("Reading plot %d", lParam);
			m_wndStatusBar.SetPaneText(INFO_PANE, buffer);
			break;
		//case PP_READ_POINT:
		//	buffer.Format("Reading point %d", lParam);
		//	m_wndStatusBar.SetPaneText(INFO_PANE, buffer);
		//	break;
	};

	return 0L;
}

void CMainFrame::EnableWYSIWYG(float m_aspectWYSIWYG)
{
	// Set WYSIWYG flag
	isWYSIWYG = TRUE;

	// Set the aspect ratio
	aspectWYSIWYG = m_aspectWYSIWYG;

	RECT tmpRect;
	RECT tmpClientRect;

	// Get the current window dimentions
	GetWindowRect(&tmpRect);

	// Get the current client drawing area dimentions
	GetClientRect(&tmpClientRect);

	// Adjust the window so that the client area has the
	// desired aspect ratio.
	int diff_x = tmpRect.right - tmpRect.left - 
				(tmpClientRect.right - tmpClientRect.left);

	int diff_y = tmpRect.bottom - tmpRect.top - 
				(tmpClientRect.bottom - tmpClientRect.top);

	// Use the dimensions of the client area to compute the aspect ratio
	int dx = tmpClientRect.right - tmpClientRect.left;
	int dy = (int)(aspectWYSIWYG*dx);

	// Compute the new window dimensions
	tmpRect.right = tmpRect.left + dx + diff_x;
	tmpRect.bottom = tmpRect.top + dy + diff_y;

	MoveWindow(&tmpRect);
}

void CMainFrame::DisableWYSIWYG()
{
	// Unset WYSIWYG flag
	isWYSIWYG = FALSE;

	// Set the aspect ratio to a default value
	aspectWYSIWYG = 1.0f;	
}

void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
{
	// If we're not in WYSIWYG mode, we don't need to adjust the max allowed
	// window size
	if(!isWYSIWYG)
		return;

	RECT tmpRect;
	RECT tmpClientRect;

	// Get the current window dimentions
	GetWindowRect(&tmpRect);

	// Get the current client drawing area dimentions
	GetClientRect(&tmpClientRect);

	// Adjust the window so that the client area has the
	// desired aspect ratio.
	int diff_x = tmpRect.right - tmpRect.left - 
				(tmpClientRect.right - tmpClientRect.left);

	int diff_y = tmpRect.bottom - tmpRect.top - 
				(tmpClientRect.bottom - tmpClientRect.top);
	
	int max_x = lpMMI->ptMaxSize.x - diff_x;
	int max_y = lpMMI->ptMaxSize.y - diff_y;

	// Aspect ratio = dy/dx
	float winAspectRatio = (float)(max_y)/max_x;

	if(winAspectRatio >= aspectWYSIWYG){
		// Height limited
		lpMMI->ptMaxSize.y = (int)(max_x*aspectWYSIWYG) + diff_y;
		lpMMI->ptMaxTrackSize.y = (int)((lpMMI->ptMaxTrackSize.x - diff_x)*
			aspectWYSIWYG) + diff_y;
	}
	else{
		// Width limited
		lpMMI->ptMaxSize.x = (int)(max_y/aspectWYSIWYG);
		lpMMI->ptMaxTrackSize.x = (int)((lpMMI->ptMaxTrackSize.y - diff_y)
			/aspectWYSIWYG + diff_x);
	}
}

void CMainFrame::OnSizing(UINT fwSide, LPRECT pRect) 
{
	// Do we need to maintain a fixed aspect ratio?
	if(isWYSIWYG){
		int dx;
		int dy;

		RECT tmpRect;
		RECT tmpClientRect;

		// Get the previous window dimentions
		GetWindowRect(&tmpRect);

		// Get the previous client drawing area dimentions
		GetClientRect(&tmpClientRect);

		// Adjust the window so that the client area has the
		// desired aspect ratio.
		int diff_x = tmpRect.right - tmpRect.left - 
					(tmpClientRect.right - tmpClientRect.left);

		int diff_y = tmpRect.bottom - tmpRect.top - 
					(tmpClientRect.bottom - tmpClientRect.top);

		// Which edge is the user moving?
		switch(fwSide){
			case 1: // Left
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->bottom = pRect->top + dy;
				break;
			case 2: // Right
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->bottom = pRect->top + dy;
				break;
			case 4: // Top left
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->top = pRect->bottom - dy;
				break;
			case 5: // Top right
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->top = pRect->bottom - dy;
				break;
			case 7: // Bottom left
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->bottom = pRect->top + dy;
				break;
			case 8: // Bottom right
				dx = pRect->right - pRect->left - diff_x;
				dy = (int)(aspectWYSIWYG*dx) + diff_y;

				pRect->bottom = pRect->top + dy;
				break;
			case 3: // Top
				dy = pRect->bottom - pRect->top - diff_y;
				dx = (int)(dy/aspectWYSIWYG) + diff_x;

				pRect->right = pRect->left + dx;
				break;
			case 6: // Bottom
				dy = pRect->bottom - pRect->top - diff_y;
				dx = (int)(dy/aspectWYSIWYG) + diff_x;

				pRect->right = pRect->left + dx;
				break;
		};
	}

	CFrameWnd::OnSizing(fwSide, pRect);
}
