//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#ifndef __MATRIX_INVERSE
#define  __MATRIX_INVERSE

#include "Excepts.h"
#include "tnt\lu.h"

namespace TNT
{

// Compute the inverse of a square, symmetric matrix by LU decomposition
// (as outlined in Numerical Recipies, Press et. al.). The TNT library 
// http://math.nist.gov/tnt is used to perform all of the matrix 
// manipulations.
template <class MaTRiX>
void InvertMatrix( MaTRiX &inv_a)
{
	int dim = inv_a.num_rows();

	if(dim != inv_a.num_cols()){
		throw Error("InvertMatrix: Only square matricies are allowed!");
	}

	Vector<Subscript> index(dim);

	// Use a vector that has the same type as the input matrix
	Vector<MaTRiX::element_type> tmp(dim);

	// a == input matrix
	MaTRiX a(inv_a);

	if(LU_factor(a, index)){
		throw Error("InvertMatrix: Failed to factor matrix");
	}

	for(int i = 0;i < dim;i++){
		tmp = 0.0f;
		tmp[i] = 1.0f;
		LU_solve(a, index, tmp);

		// Use the C style zero-based array indexing
		// (i.e. inv_a[j][i] as opposed to inv_a(j, i)
		for(int j = 0;j < dim;j++)
			inv_a[j][i] = tmp[j];
	}
}

}
#endif // __MATRIX_INVERSE