//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "ParametricFunctionDialog.h"
#include "sgraphDoc.h"
#include "Equation/Equation.h"
#include "PrestoUtil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// The independant variable
#define		INDEPENDANT_VAR		"t"

/////////////////////////////////////////////////////////////////////////////
// CParametricFunctionDialog dialog


CParametricFunctionDialog::CParametricFunctionDialog(CDocument* pDoc, 
		list<ParametricEquationInfo> &m_paraEqHistory)
	: CDialog(CParametricFunctionDialog::IDD, NULL)
{
	m_pDoc = pDoc;

	ptrParaEqHistory = &m_paraEqHistory;

	//{{AFX_DATA_INIT(CParametricFunctionDialog)
	m_num_points = 1;
	m_t_max = _T("");
	m_t_min = _T("");
	m_x_of_t = _T("");
	m_y_of_t = _T("");
	//}}AFX_DATA_INIT
}


void CParametricFunctionDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CParametricFunctionDialog)
	DDX_Control(pDX, IDC_EQUATION_TAB, m_eq_tab);
	DDX_Control(pDX, IDC_EVAL_PROGRESS, m_status);
	DDX_Text(pDX, IDC_NUM_POINTS, m_num_points);
	DDV_MinMaxUInt(pDX, m_num_points, 1, 4294967295);
	DDX_Text(pDX, IDC_T_MAX, m_t_max);
	DDX_Text(pDX, IDC_T_MIN, m_t_min);
	DDX_Text(pDX, IDC_X_EDIT, m_x_of_t);
	DDX_Text(pDX, IDC_Y_EDIT, m_y_of_t);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CParametricFunctionDialog, CDialog)
	//{{AFX_MSG_MAP(CParametricFunctionDialog)
	ON_NOTIFY(TCN_SELCHANGE, IDC_EQUATION_TAB, OnSelchangeEquationTab)
	ON_NOTIFY(TCN_SELCHANGING, IDC_EQUATION_TAB, OnSelchangingEquationTab)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParametricFunctionDialog message handlers
BOOL CParametricFunctionDialog::OnInitDialog()
{	
	CDialog::OnInitDialog();

	// Set the default number of points
	m_num_points = 100;

	// The default range of t values.
	float minT = 0.0f;	// Default t min value
	float maxT = 1.0f;	// Default t max value

	m_t_max.Format("%g", maxT);
	m_t_min.Format("%g", minT);
	
	// Save the default settings in the history list
	SaveEquationHistory();

	// Set the history tabs to reflect the number of equations in 
	// the history list
	for(int i = 0;i < ptrParaEqHistory->Length();i++){
		char tmpString[64];

		// Start numbering equations from 1
		sprintf(tmpString, "Eq %d", i+1);

		m_eq_tab.InsertItem(i, tmpString);
	}

	// Select the default tab
	m_eq_tab.SetCurSel(0);

	// Update all the values we just set
	UpdateData(FALSE);

	// Give the y(t) text edit control the window focus to get the
	// ball rolling.
	GotoDlgCtrl(GetDlgItem(IDC_Y_EDIT));

	return FALSE; // Return false because we are manually setting the focus
}

void CParametricFunctionDialog::OnOK() 
{
	// Cast the document pointer
	SGraph *m_ptr = &(((CSgraphDoc *)m_pDoc)->
		graph[((CSgraphDoc *)m_pDoc)->selected_graph]);

	float minT;
	float maxT;

	CString error_buffer;

	// Read in all of the variables from the dialog box
	UpdateData();

	// Must plot at least a single data point
	if(m_num_points < 1)
		return;


	if(StringToFloat(m_t_max, maxT) == FALSE){
		AfxMessageBox("Invalid max t value!");
		return;
	}

	if(StringToFloat(m_t_min, minT) == FALSE){
		AfxMessageBox("Invalid min t value!");
		return;
	}
	
	float dt;
	
	if(m_num_points == 1){
		dt = maxT - minT;
	}
	else{
		dt = (maxT - minT)/(m_num_points - 1);
	}

	// Set the cursor to the hourglass
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	// Prepare to evaluate the equation
	try{
		// Make a copy of the input strings WITHOUT any \n or \r characters
		char *x_buffer = new char[m_x_of_t.GetLength() + 1];
		char *y_buffer = new char[m_y_of_t.GetLength() + 1];
		int i, j;

		for(i = 0, j = 0;i < m_x_of_t.GetLength();i++){
			char c = m_x_of_t[i];

			if((c != '\n') && (c != '\r'))
				x_buffer[j++] = c;
		}

		// Terminate the string
		x_buffer[j] = '\0';

		for(i = 0, j = 0;i < m_y_of_t.GetLength();i++){
			char c = m_y_of_t[i];

			if((c != '\n') && (c != '\r'))
				y_buffer[j++] = c;
		}

		// Terminate the string
		y_buffer[j] = '\0';

		CString xoft_string(x_buffer);
		CString yoft_string(y_buffer);

		delete [] x_buffer;
		delete [] y_buffer;

		SEquation eq_x((LPCTSTR)(xoft_string));
		SEquation eq_y((LPCTSTR)(yoft_string));

		if(eq_x.GetError())
			throw eq_x.GetError();

		if(eq_y.GetError())
			throw eq_y.GetError();

		if(!eq_x.DefineIndepVar(INDEPENDANT_VAR)){
			// The user has (a) not specifed the correct independant variable
			// or (b) not specified ANY independant variable. Test for the
			// latter before throwing an error
			if(eq_x.NumVar() != 0)
				throw "Please specify a function of "INDEPENDANT_VAR" for x.";
		}

		if(!eq_y.DefineIndepVar(INDEPENDANT_VAR)){
			// The user has (a) not specifed the correct independant variable
			// or (b) not specified ANY independant variable. Test for the
			// latter before throwing an error
			if(eq_y.NumVar() != 0)
				throw "Please specify a function of "INDEPENDANT_VAR" for y.";
		}

		// Make sure that the user has not specified any other
		// variables (only plotting functions of a single variable -- for
		// now).
		if(eq_x.NumVar() != eq_x.NumIndepVar()){
			
			error_buffer.Format(
				"The symbol '%s' is undefined in x("INDEPENDANT_VAR")", 
				eq_x.GetVarName(i));
			
			throw (LPCTSTR)(error_buffer);
		}

		// Simplify the equation for efficient evaluation
		eq_x.Simplify();

		// Did we run into trouble simplifying the equation?
		if(eq_x.GetError())
			throw eq_x.GetError();

		if(eq_y.NumVar() != eq_y.NumIndepVar()){
			
			error_buffer.Format(
				"The symbol '%s' is undefined in y("INDEPENDANT_VAR")", 
				eq_y.GetVarName(i));
			
			throw (LPCTSTR)(error_buffer);
		}

		// Simplify the equation for efficient evaluation
		eq_y.Simplify();

		// Did we run into trouble simplifying the equation?
		if(eq_y.GetError())
			throw eq_y.GetError();

		// Prepare to evaluate the equation
		SPlot newPlot;
		DataPoint xy;
		BOOL first_time = TRUE;
		float minY, minX;

		// Prepare the status bar
		int eval_count = 0;
		int update_every;

		if(m_num_points > 100){
			double tmp = log10(m_num_points);
			update_every = (int)(pow(10.0, tmp - 2.0));
		}
		else{
			update_every = 10;
		}

		m_status.SetRange32(0, m_num_points);
		m_status.SetStep(update_every);

		// To satisfy the revesed plot storage, plot
		// the function from maxT to minT
		//for(float t = minT;t <= maxT;t += dt){
		for(i = 0;i < (int)m_num_points;i++){
			float t = maxT - i*dt;

			xy.x = eq_x(t);
			xy.y = eq_y(t);

			// Did we successfully evaluate the function?
			if(eq_x.GetError() || eq_y.GetError()){
				// Nope. A error was encountered
				eq_x.ClearError();
				eq_y.ClearError();
			}
			else{
				// Yes! Save the function value in the new plot
				newPlot.data.Push(xy);
				
				// Track the smallest value to see if we can plot the
				// equation on a log plot.
				if(first_time){
					first_time = FALSE;

					minY = xy.y;
					minX = xy.x;
				}
				else{
					if(xy.y < minY)
						minY = xy.y;

					if(xy.x < minX)
						minX = xy.x;
				}
			}

			if((++eval_count) % update_every == 0)
				m_status.StepIt();
		}

		// If we never evaluated the function, don't add the plot!
		if(first_time == TRUE)
			throw "Unable to evalute the function within the given range of "INDEPENDANT_VAR;

		// Give the plot a name (use the equation)
		newPlot.info.Format("{%s, %s}", xoft_string, yoft_string);

		// Set the plot style
		newPlot.pointType = SPlot::POINT_NONE;
		newPlot.lineColor = 0x00000000; // Black
		newPlot.lineType = SPlot::LINE_SOLID;
	
		// Set the scale type from the current graph
		newPlot.x_scaleType = m_ptr->x_scaleType;
		newPlot.y_scaleType = m_ptr->y_scaleType;

		if(minX <= 0.0)
			newPlot.canXLog = FALSE;
		else
			newPlot.canXLog = TRUE;

		if(minY <= 0.0)
			newPlot.canYLog = FALSE;
		else
			newPlot.canYLog = TRUE;

		// Add the plot of the function to the current graph
		m_ptr->plot.Push(newPlot);

		// Update the graph scaling
		m_ptr->SetPlotScaleType();
		m_ptr->SetPlotRange();
		m_ptr->SetBoundingBox();

	}
	catch(const char *error){
		// Restore the cursor
		::SetCursor(prevCursor);

		// Invalid equation!
		AfxMessageBox(error);
		return;
	}
	catch(...){
		// Restore the cursor
		::SetCursor(prevCursor);

		// Something really, really bad happened
		AfxMessageBox("Ooof! Unable to evaluate equation!");
		return;
	}

	// Remove the default equation info from the head of the history list
	// (there is no need to store it)
	ptrParaEqHistory->Remove(0);

	// Save this equation information for the equation history list
	SaveEquationHistory();

	// Restore the cursor
	::SetCursor(prevCursor);

	CDialog::OnOK();
}

void CParametricFunctionDialog::SaveEquationHistory()
{
	ParametricEquationInfo tmp;

	tmp.m_x_of_t = m_x_of_t;
	tmp.m_y_of_t = m_y_of_t;
	tmp.m_num_points = m_num_points;
	tmp.m_t_max = m_t_max;
	tmp.m_t_min = m_t_min;

	// Make sure the current info doesn't match an existing entry
	for(int i = 0;i < ptrParaEqHistory->Length();i++){
		if(tmp == (*ptrParaEqHistory)[i])
			return;
	}

	ptrParaEqHistory->Insert(tmp, 0);
}

void CParametricFunctionDialog::OnSelchangeEquationTab(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// The user has selected an equation from the history list, update
	// the dialog
	
	m_x_of_t = (*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_x_of_t;
	m_y_of_t = (*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_y_of_t;
	m_num_points = (*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_num_points;
	m_t_max = (*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_t_max;
	m_t_min = (*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_t_min;

	// Update all the values we just set
	UpdateData(FALSE);
	
	*pResult = 0;
}

void CParametricFunctionDialog::OnSelchangingEquationTab(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Read the values set by the user
	UpdateData(TRUE);

	// The user has selected an equation from the history list, update
	// the dialog before switching to the new equation. This keeps the history
	// list current and allows the user to switch back and forth without losing
	// any values.
	(*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_x_of_t = m_x_of_t;
	(*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_y_of_t = m_y_of_t;
	(*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_num_points = m_num_points;
	(*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_t_max = m_t_max;
	(*ptrParaEqHistory)[m_eq_tab.GetCurSel()].m_t_min = m_t_min;

	*pResult = 0;
}
