//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "PrestoColor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPrestoColor dialog


CPrestoColor::CPrestoColor(COLORREF m_color)
	: CDialog(CPrestoColor::IDD, NULL)
{
	//{{AFX_DATA_INIT(CPrestoColor)
	m_red = 0;
	m_green = 0;
	m_blue = 0;
	m_no_color = FALSE;
	//}}AFX_DATA_INIT

	if(m_color == NO_COLOR){
		m_no_color = TRUE;

		// Set the color fields to a plain white background
		m_red = 255;
		m_green = 255;
		m_blue = 255;
	}
	else{
		m_no_color = FALSE;

		m_red = m_color & 0x000000ff;
		m_green = (m_color >> 8) & 0x000000ff;
		m_blue = (m_color >> 16) & 0x000000ff;
	}
}


void CPrestoColor::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrestoColor)
	DDX_Control(pDX, IDC_B_SLIDER, m_blue_slider);
	DDX_Control(pDX, IDC_G_SLIDER, m_green_slider);
	DDX_Control(pDX, IDC_R_SLIDER, m_red_slider);
	DDX_Text(pDX, IDC_R_EDIT, m_red);
	DDV_MinMaxUInt(pDX, m_red, 0, 255);
	DDX_Text(pDX, IDC_G_EDIT, m_green);
	DDV_MinMaxUInt(pDX, m_green, 0, 255);
	DDX_Text(pDX, IDC_B_EDIT, m_blue);
	DDV_MinMaxUInt(pDX, m_blue, 0, 255);
	DDX_Check(pDX, IDC_TRANSPARENT_CHECK, m_no_color);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPrestoColor, CDialog)
	//{{AFX_MSG_MAP(CPrestoColor)
	ON_WM_PAINT()
	ON_EN_CHANGE(IDC_R_EDIT, OnChangeREdit)
	ON_EN_CHANGE(IDC_G_EDIT, OnChangeGEdit)
	ON_EN_CHANGE(IDC_B_EDIT, OnChangeBEdit)
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_TRANSPARENT_CHECK, OnTransparentCheck)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPrestoColor message handlers
BOOL CPrestoColor::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	m_red_slider.SetRange(0, 255);
	m_green_slider.SetRange(0, 255);
	m_blue_slider.SetRange(0, 255);

	m_red_slider.SetPos(255 - m_red);
	m_green_slider.SetPos(255 - m_green);
	m_blue_slider.SetPos(255 - m_blue);

	// Are the color controls active or is a transparent option 
	// selected?
	if(m_no_color){
		GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);

		GetDlgItem(IDC_R_SLIDER)->EnableWindow(FALSE);
		GetDlgItem(IDC_G_SLIDER)->EnableWindow(FALSE);
		GetDlgItem(IDC_B_SLIDER)->EnableWindow(FALSE);
	}

	// Dynamically creat a painted rectangle in the dialog
	// The placeholder is a static rectangle that
	// lets us easily size the painted area.
	GetDlgItem(IDC_COLOR_PATCH)->GetWindowRect(&PaintRect);
	ScreenToClient(&PaintRect);
	
	UpdateData(FALSE);

	UpdateColor();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CPrestoColor::OnChangeREdit() 
{
	UpdateData(TRUE);

	m_red_slider.SetPos(255 - m_red);

	UpdateColor();
}

void CPrestoColor::OnChangeGEdit() 
{
	UpdateData(TRUE);

	m_green_slider.SetPos(255 - m_green);

	UpdateColor();
}

void CPrestoColor::OnChangeBEdit() 
{
	UpdateData(TRUE);

	m_blue_slider.SetPos(255 - m_blue);

	UpdateColor();
}

void CPrestoColor::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	m_red = 255 - m_red_slider.GetPos();
	m_green = 255 - m_green_slider.GetPos();
	m_blue = 255 - m_blue_slider.GetPos();
	
	UpdateData(FALSE);

	UpdateColor();
	
	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CPrestoColor::OnPaint() 
{
	// device context for painting. UpdateColor has
	// been overloaded to use this device context. If we 
	// attempt to use GetDC(), the message cue gets jamed for the
	// entire program!
	CPaintDC dc(this); 
	
	UpdateColor(&dc);
}

void CPrestoColor::UpdateColor(CDC *pDC /* = NULL */)
{
	COLORREF tmp_clr = m_red + (m_green << 8) + (m_blue << 16);

	// Repaint the color strip
	CBrush paintBrush(tmp_clr);

	CBrush *save_brush;

	if(pDC == NULL){
		pDC = GetDC();

		// Paint the rectange the given color
		save_brush = pDC->SelectObject(&paintBrush);
		pDC->Rectangle(PaintRect);

		pDC->SelectObject(save_brush);
		ReleaseDC(pDC);
	}
	else{

		// Paint the rectange the given color
		save_brush = pDC->SelectObject(&paintBrush);
		pDC->Rectangle(PaintRect);

		pDC->SelectObject(save_brush);
	}
}

void CPrestoColor::OnOK() 
{
	UpdateData(TRUE);
	
	CDialog::OnOK();
}

void CPrestoColor::OnTransparentCheck() 
{
	// Toggle the no color state
	m_no_color = !m_no_color;

	// Are the color controls active or is a transparent option 
	// selected?
	if(m_no_color){
		GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);

		GetDlgItem(IDC_R_SLIDER)->EnableWindow(FALSE);
		GetDlgItem(IDC_G_SLIDER)->EnableWindow(FALSE);
		GetDlgItem(IDC_B_SLIDER)->EnableWindow(FALSE);
	}
	else{
		GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
		GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
		GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);

		GetDlgItem(IDC_R_SLIDER)->EnableWindow(TRUE);
		GetDlgItem(IDC_G_SLIDER)->EnableWindow(TRUE);
		GetDlgItem(IDC_B_SLIDER)->EnableWindow(TRUE);
	}
}
