/*	
 *	(c) 2000 J. D. Gans (jdg9@cornell.edu), Shalloway Lab, 
 *	Cornell University. You can modify and freely distribute 
 *	this code, but please keep this header intact.
 */

/*
 *	Change Log
 *
 *	3/22/01		Added a test for finite (i.e. not INF or NAN) numbers
 *				before packing values in the PrestoWritePoint*() functions.
 *	5/9/01		Added a #ifndef __PRESTOLINK to protect this header.
 *	7/27/01		Fixed bug in set symbol color (brush type was not set). 
 *				Added option to set symbol and line sizes.
 *	1/22/02		Added new functionality to the PrestoLink API:
 *				PrestoExit, PrestoSaveandExit, PrestoSetTickSpacing and
 *				PrestoSetScale.
 *	4/15/02		Removed last vestigial fprintf functions.
 *	4/23/02		Added ability to save BMP and PNG images with 
 *				PrestoExportandExit function.
 */

#ifndef __PRESTOLINK
#define  __PRESTOLINK
/****************************************************************************/
/* Internal messages -- don't modify!
/****************************************************************************/
/* Version control */
#define		PRESTO_LINK_VERSION (4) /* Version integer */

/*
 * Include windows.h to get the definitions of things like HANDLE and DWORD
 */
#include <windows.h>

enum {PP_NEW_PLOT = 1, 
	  PP_READ_PLOT, 
	  PP_READ_POINT, 
	  PP_CLOSE_PIPE, 
	  PP_GRAPH_TITLE,
	  PP_GRAPH_TITLE_SIZE,
	  PP_X_AXIS_TITLE,
	  PP_X_LABEL_SIZE,
	  PP_Y_AXIS_TITLE,
	  PP_Y_LABEL_SIZE,
	  PP_SYMBOL_TYPE,
	  PP_SYMBOL_COLOR,
	  PP_SYMBOL_CUSTOM_COLOR,
	  PP_SYMBOL_SIZE,
	  PP_LINE_TYPE,
	  PP_LINE_COLOR,
	  PP_LINE_CUSTOM_COLOR,
	  PP_LINE_SIZE,
	  PP_OPEN_PIPE,
	  PP_PRESTO_VERSION,
	  PP_SCALE_X,
	  PP_TICK_X,
	  PP_SCALE_Y,
	  PP_TICK_Y,
	  PP_EXIT,
	  PP_SAVE_PRESTO_EXIT,
	  PP_EXPORT_BMP_EXIT,
	  PP_EXPORT_PNG_EXIT};
/****************************************************************************/

/* 
 * Plot type that can be passed to PrestoNewPlot().
 * These values can be combined with the binary | operator.
 */
#define		PRESTO_XY				1
#define		PRESTO_ERROR_BAR_X		2
#define		PRESTO_ERROR_BAR_Y		4

/* Options that can be passes to PrestoSetOption() */
enum{PRESTO_SYMBOL_TYPE,
	 PRESTO_SYMBOL_COLOR,
	 PRESTO_LINE_TYPE,
	 PRESTO_LINE_COLOR,
	 PRESTO_SYMBOL_SIZE,
	 PRESTO_LINE_SIZE,
	 PRESTO_GRAPH_TITLE_FONT_SIZE,
	 PRESTO_X_LABEL_FONT_SIZE,
	 PRESTO_Y_LABEL_FONT_SIZE};

/* Valid colors for PrestoPlot objects */
enum {PRESTO_CLR_BLACK = 0, 
	  PRESTO_CLR_WHITE, 
	  PRESTO_CLR_RED, 
	  PRESTO_CLR_GREEN, 
	  PRESTO_CLR_BLUE,
	  PRESTO_CLR_CYAN, 
	  PRESTO_CLR_GREY, 
	  PRESTO_CLR_NONE};

/* Axes flags for seting axis dependant options (i.e. scaling, tick marks, etc.) */
enum {PRESTO_X_AXIS, PRESTO_Y_AXIS};

/* Enumerate the allowed export formats */
enum {PRESTO_EXPORT_BMP, PRESTO_EXPORT_PNG};

/* 
 * Enumerate the symbol types. The order of the inumeration
 * must match the order of appearance in the plot
 * options dialog box.
 */	
enum{PRESTO_POINT_CIRCLE = 0, 
	 PRESTO_POINT_SQUARE, 
	 PRESTO_POINT_TRIANGLE, 
	 PRESTO_POINT_DOT, 
	 PRESTO_POINT_HISTOGRAM_X, 
	 PRESTO_POINT_HISTOGRAM_Y,
	 PRESTO_POINT_VECTOR,
	 PRESTO_POINT_NONE};

/* 
 * Enumerate the line types. The order of the inumeration
 * must match the order of appearance in the plot
 * options dialog box.
 */
enum{PRESTO_LINE_SOLID = 0, 
	 PRESTO_LINE_DOTTED, 
	 PRESTO_LINE_DASHED, 
	 PRESTO_LINE_DASHDOT, 
	 PRESTO_LINE_DASHDOTDOT, 
	 PRESTO_LINE_NONE};

/* 
 * The full path to the PrestoPlot executable. Only define the
 * path here if it has not been previously set.
 */
#ifndef PRESTO_PLOT_PATH
#define	PRESTO_PLOT_PATH	"presto.exe"
#endif

/* 
 * A data structure for storing relevant plot info
 */
typedef struct {
	/* Is this PrestoLink valid? */
	BOOL valid;

	/* Pipe handle for inter-process communication */
	HANDLE pipeHandle[2];

	/* What kind of plot is this (xy, xydx, xydy, xydxdy)? */
	int plotType;

	/* Has the user called PrestoNewPlot? */
	BOOL newPlot;
} PrestoLink;

int PrestoOpen(PrestoLink* lpPrestoLink,
			   const char *title, 
			   const char *x_axis, 
			   const char *y_axis);

int PrestoClose(PrestoLink* lpPrestoLink);

/* 
 * Quit the currently running copy of PrestoPlot.
 */
int PrestoExit(PrestoLink* lpPrestoLink);

/* 
 * Quit the currently running copy of PrestoPlot and
 * save the session to disk.
 */
int PrestoSaveandExit(PrestoLink* lpPrestoLink, const char *filename);

int PrestoExportandExit(PrestoLink* lpPrestoLink, const char *filename, 
						int width, int height, int export_format);

int PrestoNewPlot(PrestoLink* lpPrestoLink, 
				  const char *info, int plotType);

int PrestoSetOption(PrestoLink* lpPrestoLink, 
					int option, int value);
int PrestoSetCustomColor(PrestoLink* lpPrestoLink, 
						 int option, char r, char g, char b);

/* Set the scaling for axis */
int PrestoSetScale(PrestoLink* lpPrestoLink, int axis, float min, float max);

/* Set the major and minor tick mark spacing*/
int PrestoSetTickSpacing(PrestoLink* lpPrestoLink, int axis, 
					     float major, float minor);

/* Write the plot one point at a time */
int PrestoWritePointXY(PrestoLink* lpPrestoLink, 
					   float x, float y);

/* Write the plot one point at a time with error bars */
int PrestoWritePointXYDX(PrestoLink* lpPrestoLink,
						 float x, float y, float eb_x);
int PrestoWritePointXYDY(PrestoLink* lpPrestoLink,
						 float x, float y, float eb_y);
int PrestoWritePointXYDXDY(PrestoLink* lpPrestoLink,
						   float x, float y, float eb_x, float eb_y);

/* 
 * Write a vector (pointing from tail to head). This function is
 * merely a friendly wrapper for two calls to PrestoWritePointXY().
 * To generate a vector plot, the symbol type must be set to PRESTO_POINT_VECTOR.
 */
int PrestoWriteVector(PrestoLink* lpPrestoLink, 
					  float tail_x, float tail_y,
					  float head_x, float head_y);
#endif // __PRESTOLINK