//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "SetControlDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSetControlDialog dialog


CSetControlDialog::CSetControlDialog(int action, list<SPlot> *plot_list,
									 CWnd* pParent /*=NULL*/)
	: CDialog(CSetControlDialog::IDD, pParent)
{
	m_action = action;
	m_ptr_list = plot_list;
	
	// Set the list box font to match the other list boxes.
	NONCLIENTMETRICS tmpSysData;
	
	tmpSysData.cbSize = sizeof(NONCLIENTMETRICS);

	// Grab font (and other data) from the system
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS,
						 sizeof(NONCLIENTMETRICS),
						 &tmpSysData,
						 0);
	
	m_font.CreateFontIndirect(&(tmpSysData.lfMenuFont));
}


void CSetControlDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CSetControlDialog, CDialog)
	//{{AFX_MSG_MAP(CSetControlDialog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSetControlDialog message handlers

BOOL CSetControlDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// Dynamically creat a list (and dynamically size it)
	// The placeholder is a block of static text that
	// lets us easily size the list.
	RECT listBoxRect;
	DWORD listOpt;
	GetDlgItem(IDC_PLACE_HOLDER)->GetWindowRect(&listBoxRect);
	ScreenToClient(&listBoxRect);

	// The style of list box we create depends on the type
	// of plot selection we will be performing.
	switch(m_action){
		case SET_SEND_TO_BACK:
		case SET_BRING_TO_FRONT:
		case SET_SELECT_SINGLE:
			// Single selection enabled
			listOpt = LBS_NOINTEGRALHEIGHT | 
				WS_VSCROLL | WS_HSCROLL | 
				WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER;
			break;
		case SET_ACTIVATE:
		case SET_DEACTIVATE:
		case SET_DELETE:
		case SET_SELECT_MULTIPLE:
			// Multiple selection enabled
			listOpt =  LBS_NOINTEGRALHEIGHT | 
				WS_VSCROLL | WS_HSCROLL | 
				WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER | 
				LBS_MULTIPLESEL | LBS_EXTENDEDSEL;
			break;
		default:
			AfxMessageBox("Illegal m_action in CSetControlDialog");
			
			// Single selection enabled
			listOpt = LBS_NOINTEGRALHEIGHT | 
				WS_VSCROLL | WS_HSCROLL | 
				WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER;
			break;
	};

	m_set_list.CreateEx(WS_EX_CLIENTEDGE, "LISTBOX", "", 
			listOpt, 
			listBoxRect, this, IDC_SET_LIST);

	m_set_list.SetFont(&m_font);

	int i, max_string_length = 0;
	char buffer[1024];

	if(m_action == SET_ACTIVATE){
		SetWindowText("Activate Sets");

		// Display the inactive data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++)
			if((i <= MAX_LIST_BOX_ENTRIES) && !(*m_ptr_list)[i].isDrawn){
				sprintf(buffer,"Set #%d (N = %d) %s", i,
					(*m_ptr_list)[i].data.Length(),
					(*m_ptr_list)[i].info);

				int len = strlen(buffer);
				CDC *pDC = m_set_list.GetDC();

				if(pDC->GetTextExtent(buffer, len).cx > max_string_length)
					max_string_length = pDC->GetTextExtent(buffer, len).cx;
				
				m_set_list.ReleaseDC(pDC);

				m_set_list.AddString(buffer);
			}

		// Explain to users that they've encountered a limitation of the 
		// MFC CListBox class!
		if(i > MAX_LIST_BOX_ENTRIES){
			AfxMessageBox("The maximum number of plots that can be displayed in a\r\n"
				"ListBox has been reached - plot list has been truncated!");
		}

		// Set the width of the list box for proper horizontal
		// scrolling (a kludge to get around horizontal scroll
		// limitations of the CListBox).
		m_set_list.SetHorizontalExtent(max_string_length);

		// Select the first data set by default
		if(m_ptr_list->Length() > 0)
			m_set_list.SetSel(0);

		return TRUE;
	}

	if(m_action == SET_DEACTIVATE){
		SetWindowText("Dectivate Sets");

		// Display the active data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++)
			if((i <= MAX_LIST_BOX_ENTRIES) && (*m_ptr_list)[i].isDrawn){
				sprintf(buffer,"Set #%d (N = %d) %s", i,
					(*m_ptr_list)[i].data.Length(),
					(*m_ptr_list)[i].info);
				int len = strlen(buffer);

				CDC *pDC = m_set_list.GetDC();

				if(pDC->GetTextExtent(buffer, len).cx > max_string_length)
					max_string_length = pDC->GetTextExtent(buffer, len).cx;
				
				m_set_list.ReleaseDC(pDC);

				m_set_list.AddString(buffer);
			}

		// Explain to users that they've encountered a limitation of the 
		// MFC CListBox class!
		if(i > MAX_LIST_BOX_ENTRIES){
			AfxMessageBox("The maximum number of plots that can be displayed in a\r\n"
				"ListBox has been reached - plot list has been truncated!");
		}

		// Set the width of the list box for proper horizontal
		// scrolling (a kludge to get around horizontal scroll
		// limitations of the CListBox).
		m_set_list.SetHorizontalExtent(max_string_length);

		// Select the first data set by default
		if(m_ptr_list->Length() > 0)
			m_set_list.SetSel(0);

		return TRUE;
	}
	
	if(m_action == SET_DELETE){
		SetWindowText("Delete Sets");

		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++){
			if(i <= MAX_LIST_BOX_ENTRIES){
				sprintf(buffer,"Set #%d (N = %d) %s", i,
						(*m_ptr_list)[i].data.Length(),
						(*m_ptr_list)[i].info);
				int len = strlen(buffer);

				CDC *pDC = m_set_list.GetDC();

				if(pDC->GetTextExtent(buffer, len).cx > max_string_length)
					max_string_length = pDC->GetTextExtent(buffer, len).cx;
				
				m_set_list.ReleaseDC(pDC);

				m_set_list.AddString(buffer);
			}
		}

		// Explain to users that they've encountered a limitation of the 
		// MFC CListBox class!
		if(i > MAX_LIST_BOX_ENTRIES){
			AfxMessageBox("The maximum number of plots that can be displayed in a\r\n"
				"ListBox has been reached - plot list has been truncated!");
		}

		// Set the width of the list box for proper horizontal
		// scrolling (a kludge to get around horizontal scroll
		// limitations of the CListBox).
		m_set_list.SetHorizontalExtent(max_string_length);

		// Select the first data set by default
		if(m_ptr_list->Length() > 0)
			m_set_list.SetSel(0);

		return TRUE;
	}
	
	if((m_action == SET_SELECT_SINGLE) || (m_action == SET_SELECT_MULTIPLE) ||
	   (m_action == SET_BRING_TO_FRONT) || (m_action == SET_SEND_TO_BACK)){

		if(m_action == SET_SELECT_MULTIPLE)
			SetWindowText("Select a Set(s)");
		else
			SetWindowText("Select Set");

		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++){
			if(i <= MAX_LIST_BOX_ENTRIES){
				sprintf(buffer,"Set #%d (N = %d)", i,
					(*m_ptr_list)[i].data.Length());

				int len = strlen(buffer);

				CDC *pDC = m_set_list.GetDC();

				if(pDC->GetTextExtent(buffer, len).cx > max_string_length)
					max_string_length = pDC->GetTextExtent(buffer, len).cx;
				
				m_set_list.ReleaseDC(pDC);

				m_set_list.AddString(buffer);
			}
		}

		// Explain to users that they've encountered a limitation of the 
		// MFC CListBox class!
		if(i > MAX_LIST_BOX_ENTRIES){
			AfxMessageBox("The maximum number of plots that can be displayed in a\r\n"
				"ListBox has been reached - plot list has been truncated!");
		}

		// Set the width of the list box for proper horizontal
		// scrolling (a kludge to get around horizontal scroll
		// limitations of the CListBox).
		m_set_list.SetHorizontalExtent(max_string_length);

		// Select the first data set by default
		if(m_ptr_list->Length() > 0)
			m_set_list.SetSel(0);

		return TRUE;
	}

	return TRUE;
}

void CSetControlDialog::OnOK() 
{
	int i, index = 0;

	if(m_action == SET_ACTIVATE){
		for(i = 0;i < m_ptr_list->Length();i++)
			if(!(*m_ptr_list)[i].isDrawn){
				if(m_set_list.GetSel(index)){
					// Activate the set
					(*m_ptr_list)[i].isDrawn = TRUE;

					// Save a list of all the selected states
					m_selected.Push(i);
				}
				index ++;
			}

		CDialog::OnOK();
		return;
	}

	if(m_action == SET_DEACTIVATE){
		// Display the active data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++)
			if((*m_ptr_list)[i].isDrawn){
				if(m_set_list.GetSel(index)){
					// Deactivate the set
					(*m_ptr_list)[i].isDrawn = FALSE;

					// Save a list of all the selected states
					m_selected.Push(i);
				}
				index ++;
			}

		CDialog::OnOK();
		return;
	}
	
	if(m_action == SET_DELETE){
		// First reset the isSelected flag for all of the
		// plots
		for(i = 0;i < m_ptr_list->Length();i++)
			(*m_ptr_list)[i].isSelected = FALSE;

		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++){
			if(m_set_list.GetSel(index)){
				// Mark the set for deletion
				(*m_ptr_list)[i].isSelected = TRUE;

				// Save a list of all the selected states
				m_selected.Push(i);
			}
			index ++;
		}

		for(i = 0;i < m_ptr_list->Length();i++)
			if((*m_ptr_list)[i].isSelected){
				(*m_ptr_list).Remove(i);

				// Step the counter back a single step
				// to avoid skipping a plot!
				i--;
			}
		
		CDialog::OnOK();
		return;
	}
	
	if((m_action == SET_SELECT_SINGLE) || (m_action == SET_SELECT_MULTIPLE)){
		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++)
			if(m_set_list.GetSel(i)){
				// Save a list of all the selected states
				m_selected.Push(i);
			}

		CDialog::OnOK();
		return;
	}

	if(m_action == SET_BRING_TO_FRONT){
		// First reset the isSelected flag for all of the
		// plots
		for(i = 0;i < m_ptr_list->Length();i++)
			(*m_ptr_list)[i].isSelected = FALSE;

		int set_id = -1;

		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++){
			if(m_set_list.GetSel(index)){

				// Save the selected state
				m_ptr_list->BringToFront(index);

				CDialog::OnOK();
				return;
			}
			index ++;
		}
		
		CDialog::OnOK();
		return;
	}

	if(m_action == SET_SEND_TO_BACK){
		// First reset the isSelected flag for all of the
		// plots
		for(i = 0;i < m_ptr_list->Length();i++)
			(*m_ptr_list)[i].isSelected = FALSE;

		int set_id = -1;

		// Display all data sets in the list box
		for(i = 0;i < m_ptr_list->Length();i++){
			if(m_set_list.GetSel(index)){

				// Save the selected state
				m_ptr_list->SendToBack(index);

				CDialog::OnOK();
				return;
			}
			index ++;
		}
		
		CDialog::OnOK();
		return;
	}

	CDialog::OnOK();
}
