//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include "TickMarkDialog.h"
#include "AxisDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTickMarkDialog dialog


CTickMarkDialog::CTickMarkDialog(CSgraphDoc* pDoc, int axis, 
								 CWnd* pParent /*=NULL*/)
	: CDialog(CTickMarkDialog::IDD, pParent)
{
	m_pDoc = pDoc;
	selected_axis = axis;

	//{{AFX_DATA_INIT(CTickMarkDialog)
	//}}AFX_DATA_INIT
}


void CTickMarkDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTickMarkDialog)
	DDX_Control(pDX, IDC_APPLY, m_apply);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTickMarkDialog, CDialog)
	//{{AFX_MSG_MAP(CTickMarkDialog)
	ON_BN_CLICKED(IDC_RADIO_BOTH, OnRadioBoth)
	ON_BN_CLICKED(IDC_RADIO_IN, OnRadioIn)
	ON_BN_CLICKED(IDC_RADIO_NORMAL, OnRadioNormal)
	ON_BN_CLICKED(IDC_RADIO_OPPOSITE, OnRadioOpposite)
	ON_BN_CLICKED(IDC_RADIO_OUT, OnRadioOut)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTickMarkDialog message handlers

BOOL CTickMarkDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	SAxisObject *normal_ptr, *opposite_ptr;
	
	if(selected_axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].bottomAxis);

		opposite_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].topAxis);

		m_tick_location = m_pDoc->graph[m_pDoc->selected_graph].x_tick_location;
	}
	else{
		normal_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].leftAxis);

		opposite_ptr = &(m_pDoc->graph[m_pDoc->selected_graph].rightAxis);

		m_tick_location = m_pDoc->graph[m_pDoc->selected_graph].y_tick_location;
	}

	// Set the tick direction radio buttons from the the current axis
	switch(m_tick_location){
		case SGraph::TICK_MARK_NORMAL:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_NORMAL);		// Button to check
			
			m_tick_direction = normal_ptr->direction;
			break;
		case SGraph::TICK_MARK_BOTH:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_BOTH);		// Button to check
			
			m_tick_direction = normal_ptr->direction;
			break;
		case SGraph::TICK_MARK_OPPOSITE:
			CheckRadioButton(IDC_RADIO_NORMAL,		// First button
							 IDC_RADIO_OPPOSITE,	// Last bottom	
							 IDC_RADIO_OPPOSITE);	// Button to check
			
			m_tick_direction = opposite_ptr->direction;
			break;
	};
	
	if(((m_tick_location == SGraph::TICK_MARK_NORMAL) && 
		((m_tick_direction == SAxisObject::LEFT) || 
	    (m_tick_direction == SAxisObject::DOWN))) ||
		((m_tick_location == SGraph::TICK_MARK_BOTH) && 
		((m_tick_direction == SAxisObject::LEFT) || 
	    (m_tick_direction == SAxisObject::DOWN))) ||
		((m_tick_location == SGraph::TICK_MARK_OPPOSITE) && 
		((m_tick_direction == SAxisObject::RIGHT) || 
	    (m_tick_direction == SAxisObject::UP)))){
		// The tick direction is "out"
		CheckRadioButton(IDC_RADIO_IN,		// First button
						 IDC_RADIO_OUT,		// Last bottom	
						 IDC_RADIO_OUT);	// Button to check
	}
	else{
		// The tick direction is "in"
		CheckRadioButton(IDC_RADIO_IN,		// First button
						 IDC_RADIO_OUT,		// Last bottom	
						 IDC_RADIO_IN);		// Button to check
	}

	// Set the selected axis combo box
	m_apply.SetCurSel(0 /* Current Axis */);

	UpdateData(FALSE /* Initialize dialog */);

	return TRUE;
}

void CTickMarkDialog::OnRadioBoth() 
{
	m_tick_location = SGraph::TICK_MARK_BOTH;

	if(selected_axis == CAxisDialog::XAxis){
		m_tick_direction = SAxisObject::DOWN;
	}
	else{ // selected_axis == CAxisDialog::YAxis
		m_tick_direction = SAxisObject::LEFT;
	}
}

void CTickMarkDialog::OnRadioNormal() 
{
	m_tick_location = SGraph::TICK_MARK_NORMAL;

	if(selected_axis == CAxisDialog::XAxis){
		m_tick_direction = SAxisObject::DOWN;
	}
	else{ // selected_axis == CAxisDialog::YAxis
		m_tick_direction = SAxisObject::LEFT;
	}
}

void CTickMarkDialog::OnRadioOpposite() 
{
	m_tick_location = SGraph::TICK_MARK_OPPOSITE;

	if(selected_axis == CAxisDialog::XAxis){
		m_tick_direction = SAxisObject::UP;
	}
	else{ // selected_axis == CAxisDialog::YAxis
		m_tick_direction = SAxisObject::RIGHT;
	}
}

void CTickMarkDialog::OnRadioIn() 
{
	if(selected_axis == CAxisDialog::XAxis){
		if((m_tick_location == SGraph::TICK_MARK_BOTH) ||
		   (m_tick_location == SGraph::TICK_MARK_NORMAL)){
			m_tick_direction = SAxisObject::UP;
		}
		else{
			m_tick_direction = SAxisObject::DOWN;
		}
	}
	else{
		if((m_tick_location == SGraph::TICK_MARK_BOTH) ||
		   (m_tick_location == SGraph::TICK_MARK_NORMAL)){
			m_tick_direction = SAxisObject::RIGHT;
		}
		else{
			m_tick_direction = SAxisObject::LEFT;
		}
	}
}

void CTickMarkDialog::OnRadioOut() 
{
	if(selected_axis == CAxisDialog::XAxis){
		if((m_tick_location == SGraph::TICK_MARK_BOTH) ||
		   (m_tick_location == SGraph::TICK_MARK_NORMAL)){
			m_tick_direction = SAxisObject::DOWN;
		}
		else{
			m_tick_direction = SAxisObject::UP;
		}
	}
	else{
		if((m_tick_location == SGraph::TICK_MARK_BOTH) ||
		   (m_tick_location == SGraph::TICK_MARK_NORMAL)){
			m_tick_direction = SAxisObject::LEFT;
		}
		else{
			m_tick_direction = SAxisObject::RIGHT;
		}
	}
}

void CTickMarkDialog::UpdateTicks(int graph, int axis)
{
	SAxisObject *normal_ptr, *opposite_ptr;

	if(axis == CAxisDialog::XAxis){
		normal_ptr = &(m_pDoc->graph[graph].bottomAxis);
		opposite_ptr = &(m_pDoc->graph[graph].topAxis);
		m_pDoc->graph[graph].x_tick_location = m_tick_location;

		switch(m_tick_location){
			case SGraph::TICK_MARK_NORMAL:
				normal_ptr->isTickDrawn = TRUE;
				opposite_ptr->isTickDrawn = FALSE;
				normal_ptr->direction = m_tick_direction;
				break;
			case SGraph::TICK_MARK_BOTH:
				normal_ptr->isTickDrawn = TRUE;
				opposite_ptr->isTickDrawn = TRUE;
				normal_ptr->direction = m_tick_direction;
				if(m_tick_direction == SAxisObject::UP)
					opposite_ptr->direction = SAxisObject::DOWN;
				else
					opposite_ptr->direction = SAxisObject::UP;
				break;
			case SGraph::TICK_MARK_OPPOSITE:
				normal_ptr->isTickDrawn = FALSE;
				opposite_ptr->isTickDrawn = TRUE;
				opposite_ptr->direction = m_tick_direction;
				break;
		};
	}
	else{
		normal_ptr = &(m_pDoc->graph[graph].leftAxis);
		opposite_ptr = &(m_pDoc->graph[graph].rightAxis);
		m_pDoc->graph[graph].y_tick_location = m_tick_location;

		switch(m_tick_location){
			case SGraph::TICK_MARK_NORMAL:
				normal_ptr->isTickDrawn = TRUE;
				opposite_ptr->isTickDrawn = FALSE;
				normal_ptr->direction = m_tick_direction;
				break;
			case SGraph::TICK_MARK_BOTH:
				normal_ptr->isTickDrawn = TRUE;
				opposite_ptr->isTickDrawn = TRUE;
				normal_ptr->direction = m_tick_direction;
				if(m_tick_direction == SAxisObject::LEFT)
					opposite_ptr->direction = SAxisObject::RIGHT;
				else
					opposite_ptr->direction = SAxisObject::LEFT;
				break;
			case SGraph::TICK_MARK_OPPOSITE:
				normal_ptr->isTickDrawn = FALSE;
				opposite_ptr->isTickDrawn = TRUE;
				opposite_ptr->direction = m_tick_direction;
				break;
		};
	}
}

void CTickMarkDialog::OnOK() 
{
	// What graphs and axes do we apply the changes to?
	int apply = m_apply.GetCurSel();

	// The four different possibilities for the apply combo box
	if(apply == 0 /* Current Graph */){
		UpdateTicks(m_pDoc->selected_graph, selected_axis);
	}
	
	if(apply == 1 /* All Axes, Current Graph */){
		UpdateTicks(m_pDoc->selected_graph, CAxisDialog::XAxis);
		UpdateTicks(m_pDoc->selected_graph, CAxisDialog::YAxis);
	}
	
	if(apply == 2 /* Current Axis, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++)
			UpdateTicks(i, selected_axis);
	}

	if(apply == 3 /* All Axes, All Graphs */){
		for(int i = 0;i < m_pDoc->num_graph;i++){
			UpdateTicks(i, CAxisDialog::XAxis);
			UpdateTicks(i, CAxisDialog::YAxis);
		}		
	}

	CDialog::OnOK();
}
