//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "EquationUtil.h"
#include <math.h>

EqElemVal *GetNextConstVal(EqElem *ptr, int &sop, int type)
{
	if(ptr == NULL)
		throw "NULL pointer in GetNextConstVal";

	if(ptr->GetType() == VALUE_CONST)
		return (EqElemVal *)ptr;

	if((type == BINARY_ADD) || (type == BINARY_SUB)){
		if(ptr->GetType() == BINARY_ADD){
			int tmp_sop = sop;

			EqElem *tmp_ptr = GetNextConstVal(BINARY(ptr)->L, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return (EqElemVal *)tmp_ptr;
			}

			tmp_sop = sop;

			tmp_ptr = GetNextConstVal(BINARY(ptr)->R, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			return NULL;
		}
		
		if(ptr->GetType() == BINARY_SUB){
			int tmp_sop = sop;
			EqElem *tmp_ptr = GetNextConstVal(BINARY(ptr)->L, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			tmp_sop = -sop;

			tmp_ptr = GetNextConstVal(BINARY(ptr)->R, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			return NULL;
		}
	}

	if((type == BINARY_DIV) || (type == BINARY_MULTI)){

		if(ptr->GetType() == BINARY_DIV){
			int tmp_sop = sop;
			EqElem *tmp_ptr = GetNextConstVal(BINARY(ptr)->L, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			tmp_sop = -sop;

			tmp_ptr = GetNextConstVal(BINARY(ptr)->R, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			return NULL;
		}

		if(ptr->GetType() == BINARY_MULTI){
			int tmp_sop = sop;
			EqElem *tmp_ptr = GetNextConstVal(BINARY(ptr)->L, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			tmp_ptr = GetNextConstVal(BINARY(ptr)->R, tmp_sop, type);

			if(tmp_ptr){
				sop = tmp_sop;
				return VALUE(tmp_ptr);
			}

			return NULL;
		}
	}

	return NULL;
}

BOOL isElemZero(EqElem *m_ptr)
{
#ifdef _DEBUG
	if(m_ptr == NULL)
		throw "NULL pointer in isElemZero";
#endif // _DEBUG

	if(isValueOp(m_ptr->GetType()) && VALUE(m_ptr)->isConst){
		// Use a very restrictive definition of "zero".
		return (VALUE(m_ptr)->constval == 0.0);
	}

	return FALSE;
}

BOOL isElemOne(EqElem *m_ptr)
{
#ifdef _DEBUG
	if(m_ptr == NULL)
		throw "NULL pointer in isElemZero";
#endif // _DEBUG

	if(isValueOp(m_ptr->GetType()) && VALUE(m_ptr)->isConst){
		// Use a loose definition of "one".
		if(fabs(VALUE(m_ptr)->constval - 1.0) < EQ_EPSILON){
			return TRUE;
		}
	}

	return FALSE;
}

int MatchUnaryBrace(list<EqToken> &m_tokenlist, int m_i)
{
	int sum = 1;

	for(int i = m_i + 1;i < m_tokenlist.Length();i++){
		if(m_tokenlist[i].token_type == UNARY_L_BRACE)
			sum ++;
		else{
			if(m_tokenlist[i].token_type == UNARY_R_BRACE)
				sum --;
		}

		if(sum == 0)
			return i;
	}

	return -1;
}

// Find the matching curly brace or bracket (i.e. ')', ']', '}'). 
// Return null if a matching brace is not found.
char *MatchUnaryBrace(char l_brace, char *buffer)
{
	int i = 0;
	int count = 1;
	char r_brace;
	char c;
	
	switch (l_brace){
		case '(':
			r_brace = ')';
			break;
		case '[':
			r_brace = ']';
			break;
		case '{':
			r_brace = '}';
			break;
		default:
			return NULL;
	};

	while((c = buffer[i]) != '\0'){
		if(c == l_brace){
			count ++;
		}
		else{
			 if(c == r_brace){
				 count --;
			 }
		}

		// We've matched up left and right braces
		if(count == 0)
			return buffer + i;

		i++;
	}

	return NULL;
}

// Is the given token type a binary operator?
BOOL isBinaryOp(int m_token_type)
{
	if((m_token_type >= BINARY_ADD) && (m_token_type <= BINARY_POW))
		return TRUE;

	return FALSE;
}

BOOL isUnaryOp(int m_token_type)
{
	if((m_token_type >= UNARY_NEG) && (m_token_type <= UNARY_ABS))
		return TRUE;

	return FALSE;
}

BOOL isValueOp(int m_token_type)
{
	if((m_token_type >= VALUE_CONST) && (m_token_type <= VALUE_VAR))
		return TRUE;

	return FALSE;
}

BOOL needsBrace(int m_token_type)
{
	if((m_token_type >= UNARY_SQRT) && (m_token_type <= UNARY_ABS))
		return TRUE;

	return FALSE;
}

int braceSum(list<EqToken> &m_tokenlist)
{
	int sum = 0;

	for(int i = 0;i < m_tokenlist.Length();i++){
		if(m_tokenlist[i].token_type == UNARY_L_BRACE)
			sum ++;
		else{
			if(m_tokenlist[i].token_type == UNARY_R_BRACE)
				sum --;
		}
	}

	return sum;
}

int numValue(list<EqToken> &m_tokenlist, int m_start, int m_stop)
{
	int numval = 0;

	for(int i = m_start;i < m_stop;i++){
		if((m_tokenlist[i].token_type == VALUE_CONST) ||
		   (m_tokenlist[i].token_type == VALUE_VAR))
		   numval ++;
	}

	return numval;
}

#ifdef _DEBUG

// For debugging:
// Check the integrity of a "complete" tree and throw an
// error if the connectivity is incorrect.
void CheckTree(EqElem* ptr)
{
	if(ptr == NULL)
		throw "NULL ptr in CheckNode";

	if(isBinaryOp(ptr->GetType())){
		EqElemBinary *tmp = BINARY(ptr);
			
		if(tmp->L == NULL)
			throw "CheckNode (binary) tmp->L == NULL";

		if(tmp->R == NULL)
			throw "CheckNode (binary) tmp->R == NULL";

		if(tmp->L->P != tmp)
			throw "CheckNode (binary) tmp->L->P != tmp";

		if(tmp->R->P != tmp)
			throw "CheckNode (binary) tmp->R->P != tmp";

		// Do we have a valid operation?
		if((tmp->binary_op < BINARY_ADD) || (tmp->binary_op > BINARY_POW))
			throw "Invalid binary operation";

		CheckTree(tmp->L);
		CheckTree(tmp->R);

		return;
	}

	if(isUnaryOp(ptr->GetType())){
		EqElemUnary *tmp = UNARY(ptr);

		if(tmp->C == NULL)
			throw "CheckNode (unary) tmp->C == NULL";

		if(tmp->C->P != tmp)
			throw "CheckNode (unary) tmp->C->P != tmp";

		// Do we have a valid operation?
		if((tmp->unary_op < UNARY_NEG) || (tmp->unary_op > UNARY_ABS))
			throw "Invalid unary operation";

		CheckTree(tmp->C);
		return;
	}
}

#endif // _DEBUG

EqElem* MakeConst(double m_const, EqElem* m_p)
{
	EqElem* tmp = new EqElemVal;

	tmp->P = m_p;

	VALUE(tmp)->isConst = TRUE;
	VALUE(tmp)->constval = m_const;

	return tmp;
}

// Search (sub)trees for identical nodes
BOOL FindIdentical(EqElem **tmp_L, EqElem **tmp_R, int op)
{
	EqElem *ptr_L, *ptr_R;

	EqElemIterator nextL(*tmp_L, op);

	while(ptr_L = nextL()){
		EqElemIterator nextR(*tmp_R, op);

		while(ptr_R = nextR()){
			if(ptr_L->IsEqual(ptr_R)){
				
				// Don't attempt to simplify constants. If we
				// get to this point ptr_R = ptr_L.
				if(ptr_R->GetType() == VALUE_CONST)
					return FALSE;

				*tmp_L = ptr_L;
				*tmp_R = ptr_R;
				return TRUE;
			}
		}
	}

	return FALSE;
}

// What is the operator that combines tmp_L and tmp_R at the node tmp_R?
int GetOp(EqElem *tmp_L, EqElem *tmp_R, EqElemBinary *tmp_P)
{
#ifdef _DEBUG
	if(tmp_P == NULL)
		throw "NULL tmp_P in GetOp";

	if(tmp_L == NULL)
		throw "NULL tmp_L in GetOp";

	if(tmp_L->P == NULL)
		throw "NULL tmp_L->P in GetOp";

	if(tmp_R == NULL)
		throw "NULL tmp_R in GetOp";

	if(tmp_R->P == NULL)
		throw "NULL tmp_R->P in GetOp";

	if(tmp_L == tmp_R)
		throw "tmp_L == tmp_R in GetOp";
#endif // _DEBUG

	if((tmp_P->GetType() == BINARY_ADD) || (tmp_P->GetType() == BINARY_SUB)){
		int op_L = 1; 
		int op_R = 1;

		// Walk up the tree until we get to tmp_P
		while(tmp_L->P != tmp_P){
			if(tmp_L->P == NULL)
				throw "NULL tmp_L->P in GetOp";

			if(tmp_L->P->GetType() == BINARY_SUB)
				op_L *= -1;

			tmp_L = tmp_L->P;
		}

		// Walk up the tree until we get to tmp_P
		while(tmp_R->P != tmp_P){
			if(tmp_R->P == NULL)
				throw "NULL tmp_R->P in GetOp";

			if(tmp_R->P->GetType() == BINARY_SUB)
				op_R *= -1;

			tmp_R = tmp_R->P;
		}

		// Return the parent operator
		if(op_L * op_R > 0)
			return tmp_P->GetType();

		// Return the conjugate parent operator
		if(tmp_P->GetType() == BINARY_ADD)
			return BINARY_SUB;
		
		return BINARY_ADD;
	}
	
	if((tmp_P->GetType() == BINARY_MULTI) || (tmp_P->GetType() == BINARY_DIV)){
		int op_L = 1; 
		int op_R = 1;

		// Walk up the tree until we get to tmp_P
		while(tmp_L->P != tmp_P){
			if(tmp_L->P == NULL)
				throw "NULL tmp_L->P in GetOp";

			if(tmp_L->P->GetType() == BINARY_DIV)
				op_L *= -1;

			tmp_L = tmp_L->P;
		}

		// Walk up the tree until we get to tmp_P
		while(tmp_R->P != tmp_P){
			if(tmp_R->P == NULL)
				throw "NULL tmp_R->P in GetOp";

			if(tmp_R->P->GetType() == BINARY_DIV)
				op_R *= -1;

			tmp_R = tmp_R->P;
		}

		// Return the parent operator
		if(op_L * op_R > 0)
			return tmp_P->GetType();

		// Return the conjugate parent operator
		if(tmp_P->GetType() == BINARY_MULTI)
			return BINARY_DIV;
		
		return BINARY_MULTI;
	}

	throw "Bad operator in GetOp";
	return - 1;
}
