//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "Equation.h"
#include "EquationUtil.h"

EqElemIterator::EqElemIterator(EqElem* m_root, int m_mask /*= -1*/)
{
#ifdef _DEBUG
	if(m_root == NULL)
		throw "NULL m_root in EqElemIterator::EqElemIterator";
#endif // _DEBUG

	root = cur = m_root;
	
	mask = m_mask;
}

EqElem* EqElemIterator::operator()()
{

	if(cur == NULL)
		return NULL;
	
	// Save the current element to be returned at the end of this function
	EqElem* tmp = cur;
	int stack_len;

	if(isBinaryOp(cur->GetType())){
		// Is this state consistent with the mask?
		if(TestMask()){
			// Save the right branch on the stack
			ptr_stack.Push(BINARY(cur)->R);
			cur = BINARY(cur)->L;
		}
		else{
			// The current element is inconsistent with the mask!
			// Are there any elements on the stack?
			stack_len = ptr_stack.Length();

			if(stack_len != 0){
				cur = ptr_stack[stack_len - 1];
				ptr_stack.Pop();
			}
			else{
				cur = NULL;
			}
		}

		return tmp;
	}

	if(isUnaryOp(cur->GetType())){
		if(TestMask()){
			cur = UNARY(cur)->C;
		}
		else{
			// The current element is inconsistent with the mask!
			// Are there any elements on the stack?
			stack_len = ptr_stack.Length();

			if(stack_len != 0){
				cur = ptr_stack[stack_len - 1];
				ptr_stack.Pop();
			}
			else{
				cur = NULL;
			}
		}

		return tmp;
	}

	// Must be a value
	// Are there any elements on the stack?
	stack_len = ptr_stack.Length();

	if(stack_len != 0){
		cur = ptr_stack[stack_len - 1];
		ptr_stack.Pop();
	}
	else{
		cur = NULL;
	}
	
	return tmp;
}


BOOL EqElemIterator::TestMask()
{

#ifdef _DEBUG
	if(cur == NULL)
		throw "NULL pointer in EqElemIterator::TestMask";
#endif // _DEBUG

	// If mask == -1, TestMask() always returns TRUE.
	if(mask < 0)
		return TRUE;

	// The rules of binary operator precedence
	switch(cur->GetType()){
		case BINARY_ADD:
		case BINARY_SUB:
			if((mask == BINARY_ADD) || (mask == BINARY_SUB))
				return TRUE;
			return FALSE;
		case BINARY_DIV:
		case BINARY_MULTI:
			if((mask == BINARY_DIV) || (mask == BINARY_MULTI))
				return TRUE;
			return FALSE;
		case BINARY_POW:
			if(mask == BINARY_POW)
				return TRUE;
			return FALSE;
	}

	// Unary and value operators always return FALSE.
	return FALSE;
}
