//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#if !defined(AFX_SGRAPH_H__828224E2_F061_11D3_A367_00C04F72D141__INCLUDED_)
#define AFX_SGRAPH_H__828224E2_F061_11D3_A367_00C04F72D141__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif


#include "resource.h"       // main symbols
#include "sobject.h"
#include "slineobject.h"
#include "stextobject.h"
#include "saxisobject.h"
#include "data.h"
#include "splot.h"
#include "slegend.h"
#include "presto.h"

#include "parseOptions.h"
#include "IOBuffer.h"
#include "equation/Equation.h"

/////////////////////////////////////////////////////////////////////////////
// CSgraphApp:
// See sgraph.cpp for the implementation of this class
//

// A forward definition of the iterator class (to allow friend status)
class SPlotIterator;

class SGraph: public SObject {
private:
	// The dimensions of the graph in the real (user defined)
	// units.
	float minX, maxX, minY, maxY;

	// Are the graph dimesions fixed to the user defined values?
	BOOL isStickyScalingX;
	BOOL isStickyScalingY;

	// Default (user determined) graph dimensions
	float defaultMinX, defaultMaxX, defaultMinY, defaultMaxY;

	// An internal flag (not saved to disk) that determines which
	// graph elements get moved by the mouse.
	enum {GRAPH_MOVE_ALL, 
		  GRAPH_MOVE_LEFT_AXIS, 
		  GRAPH_MOVE_RIGHT_AXIS,
		  GRAPH_MOVE_TOP_AXIS,
		  GRAPH_MOVE_BOTTOM_AXIS};
	int graph_move;
public:
	friend class SPlotIterator;

	// Enumerate the possible tick locations
	enum {TICK_MARK_NORMAL = 0, TICK_MARK_BOTH, TICK_MARK_OPPOSITE};

	// Enumerate the possible data import formats. The DX and DY indicate
	// x and y error bars respectively.
	enum {SINGLE_PLOT = 0, MULTI_PLOT, SINGLE_PLOT_DX, 
		SINGLE_PLOT_DY, SINGLE_PLOT_DX_DY, AUDIO_WAVE};

	// A list of plots
	list<SPlot> plot;

	STextObject title;
	
	STextObject leftAxisLable;
	STextObject bottomAxisLable;

	SAxisObject leftAxis, 
				rightAxis,
				topAxis,
				bottomAxis;
	
	SLegend legend;

	list<SLineObject> lines;

	// The plot bounding box
	float x1, y1, // Lower left corner
		  x2, y2; // Upper right corner

	COLORREF background_color;

	// How will the data be scaled to the display (i.e. linear or log)?
	int x_scaleType, y_scaleType;
	
	// Where are the tick marks located for each axis?
	int x_tick_location, y_tick_location;
	
	// Where are the tick labels located for each axis?
	int x_tick_label_location, y_tick_label_location;

	// Should we show multiple plots as an animation
	BOOL animate_plots;
	int curr_animate_plot;

	// An equation that can be overlayed on top of the graph
	BOOL plot_quick_eq;
	SEquation quick_eq;

	void QuickPlot(SEquation &m_quick_eq);
	void DrawQuickPlot(CDC *pDC, CRect &extent);
	int PlotX(float m_x, CRect &extent);
	int PlotY(float m_x, CRect &extent);
	void ClearQuickPlot();

	virtual void Serialize(CArchive& ar);

	SGraph();

	void Draw(CDC* pDC, CRect &extent);
	void DrawSelect(CDC* pDC, CRect &extent);
	void DrawLines(CDC* pDC, CRect &extent);

	CSelectRect SelectRect(CDC* pDC, CRect &extent);
	BOOL Select(BOOL status, CPoint &xy, CDC* pDC, BOOL doubleBuffer,
				CDC* backDC, CRect &extent);

	// What is the nearest plot to the point selected 
	// by the user?
	int PlotHitTest(int m_x, int m_y, float &f_x, float &f_y, CRect &extent);

	void PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent);
	void CleanUpPrinting(CDC* pDC, CRect &extent);
	void Printing(BOOL p);

	// Scan all of the data in the plot(s) and set
	// plotting range to display all data
	void SetPlotRange(BOOL force_sticky = FALSE);
	void SetPlotRangeX(BOOL force_sticky = FALSE);
	void SetPlotRangeY(BOOL force_sticky = FALSE);

	// Set the plot range "by hand".
	void SetPlotRange(float m_minX, float m_maxX, 
					  float m_minY, float m_maxY, 
					  BOOL force_sticky = FALSE);

	void SetPlotRangeX(float m_minX, float m_maxX,
					   BOOL force_sticky = FALSE);
	void SetPlotRangeY(float m_minY, float m_maxY,
					   BOOL force_sticky = FALSE);
	
	// Set the default plot ranges and set the sticky flags
	void SetDefaultPlotRange(float m_minX, float m_maxX, 
							 float m_minY, float m_maxY);
	void SetDefaultPlotRangeX(float m_minX, float m_maxX);
	void SetDefaultPlotRangeY(float m_minY, float m_maxY);

	void SetDefaultPlotRange();
	void SetDefaultPlotRangeX();
	void SetDefaultPlotRangeY();

	void GetPlotRange(float &m_minX, float &m_maxX, 
					  float &m_minY, float &m_maxY);

	// Move the view around in fixed increments
	void ShiftY(float scale);
	void ShiftX(float scale);

	// Set the bounding box for each of the plots
	void SetBoundingBox();

	BOOL SetGraphType(int xAxisType, int yAxisType);

	void SetPlotScaleType();

	// Import data from a text file into plot(s)
	// (i.e. type == SINGLE_PLOT) or multiple plots
	// (i.e. type == MULTI_PLOT). ImportPlot(CString &name, int type) is
	// a wrapper function for ImportPlot(ifstream &fin, CString &name, 
	// int type).
	int ImportPlot(CString &name, int type);
	
	// Import data from a memory handle into plot(s)
	// (i.e. type == SINGLE_PLOT) or multiple plots
	// (i.e. type == MULTI_PLOT). ImportPlot(HANDLE memPtr, int type) is
	// a wrapper function for ImportPlot(void *ioBuf, CString &name, 
	// int type).
	int ImportPlot(HANDLE memPtr, int type);

	int ImportPlot(IOBuffer &io_buff, 
		int &curr_Set, 
		ImportOptionList &cmd,
		CString &name, 
		int type);

	// Export data from plot(s) to an ASCII data set (on disk)
	void ExportPlot();

	// Export data from plot(s) to an ASCII data set (on the clipboard)
	void CopyPlot();

	float ScreentoDataX(int m_x, CRect &extent);
	float ScreentoDataY(int m_y, CRect &extent);

	// Pop up a modal dialog box to allow the user to activate,
	// deactivate and delete plots.
	BOOL ModalActivatePlot();
	BOOL ModalDeactivatePlot();
	BOOL ModalDeletePlot();

	// Change the plot order within a graph
	BOOL ModalPlotToFront();
	BOOL ModalPlotToBack();

	// Pop up a modal dialog box to allow the user to modify the
	// plots contained within this graph
	BOOL ModalPlotOptions();

	void PrepareMove();

	float UpdateX(float m_x);
	float UpdateY(float m_y);

	float MoveX(float m_dx);
	float MoveY(float m_dy);

	// Overlap the current graph on top of m_graph.
	void Overlap(SGraph &m_graph);

	// Audio functions
	int ImportAudioPlot(CString &name, int type);
	BOOL ValidateMonoAudio(int plotId, int samplesSec, float duration, int volume);
	BOOL ValidateStereoAudio(int leftPlotId, int rightPlotId, int samplesSec, float duration, int volume);
	void ExportMonoAudio(CString &name, int plotId, int samplesSec, float duration, int volume);
	void ExportStereoAudio(CString &name, int leftPlotId, int rightPlotId, int samplesSec, float duration, int volume);
	char* BuildAudioBuffer(int plotId, int bufferSize, int samplesSec, float duration, int volume);
};

// The SPlotIterator class provides sequential access plot information
// withing a given graph.
class SPlotIterator: ListIterator<SPlot>{
private:
	int access_mode;
public:
	// How will the plots be accessed? This variable must be one of the
	// following:
	
	enum {ALL, DRAWN, NOT_DRAWN};

	// The constructor requires both a graph and an access mode.
	SPlotIterator(SGraph &m_graph, int m_access);

	// Return the next plot in the list that is consistent with the access
	// code, or NULL if there are no matching plots.
	SPlot* operator()();
	SPlot* operator()(int &m_index);
};

class CSgraphApp : public CWinApp
{
public:
	CSgraphApp();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSgraphApp)
	public:
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

// Implementation
	//{{AFX_MSG(CSgraphApp)
	afx_msg void OnAppAbout();
	afx_msg void OnAppHelpOption();
	afx_msg void OnAppHelpCommand();
	afx_msg void OnFilePrintSetup();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

// Make the global app variable visable to everybody ...
extern CSgraphApp theApp;

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SGRAPH_H__828224E2_F061_11D3_A367_00C04F72D141__INCLUDED_)
