//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "sgraph.h"
#include <cderr.h>

#include "sgraphDoc.h"
#include "PlotOptionsDialog.h"
#include "SetGraphDialog.h"
#include "ErrorBarDialog.h"
#include "LegendDialog.h"
#include "color.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSgraphDoc
//IMPLEMENT_DYNCREATE(CSgraphDoc, CDocument)
IMPLEMENT_SERIAL(CSgraphDoc, CDocument, 1)

BEGIN_MESSAGE_MAP(CSgraphDoc, CDocument)
	//{{AFX_MSG_MAP(CSgraphDoc)
	ON_COMMAND(ID_FILE_IMPORT, ModalFileImportXY)
	ON_COMMAND(ID_PLOT_TITLE, OnPlotTitle)
	ON_COMMAND(ID_PLOT_XAXIS, OnPlotXAxis)
	ON_COMMAND(ID_PLOT_YAXIS, OnPlotYAxis)
	ON_COMMAND(ID_DATA_ACTIVATE_SET, OnDataActivateSet)
	ON_COMMAND(ID_DATA_DEACTIVATE_SET, OnDataDeactivateSet)
	ON_COMMAND(ID_DATA_DELETE_SET, OnDataDeleteSet)
	ON_COMMAND(ID_PLOT_OPTIONS, OnPlotOptions)
	ON_COMMAND(ID_PLOT_GRAPH_TYPE, OnPlotGraphType)
	ON_COMMAND(ID_PLOT_ERROR_BARS, OnPlotErrorBars)
	ON_COMMAND(ID_PLOT_LEGEND, OnPlotLegend)
	ON_COMMAND(ID_SHIFT_UP, OnShiftUp)
	ON_COMMAND(ID_SHIFT_DOWN, OnShiftDown)
	ON_COMMAND(ID_SHIFT_LEFT, OnShiftLeft)
	ON_COMMAND(ID_SHIFT_RIGHT, OnShiftRight)
	ON_UPDATE_COMMAND_UI(ID_SHIFT_UP, OnUpdateShiftUp)
	ON_UPDATE_COMMAND_UI(ID_SHIFT_DOWN, OnUpdateShiftDown)
	ON_UPDATE_COMMAND_UI(ID_SHIFT_LEFT, OnUpdateShiftLeft)
	ON_UPDATE_COMMAND_UI(ID_SHIFT_RIGHT, OnUpdateShiftRight)
	ON_COMMAND(ID_DATA_SET_TO_BACK, OnDataSetToBack)
	ON_COMMAND(ID_DATA_SET_TO_FRONT, OnDataSetToFront)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDocPreferences
void CDocPreferences::Serialize(CArchive& ar)
{
if (ar.IsStoring()){
		ar << SGRAPH_VERSION;
		ar << lineScaling;
		ar << fontScaling;
	}
	else{
		ar >> version;
		ar >> lineScaling;

		if(lineScaling < 1){
			lineScaling = 1;
			throw;
		}

		if(version < 1.125){
			fontScaling = 1;
		}
		else{
			ar >> fontScaling;

			if(fontScaling < 1){
				fontScaling = 1;
				throw;
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSgraphDoc construction/destruction

CSgraphDoc::CSgraphDoc()
{
	// Use a single graph for now
	num_graph = 1;
	rows = 1;
	cols = 1;
	graph = NULL;

	// Select the single graph
	selected_graph = 0;

	// Background color
	background_color = NO_COLOR;

	// Graph Layout
	width = DefaultWidth;
	height = DefaultHeight;
	verticalGap = DefaultVGap;
	horizontalGap = DefaultHGap;
	startX = DefaultStartX;
	startY = DefaultStartY;

	haltPipe = FALSE;
	m_pipe = NULL;

	// By default, there are no tasks to ask the view to
	// perform at exit.
	atExitTask = AT_EXIT_NONE;
	atExitFilename = _T("");
	atExitExportWidth = 0;
	atExitExportHeight = 0;
}

CSgraphDoc::~CSgraphDoc()
{
	if(graph)
		delete [] graph;

	graph = NULL;
}

void CSgraphDoc::SetPipe(CString &addr)
{
	m_pipe = (void*)(atoi(addr));
}

BOOL CSgraphDoc::OnNewDocument()
{
	// Insert a dialog here to see if the user really wants
	// to throw away the old data and make a new document.

	if (!CDocument::OnNewDocument())
		return FALSE;

	if(graph)
		delete [] graph;
	
	while(!annotate_text.IsEmpty())
		annotate_text.Pop();

	num_graph = 1;
	rows = 1;
	cols = 1;
	graph = new SGraph[num_graph];

	// Select the single graph
	selected_graph = 0;
	graph[selected_graph].isSelected = TRUE;

	// By default, there are no tasks to ask the view to
	// perform at exit.
	atExitTask = AT_EXIT_NONE;
	atExitFilename = _T("");
	atExitExportWidth = 0;
	atExitExportHeight = 0;

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CSgraphDoc serialization

void CSgraphDoc::Serialize(CArchive& ar)
{
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	if (ar.IsStoring()){
		// Note that version control was introduced at version 1.04
		ar << SGRAPH_VERSION;

		ar << selected_graph;
		ar << rows;
		ar << cols;
		
		ar << background_color;

		ar << width;
		ar << height;
		ar << verticalGap;
		ar << horizontalGap;
		ar << startX;
		ar << startY;

		for(int i = 0;i < num_graph;i++)
			graph[i].Serialize(ar);

		// Write annotated text
		annotate_text.Serialize(ar);

		// Write the user preferences
		doc_pref.Serialize(ar);

		// Restore the cursor
		::SetCursor(prevCursor);
	}
	else{
		try{
			BOOL set_graph_dimensions = FALSE;

			// Note that version control was introduced at version 1.04
			ar >> version;
			
			ar >> selected_graph;
			ar >> rows;
			ar >> cols;

			if(version > 1.14001){
				ar >> background_color;
			}
			else{
				background_color = NO_COLOR;
			}

			// Add 0.00001 to make the version test robust.
			// Why didn't I use an int for the version?
			if(version > 1.11001){
				ar >> width;
				ar >> height;
				ar >> verticalGap;
				ar >> horizontalGap;
				ar >> startX;
				ar >> startY;
			}
			else{
				set_graph_dimensions = TRUE;
			}

			num_graph = rows*cols;

			if(num_graph <= 0)
				AfxThrowArchiveException(CArchiveException::badIndex,
					ar.m_pDocument->GetTitle());

			if((selected_graph >= num_graph) || (selected_graph < 0))
				AfxThrowArchiveException(CArchiveException::badIndex,
					ar.m_pDocument->GetTitle());

			if(graph)
				delete [] graph;

			graph = new SGraph[num_graph];

			for(int i = 0;i < num_graph;i++)
				graph[i].Serialize(ar);

			// Since we don't store the is_selected field in the
			// SObject base class, we need to select the current 
			// graph here.
			graph[selected_graph].isSelected = TRUE;

			// Clean any existing annotated text
			while(!annotate_text.IsEmpty())
				annotate_text.Pop();
			
			// Read annotated text
			annotate_text.Serialize(ar);

			if(set_graph_dimensions){
				RecomputeGraphArrangement();
			}

			// Add 0.00001 to make the version test robust.
			// Why didn't I use an int for the version?
			if(version > 1.11001){
				// Read the user preferences
				doc_pref.Serialize(ar);

				// Set the global line scaling variable!
				SObject::lineScale = doc_pref.GetLineScaling();

				// Set the global font scaling variable!
				SObject::fontScale = doc_pref.GetFontScaling();
			}

			// Restore the cursor
			::SetCursor(prevCursor);
		}
		catch(...){
			// Restore the cursor
			::SetCursor(prevCursor);

			// Clean up any corrupted structures and rethrow the
			// error
			if(graph)
				delete [] graph;
			
			while(!annotate_text.IsEmpty())
				annotate_text.Pop();

			num_graph = 1;
			rows = 1;
			cols = 1;
			graph = new SGraph[num_graph];

			// Select the single graph
			selected_graph = 0;
			graph[selected_graph].isSelected = TRUE;

			throw;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSgraphDoc diagnostics

#ifdef _DEBUG
void CSgraphDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSgraphDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSgraphDoc commands

void CSgraphDoc::FileImportXY(CString &filename, int setinfo)
{
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	switch(graph[selected_graph].ImportPlot(filename, setinfo)){
		case S_ERROR_IO_OPEN:
			::SetCursor(prevCursor);
			AfxMessageBox("Unable to open the file " + filename);
			break;
		case S_ERROR_NO_DATA:
			::SetCursor(prevCursor);
			AfxMessageBox("No data found in the file " + filename);
			break;
		case S_ERROR_IO:
			::SetCursor(prevCursor);
			AfxMessageBox("IO error reading from file " + filename);
			break;
		case S_ERROR_IO_BUFFER:
			::SetCursor(prevCursor);
			AfxMessageBox("Buffer overflow (line too long) "
				"reading from file " + filename);
			break;
		case S_ERROR_MEM_ALLOC:
			::SetCursor(prevCursor);
			AfxMessageBox("Can't allocate memory to store data while "
				"reading from file " + filename);
			break;
		case S_ERROR_IO_NUM_COL:
			::SetCursor(prevCursor);
			AfxMessageBox("Missing columns in file " + filename);
			break;
		case S_ERROR_IO_FLOAT:
			::SetCursor(prevCursor);
			// This error results from trying to store all possible
			// floating point number as type float (instead of type
			// double or even long double). We could use type double, 
			// but this would restrict the amount of data that could 
			// be stored in memory.
			AfxMessageBox("Can not convert numbers in " + filename + 
				" to float (4 bytes per value)");
			break;
		default: // No error

		// Make sure that we're not going to attempt to
		// compute the log of a number that is <= 0.
		if((graph[selected_graph].x_scaleType == SCALE_LOG) || 
		   (graph[selected_graph].y_scaleType == SCALE_LOG)){
						
			BOOL x_log_ok = TRUE, y_log_ok = TRUE;

			for(int i = 0;i < graph[selected_graph].plot.Length();i++){

				if(!graph[selected_graph].plot[i].canXLog)
					x_log_ok = FALSE;

				if(!graph[selected_graph].plot[i].canYLog)
					y_log_ok = FALSE;
			}
					
			// Tell the user what has happened.
			if((!x_log_ok && (graph[selected_graph].x_scaleType == SCALE_LOG)) || 
			   (!y_log_ok && (graph[selected_graph].y_scaleType == SCALE_LOG)))
				AfxMessageBox("Can't compute log(n<=0) in " + filename + 
					"; Switching to a linear scale.");

			// Change the scale type(s) to allow all of the
			// data to be displayed
			if(!x_log_ok)
				graph[selected_graph].x_scaleType = SCALE_LINEAR;
					
			if(!y_log_ok)
				graph[selected_graph].y_scaleType = SCALE_LINEAR;
		}

		graph[selected_graph].SetPlotScaleType();
		graph[selected_graph].SetPlotRange();

		SetModifiedFlag();

		// Update the view
		UpdateAllViews(NULL);
		::SetCursor(prevCursor);
	};
}

void CSgraphDoc::ModalFileImportXY()
{
	// Open a file dialog here to load the file name(s)
	CFileDialog fDlg(TRUE);
	
	// Remeber the filter index selected by the user. This will allow
	// persistance when importing multiple files.
	static int persistantFilterIndex = 1;

	fDlg.m_ofn.nFilterIndex = persistantFilterIndex;

	// Set the title of the file dialog
	fDlg.m_ofn.lpstrTitle = "Import";
	fDlg.m_ofn.Flags = fDlg.m_ofn.Flags | OFN_ALLOWMULTISELECT;

	// Allow the user to select multiple files [so we must construct a 
	// string buffer to hold the returned name(s)].
	fDlg.m_ofn.nMaxFile = 2562; // <-- Windows NT 4.0 max. Dooh!

	// Set up the various filter choices
	fDlg.m_ofn.lpstrFilter = "ASCII: x y\0*.*\0"
		"ASCII: x y1 y2 ... yn\0*.*\0"
		"ASCII: x y dx\0*.*\0"
		"ASCII: x y dy\0*.*\0"
		"ASCII: x y dx dy\0*.*\0"
		"AUDIO: wav\0*.wav\0";

	// Save a pointer to the buffer for easy memory clean up!
	char *buf;

	fDlg.m_ofn.lpstrFile = buf = new char [fDlg.m_ofn.nMaxFile];

	if(buf == NULL){
		AfxMessageBox("Memory allocation error!");
		return;
	}
	
	// Set the first character to null to prevent a default filename from 
	// appearing.
	fDlg.m_ofn.lpstrFile[0] = '\0';


	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		// Import the selected file name and check for
		// errors

		if(CommDlgExtendedError() == FNERR_BUFFERTOOSMALL){
			AfxMessageBox("Too many files selected! "
				"Please select fewer files and try again.");

			delete [] buf;

			return;
		}

		// What filter did the user select?
		persistantFilterIndex = fDlg.m_ofn.nFilterIndex;

		int setinfo = persistantFilterIndex - 1;

		if(setinfo < 0){
			AfxMessageBox("Unknown filter!");
			
			// Reset the persistance filter index to its default value
			persistantFilterIndex = 1;

			delete [] buf;

			return;
		}

		POSITION pos = fDlg.GetStartPosition();
		CString file;
		
		while(pos)
			FileImportXY(fDlg.GetNextPathName(pos), setinfo);
	}

	if(CommDlgExtendedError() == FNERR_BUFFERTOOSMALL)
		AfxMessageBox("Too many files selected! "
			"Please select fewer files and try again.");

	// Clean up memeory buffer
	delete [] buf;
}

void CSgraphDoc::OnPlotTitle() 
{
	CString windowTitle("Title");

	if(graph[selected_graph].title.ModalSet(windowTitle)){

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnPlotXAxis() 
{
	CString windowTitle("X Axis");
	if(graph[selected_graph].bottomAxisLable.ModalSet(windowTitle)){

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnPlotYAxis() 
{
	CString windowTitle("Y Axis");
	if(graph[selected_graph].leftAxisLable.ModalSet(windowTitle)){

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnDataActivateSet() 
{
	if(graph[selected_graph].ModalActivatePlot()){
		graph[selected_graph].SetPlotRange();

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnDataDeactivateSet() 
{
	if(graph[selected_graph].ModalDeactivatePlot()){
		graph[selected_graph].SetPlotRange();

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnDataDeleteSet() 
{
	if(graph[selected_graph].ModalDeletePlot()){
		graph[selected_graph].SetPlotRange();

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnDataSetToBack() 
{
	if(graph[selected_graph].ModalPlotToBack()){
		graph[selected_graph].SetPlotRange();

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnDataSetToFront() 
{
	if(graph[selected_graph].ModalPlotToFront()){
		graph[selected_graph].SetPlotRange();

		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

BOOL CSgraphDoc::OnPlotOptions()
{
	// Indicate that the user has selected animate plots
	// for the first time.
	BOOL tmp_animate_plots = graph[selected_graph].animate_plots;

	CPlotOptionsDialog poDlg(this);
	poDlg.DoModal();

	if(graph[selected_graph].animate_plots){
		if((graph[selected_graph].curr_animate_plot < 0) && (graph[selected_graph].plot.Length() > 0))
			graph[selected_graph].curr_animate_plot = 0;
	}

	return (graph[selected_graph].animate_plots && !tmp_animate_plots);
}

void CSgraphDoc::OnPlotGraphType() 
{
	CSetGraphDialog sgDlg(graph[selected_graph].x_scaleType, 
		graph[selected_graph].y_scaleType);

	if(sgDlg.DoModal() == IDOK){
		if(graph[selected_graph].SetGraphType(sgDlg.GetXAxisType(), 
											  sgDlg.GetYAxisType())){
			// Update the view
			SetModifiedFlag();
			UpdateAllViews(NULL);
		}
	}
}

void CSgraphDoc::OnPlotErrorBars() 
{
	CErrorBarDialog ebDlg(this);
	
	ebDlg.DoModal();
}

SObject* CSgraphDoc::SelectAnnotatedText(BOOL &status, CPoint &xy, CDC* pDC, CDC* backDC, 
										 CRect &extent) 
{
	// Have any text objects been selected?
	BOOL wasSelected;
	
	SObject* isSelected = NULL;

	for(int i = 0;i < annotate_text.Length();i++){
		wasSelected = annotate_text[i].isSelected;

		if(status)
			annotate_text[i].isSelected = FALSE;
		else
			status = annotate_text[i].Select(xy, pDC, extent);

		if(annotate_text[i].isSelected){
			if(wasSelected)
				// The currently selected text has been selected again - 
				// there is no need to draw a new selection box around it!
				isSelected = NULL;
			else
				isSelected = &(annotate_text[i]);
		}

		if((wasSelected && !annotate_text[i].isSelected)){
			annotate_text[i].DrawSelect(pDC, extent);
			if(backDC)
				annotate_text[i].DrawSelect(backDC, extent);
		}
	}

	// Was any NEW text selected?
	return isSelected;
}

void CSgraphDoc::OnPlotLegend() 
{
	CLegendDialog lDlg(this);
	
	if(lDlg.DoModal() == IDOK){
		// Update the view
		SetModifiedFlag();
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnShiftUp() 
{
	if(graph[selected_graph].y_scaleType == SCALE_LINEAR){
		graph[selected_graph].ShiftY(0.1f);
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnShiftDown() 
{
	if(graph[selected_graph].y_scaleType == SCALE_LINEAR){
		graph[selected_graph].ShiftY(-0.1f);
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnShiftLeft() 
{
	if(graph[selected_graph].x_scaleType == SCALE_LINEAR){
		graph[selected_graph].ShiftX(-0.1f);
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnShiftRight() 
{
	if(graph[selected_graph].x_scaleType == SCALE_LINEAR){
		graph[selected_graph].ShiftX(0.1f);
		UpdateAllViews(NULL);
	}
}

void CSgraphDoc::OnUpdateShiftUp(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(graph[selected_graph].y_scaleType == SCALE_LINEAR);
}

void CSgraphDoc::OnUpdateShiftDown(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(graph[selected_graph].y_scaleType == SCALE_LINEAR);
}

void CSgraphDoc::OnUpdateShiftLeft(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(graph[selected_graph].x_scaleType == SCALE_LINEAR);
}

void CSgraphDoc::OnUpdateShiftRight(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(graph[selected_graph].x_scaleType == SCALE_LINEAR);
}

// Recompute the graph arrangement, including the Graph dimensions
// (width, height) and Spacing between graphs (verticalGap, horizontalGap), 
// from the graph data. Needed for backwards compatitbility.
void CSgraphDoc::RecomputeGraphArrangement()
{
	// The width and the height can be obtained from the selected graph
	width = graph[selected_graph].x2 - graph[selected_graph].x1;

	height = graph[selected_graph].y2 - graph[selected_graph].y1;

	if(num_graph > 1){
		if(cols > 1){
			// Set the horizontalGap value.
			horizontalGap = graph[1].x1 - graph[0].x2;
		}

		if(rows > 1){
			// Set the verticalGap value.
			verticalGap = graph[cols].y1 - graph[0].y2;
		}
	}
}

