//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SGRAPHDOC_H__828224E8_F061_11D3_A367_00C04F72D141__INCLUDED_)
#define AFX_SGRAPHDOC_H__828224E8_F061_11D3_A367_00C04F72D141__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDocPreferences{
private:
	int lineScaling;
	float fontScaling;
	float version;
public:
	CDocPreferences()
	{
		version = SGRAPH_VERSION;
		lineScaling = 1;
		fontScaling = 1.0f;
	};
	virtual void Serialize(CArchive& ar);

	int GetLineScaling()
	{
		return lineScaling;
	};

	void SetLineScaling(int m_lineScaling)
	{
		if(m_lineScaling < 1){
			AfxMessageBox("Bad lineScaling parameter!");
		}
		else{
			lineScaling = m_lineScaling;
		}
	};

	float GetFontScaling()
	{
		return fontScaling;
	};

	void SetFontScaling(float m_fontScaling)
	{
		if(m_fontScaling < 0.0f){
			AfxMessageBox("Bad fontScaling parameter!");
		}
		else{
			fontScaling = m_fontScaling;
		}
	};
};

class CSgraphDoc : public CDocument
{
protected: 
	// create from serialization only
	CSgraphDoc();

	//DECLARE_DYNCREATE(CSgraphDoc)
	DECLARE_SERIAL(CSgraphDoc)

	// Compute the width, height, verticalGap and horizontalGap
	// for backwards compatitbility.
	void RecomputeGraphArrangement();

// Attributes
public:
	// Version control
	float version;

	// User preferences of a global nature
	CDocPreferences doc_pref;

	// The graph(s)
	SGraph *graph;
	int num_graph;
	int rows, cols;

	// Graph dimensions
	float width, height;

	// Spacing between graphs
	float verticalGap, horizontalGap;

	// The lower left hand corner of the lower
	// left hand graph.
	float startX, startY;

	int selected_graph;

	// The background color
	COLORREF background_color;

	// Text annotations
	list<STextObject> annotate_text;
	
	// A read pipe
	HANDLE m_pipe;

	// A flag to stop the pipe
	BOOL haltPipe;
	
	// ********** Operations to perform at exit **********
	enum {AT_EXIT_NONE,			// Do nothing
		  AT_EXIT_EXPORT_BMP,	// Export a BMP file
		  AT_EXIT_EXPORT_PNG};	// Export a PNG file

	CString atExitFilename;
	int atExitTask;
	int atExitExportWidth;
	int atExitExportHeight;

// Operations
public:

	// Allow the user to select annoated text
	SObject* SelectAnnotatedText(BOOL &status, CPoint &xy, CDC* pDC, 
		CDC* backDC, CRect &extent);

	void SetPipe(CString &addr);
	void FileImportXY(CString &filename, int setinfo);
	void ListenToPipe();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSgraphDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CSgraphDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
public:
	//{{AFX_MSG(CSgraphDoc)
	afx_msg void ModalFileImportXY();
	afx_msg void OnPlotTitle();
	afx_msg void OnPlotXAxis();
	afx_msg void OnPlotYAxis();
	afx_msg void OnDataActivateSet();
	afx_msg void OnDataDeactivateSet();
	afx_msg void OnDataDeleteSet();
	afx_msg BOOL OnPlotOptions();
	afx_msg void OnPlotGraphType();
	afx_msg void OnPlotErrorBars();
	afx_msg void OnPlotLegend();
	afx_msg void OnShiftUp();
	afx_msg void OnShiftDown();
	afx_msg void OnShiftLeft();
	afx_msg void OnShiftRight();
	afx_msg void OnUpdateShiftUp(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShiftDown(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShiftLeft(CCmdUI* pCmdUI);
	afx_msg void OnUpdateShiftRight(CCmdUI* pCmdUI);
	afx_msg void OnDataSetToBack();
	afx_msg void OnDataSetToFront();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SGRAPHDOC_H__828224E8_F061_11D3_A367_00C04F72D141__INCLUDED_)
