//	(c) 2000 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"

// Note that afxpriv.h must be included to get the declaration
// of the CPreviewDC class. The following line should be included
// at the very top of your view's .CPP file
#include <afxpriv.h>

#include "sgraph.h"
#include "MainFrm.h"
#include "sgraphDoc.h"
#include "sgraphView.h"
#include "ArrangeGraphDialog.h"
#include "PlotStatisticsDialog.h"
#include "HistogramDialog.h"
#include "DerivativeDialog.h"
#include "AxisDialog.h"
#include "SelectGraphDialog.h"
#include "IntegrateDialog.h"
#include "PasteDialog.h"
#include "GraphLineDialog.h"
#include "MovieDialog.h"
#include "ScaleDialog.h"
#include "PropertiesDialog.h"
#include "EquationDialog.h"
#include "ParametricFunctionDialog.h"
#include "PolarEquationDialog.h"
#include "NonLinFitDialog.h"
#include "PrestoColor.h"
#include "AudioDialog.h"
#include "OverlapGraphDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSgraphView

BOOL extern is_16_bit;

// Local functions
UINT PipeWrapper(LPVOID lparam);

IMPLEMENT_DYNCREATE(CSgraphView, CView)

BEGIN_MESSAGE_MAP(CSgraphView, CView)
	//{{AFX_MSG_MAP(CSgraphView)
	ON_WM_SIZE()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_PLOT_DEFAULT_SCALE, OnPlotDefaultScale)
	ON_COMMAND(ID_PLOT_ZOOM, OnPlotZoom)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MBUTTONDOWN()
	ON_WM_MBUTTONUP()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_PLOT_ARRANGE, OnPlotArrange)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_DATA_STAT, OnDataStat)
	ON_COMMAND(ID_VIEW_CROSSHAIR, OnViewCrosshair)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CROSSHAIR, OnUpdateViewCrosshair)
	ON_COMMAND(ID_DATA_HIST, OnDataHist)
	ON_COMMAND(ID_GRAPH_X_AXIS, OnGraphXAxis)
	ON_COMMAND(ID_GRAPH_Y_AXIS, OnGraphYAxis)
	ON_COMMAND(ID_PLOT_SCALING, OnPlotScaling)
	ON_COMMAND(ID_FILE_EXPORT, OnFileDataExport)
	ON_COMMAND(ID_FILE_EXPORT_AUDIO, OnFileExportAudio)
	ON_COMMAND(ID_DATA_DERIV, OnDataDeriv)
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_PLOT_USER_TEXT, OnPlotUserText)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR, OnUpdateEditClear)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_VIEW_DOUBLEBUFFER, OnViewDoublebuffer)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DOUBLEBUFFER, OnUpdateViewDoublebuffer)
	ON_WM_KILLFOCUS()
	ON_COMMAND(ID_PLOT_GRAPH_COLOR, OnPlotGraphColor)
	ON_COMMAND(ID_PLOT_GRAPH_SELECT, OnPlotGraphSelect)
	ON_COMMAND(ID_DATA_INTEGRATE, OnDataIntegrate)
	ON_COMMAND(ID_ANIMATE_PLOT, OnAnimatePlot)
	ON_UPDATE_COMMAND_UI(ID_ANIMATE_PLOT, OnUpdateAnimatePlot)
	ON_COMMAND(ID_VIEW_REFRESH, OnViewRefresh)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_PLOT_LINES, OnPlotLines)
	ON_COMMAND(ID_VIEW_WYSIWYG, OnViewWysiwyg)
	ON_UPDATE_COMMAND_UI(ID_VIEW_WYSIWYG, OnUpdateViewWysiwyg)
	ON_COMMAND(ID_FILE_EXPORT_ANIMATION, OnFileExportAnimation)
	ON_COMMAND(ID_VIEW_PROPERTIES, OnViewProperties)
	ON_COMMAND(ID_EDIT_COPY_DATA, OnEditCopyData)
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_DATA_FUNC, OnDataFunc)
	ON_COMMAND(ID_DATA_NON_LIN, OnDataNonLin)
	ON_COMMAND(ID_PAGE_COLOR, OnPageColor)
	ON_COMMAND(ID_DATA_PARA_FUNC, OnDataParaFunc)
	ON_COMMAND(ID_DATA_POLAR_FUNC, OnDataPolarFunc)
	ON_COMMAND(ID_FILE_EXPORT_IMAGE, OnFileExportImage)
	ON_COMMAND(ID_PAGE_OVERLAP, OnPageOverlap)
	ON_MESSAGE(WM_ANIMATE_CLOSE, OnAnimatePlotClose)
	ON_MESSAGE(WM_ANIMATE_ADJUST, OnAnimatePlotAdjust)
	ON_MESSAGE(WM_START_PIPE, StartPipe)
	ON_MESSAGE(WM_UPDATE_WYSIWYG, SetWYSIWYGAspectRatio)
	ON_UPDATE_COMMAND_UI(ID_PAGE_OVERLAP, OnUpdatePageOverlap)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSgraphView construction/destruction

CSgraphView::CSgraphView()
{	
	doubleBuffer = TRUE;
	drawBuffer = NULL;
	m_pDC = NULL;
	
	// The cross hair option doesn't go in the InitDefaults
	// because it is not tied to a specific graph.
	isCrossHair = FALSE;

	// Load the cursors
	m_hcurArrow = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
    m_hcurCross = AfxGetApp()->LoadStandardCursor(IDC_CROSS);
	m_hcurMove = AfxGetApp()->LoadStandardCursor(IDC_SIZEALL);
	m_hcurIBeam = AfxGetApp()->LoadStandardCursor(IDC_IBEAM);
	m_hcurPipe = AfxGetApp()->LoadStandardCursor(IDC_APPSTARTING);

	m_hcurCurrent = m_hcurArrow;

	capDlg = new CAnimatePlotDialog(this);

	isWYSIWYG = FALSE;
	aspectWYSIWYG = 1.0f;

	InitDefaults();
}

void CSgraphView::InitDefaults()
{
	pointerState = SG_POINTER_FREE;

	isPrinting = FALSE;
	isSelected = NULL;
	isMoving = FALSE;

	render_background_color = FALSE;

	// Redraw the double buffer
	flushBuffer = TRUE;

	if((capDlg != NULL) && capDlg->GetSafeHwnd()){
		capDlg->PostMessage(WM_ANIMATE_UPDATE);
	}
}

CSgraphView::~CSgraphView()
{
	if(capDlg)
		delete capDlg;
}

BOOL CSgraphView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

int CSgraphView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	if(m_pDC)
		delete m_pDC;

	m_pDC = new CDC;

	// Make sure that the DC was created!
	ASSERT_VALID(m_pDC);

	CDC *pDC = GetDC();

	m_pDC->CreateCompatibleDC(pDC);

	ReleaseDC(pDC);

	return 0;
}

void CSgraphView::OnDestroy() 
{
	// Before cleaning up, check to see if the document has any
	// tasks that need to be performed at exit
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	switch(pDoc->atExitTask){
		case CSgraphDoc::AT_EXIT_NONE:
			// Nothing to do!
			break;
		case CSgraphDoc::AT_EXIT_EXPORT_BMP:
			// Export the current image as a BMP
			ExportImageBMP(pDoc->atExitFilename, 
				pDoc->atExitExportWidth,
				pDoc->atExitExportHeight);
			break;
		case CSgraphDoc::AT_EXIT_EXPORT_PNG:
			// Export the current image as a PNG
			ExportImagePNG(pDoc->atExitFilename, 
				pDoc->atExitExportWidth,
				pDoc->atExitExportHeight);
			break;
	};

	CView::OnDestroy();

    if(m_pDC){
        delete m_pDC;
		m_pDC = NULL;
	}

	if(drawBuffer){
		delete drawBuffer;
		drawBuffer = NULL;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSgraphView drawing

BOOL CSgraphView::OnEraseBkgnd(CDC* pDC) 
{

	if(doubleBuffer && !isPrinting){
		if(!flushBuffer){
			// Don't need to modify the bitmap if nothing has changed!
			return 1;
		}

		CBrush brush (0x00ffffff); // White background for now

		m_pDC->FillRect(extent, &brush);

		return 1; // We've erased the background
	}
	
	return CView::OnEraseBkgnd(pDC);
}

void CSgraphView::ForceRedraw()
{
	flushBuffer = TRUE;
	InvalidateRect(NULL, TRUE);
}

void CSgraphView::OnDraw(CDC* pDC)
{
	CDC* drawDC;

	if(doubleBuffer && !isPrinting){
		if(!flushBuffer){
			// The window contents have not changed - don't redraw, just
			// reapply the bitmap of the window.
			pDC->BitBlt(0, 0, extent.Width(), extent.Height(), m_pDC, 0, 0, SRCCOPY);

			flushBuffer = FALSE;

			return;
		}

		drawDC = m_pDC;
	}
	else
		drawDC = pDC;

	// Hummm ... Not quite sure whether this writing to the status bar is
	// warrented. It's useful when drawing large sets (when there is a 
	// lengthy redraw time) but annoying for small data sets.
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus = &pFrame->m_wndStatusBar;

	if(pStatus)
		pStatus->SetPaneText(INFO_PANE, "Rendering ...");

	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	int i;

	// Draw a background color if requested
	if(render_background_color || pDoc->background_color != NO_COLOR){
		// If the current background color is NO_COLOR, 
		// use the system window color.
		CBrush bkgrnd(pDoc->background_color == NO_COLOR ? 
			GetSysColor(COLOR_WINDOW): pDoc->background_color);
		
		CRgn bbox;

		if(isPrinting){
			// printerExtent is copied from the CPrintInfo structure in OnPrint.
			bbox.CreateRectRgn(0, 0, printerExtent.Width(), printerExtent.Height());
		}
		else{
			bbox.CreateRectRgn(0, 0, extent.Width(), extent.Height());
		}

		drawDC->FillRgn(&bbox, &bkgrnd);
	}

	// For a single graph, take up the entire window
	// (always draw the selected graph last!)
	for(i = 0;i < pDoc->selected_graph;i++)
		pDoc->graph[i].Draw(drawDC, extent);

	for(i = pDoc->selected_graph + 1;i < pDoc->num_graph;i++)
		pDoc->graph[i].Draw(drawDC, extent);

	pDoc->graph[pDoc->selected_graph].Draw(drawDC, extent);
	
	// Draw any user defined text
	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].Draw(drawDC, extent);

	// Redraw the cross hair if necessary
	if(isCrossHair && (lastCross.x >= 0) && !isPrinting){
		// Create a black solid pen
		CPen pen(PS_SOLID, 1, RGB(0,0,0));
		CPen *oldpen = drawDC->SelectObject(&pen);

		// Select ROP2 mode
		int ropOld = drawDC->SetROP2(R2_NOT);

		// Draw the vertical portion of the previous "cross hair"
		drawDC->MoveTo(lastCross.x, extent.top);
		drawDC->LineTo(lastCross.x, extent.bottom);

		// Draw the horizontal portion of the previous "cross hair"
		drawDC->MoveTo(extent.left, lastCross.y);
		drawDC->LineTo(extent.right, lastCross.y);
		
		// Restore the previous state
		drawDC->SelectObject(oldpen);
		drawDC->SetROP2(ropOld);
	}

	if(doubleBuffer && !isPrinting)
		pDC->BitBlt(0, 0, extent.Width(), extent.Height(), m_pDC, 0, 0, SRCCOPY);

	flushBuffer = FALSE;

	if(pStatus)
		pStatus->SetPaneText(INFO_PANE, "Ready");
}

/////////////////////////////////////////////////////////////////////////////
// CSgraphView printing

void CSgraphView::OnPrint(CDC *pDC, CPrintInfo* pInfo)
{
	int i;
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	float screen_aspect_ratio;
	float print_aspect_ratio;
	long scale_div, scale_mul;

	// Set the isPrinting flag to let the OnDraw function
	// test for the current state (i.e. render to screen or
	// to another device). This is needed because some screen
	// elements (like the cross hairs) shouldn't be printed.
	isPrinting = TRUE;

	// Save the size of the printed page (we'll need it for applying the
	// background color to the entire page).
	printerExtent = pInfo->m_rectDraw;

	CRect tmpExtent = extent;
	
	screen_aspect_ratio = (tmpExtent.Width() == 0) ? 1.0f : 
		(float(tmpExtent.Height()))/tmpExtent.Width();

	if(pInfo){
		print_aspect_ratio = (pInfo->m_rectDraw.Width() == 0) ? 1.0f : 
			(float(pInfo->m_rectDraw.Height()))/pInfo->m_rectDraw.Width();
	}
	else{
		print_aspect_ratio = screen_aspect_ratio;
	}

	// Maintain the screen aspect ratio and center the graph on the page
	if(pInfo){
		if((screen_aspect_ratio > print_aspect_ratio)){
			int width = (int)(pInfo->m_rectDraw.Height()/screen_aspect_ratio);
			int dx = (pInfo->m_rectDraw.Width() - width)/2;

			extent = CRect(dx, 0, 
				width + dx, 
				pInfo->m_rectDraw.Height());
		}
		else{
			int height;
			int dy;

			height = (int)(pInfo->m_rectDraw.Width()*screen_aspect_ratio);
			dy = (pInfo->m_rectDraw.Height() - height)/2;

			extent = CRect(0, dy, pInfo->m_rectDraw.Width(), height + dy);
		}
	}

	if(isWYSIWYG){
		// To rescale the graph for printing, divide by the screen height (in pixels)
		// and multiply by the printer height (in pixels). The scale factors 
		// are computed in pixels because we want the same relative font size for the
		// screen and the printer.

		scale_div = tmpExtent.Height();	// The screen height
		scale_mul = extent.Height();	// The pinter height
	}
	else{
		// To rescale the graph for printing, divide by the screen resolution (in dpi)
		// and multiply by the printer height (in dpi). Aboslute text size is
		// maintained.

		if (pDC->IsKindOf(RUNTIME_CLASS(CPreviewDC)))
			scale_mul = GetDeviceCaps(pDC->m_hAttribDC, LOGPIXELSY);
		else
			scale_mul = GetDeviceCaps(pDC->m_hDC, LOGPIXELSY);

		CDC *screenDC = GetDC();
		scale_div = GetDeviceCaps(screenDC->m_hDC, LOGPIXELSY);
		ReleaseDC(screenDC);
	}

	// Prepare the graph(s)
	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].PreparePrinting(pDC, scale_mul, scale_div, extent);

	// Prepare the annotated text
	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].PreparePrinting(pDC, scale_mul, scale_div, extent);

	CView::OnPrint(pDC, pInfo);

	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].CleanUpPrinting(pDC, extent);

	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].CleanUpPrinting(pDC, extent);

	extent = tmpExtent;

	isPrinting = FALSE;
}

BOOL CSgraphView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return DoPreparePrinting(pInfo);
}

/////////////////////////////////////////////////////////////////////////////
// CSgraphView diagnostics

#ifdef _DEBUG
void CSgraphView::AssertValid() const
{
	CView::AssertValid();
}

void CSgraphView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSgraphDoc* CSgraphView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSgraphDoc)));
	return (CSgraphDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSgraphView message handlers
void CSgraphView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	extent = CRect(0, 0, cx, cy);

	if(doubleBuffer && !isPrinting){
		if(drawBuffer)
			delete drawBuffer;

		drawBuffer = new CBitmap;

		ASSERT_VALID(drawBuffer);
		ASSERT_VALID(m_pDC);
		
		CDC *pDC = GetDC();

		drawBuffer->CreateCompatibleBitmap(pDC, cx, cy);

		ReleaseDC(pDC);

		m_pDC->SelectObject(drawBuffer);

		flushBuffer = TRUE;
	}

	// Keep the user informed of the window size
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		CString str;
		CStatusBar* pStatus = &pFrame->m_wndStatusBar;

		str.Format("Window Size %d x %d", cx, cy);
		pStatus->SetPaneText(XY_PANE, str);
	}
}

void CSgraphView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CString str;
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus = &pFrame->m_wndStatusBar;

	// Get the current device context
	CDC *pDC = GetDC();

	if(pStatus){
		CSgraphDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		str.Format("X = %-9.6g, Y = %-9.6g", 
			pDoc->graph[pDoc->selected_graph].
				ScreentoDataX(point.x, extent), 
			pDoc->graph[pDoc->selected_graph].
				ScreentoDataY(point.y, extent));

		pStatus->SetPaneText(XY_PANE, str);

		str.Format("G%d", pDoc->selected_graph);

		pStatus->SetPaneText(GRAPH_PANE, str);
	}
	
	if(isCrossHair){
		// Create black solid pen
		CPen pen(PS_SOLID, 1, RGB(0,0,0));
		CPen *oldpen = pDC->SelectObject(&pen);

		// Select ROP2 mode
		int ropOld = pDC->SetROP2(R2_NOT);

		if(lastCross.x >= 0){
			// Draw the vertical portion of the previous "cross hair"
			pDC->MoveTo(lastCross.x, extent.top);
			pDC->LineTo(lastCross.x, extent.bottom);

			// Draw the horizontal portion of the previous "cross hair"
			pDC->MoveTo(extent.left, lastCross.y);
			pDC->LineTo(extent.right, lastCross.y);

			if(doubleBuffer){
				// Mirror the cross hair drawing on the double buffer
				m_pDC->SelectObject(&pen);
				m_pDC->SetROP2(R2_NOT);

				m_pDC->MoveTo(lastCross.x, extent.top);
				m_pDC->LineTo(lastCross.x, extent.bottom);
				m_pDC->MoveTo(extent.left, lastCross.y);
				m_pDC->LineTo(extent.right, lastCross.y);
			}
		}

		// Draw the vertical portion of the "cross hair"
		pDC->MoveTo(point.x, extent.top);
		pDC->LineTo(point.x, extent.bottom);

		// Draw the horizontal portion of the "cross hair"
		pDC->MoveTo(extent.left, point.y);
		pDC->LineTo(extent.right, point.y);
		
		if(doubleBuffer){
			// Mirror the cross hair drawing on the double buffer
			m_pDC->MoveTo(point.x, extent.top);
			m_pDC->LineTo(point.x, extent.bottom);
			m_pDC->MoveTo(extent.left, point.y);
			m_pDC->LineTo(extent.right, point.y);

			m_pDC->SelectObject(oldpen);
			m_pDC->SetROP2(ropOld);
		}

		// Save the current mouse position
		lastCross = point;

		// Restore the previous state
		pDC->SelectObject(oldpen);
		pDC->SetROP2(ropOld);
	}

	switch(pointerState){
		case SG_POINTER_ZOOM_START:
			::SetCursor(m_hcurCross);
			break;

		case SG_POINTER_ZOOM:
			::SetCursor(m_hcurCross);

			if((nFlags & MK_LBUTTON) && !(nFlags & MK_RBUTTON)){

				// Select ROP2 mode
				int ropOld = pDC->SetROP2(R2_XORPEN);
				// int ropOld = pDC->SetROP2(R2_NOT);

				// Create black dotted pen
				CPen linepen(PS_DOT, 1, COLORREF (0) /* Black */);

				CPen *oldpen = pDC->SelectObject(&linepen);

				// Move to rectangle stretch initial point
				pDC->MoveTo(zoomStart.x, zoomStart.y);
		
				// Draw the previous rectangle
				pDC->LineTo(zoomEnd.x, zoomStart.y);
				pDC->LineTo(zoomEnd.x, zoomEnd.y);
				pDC->LineTo(zoomStart.x, zoomEnd.y);
				pDC->LineTo(zoomStart.x, zoomStart.y);

				// Draw the new rectangle
				// Move to rectangle stretch initial point
				pDC->MoveTo(zoomStart.x, zoomStart.y);

				pDC->LineTo(point.x, zoomStart.y);
				pDC->LineTo(point.x, point.y);
				pDC->LineTo(zoomStart.x, point.y);
				pDC->LineTo(zoomStart.x, zoomStart.y);

				// Restore the previous state
				pDC->SelectObject(oldpen);
				pDC->SetROP2(ropOld);

				// Update the selection rectangle
				zoomEnd = point;
			}	
			break;
		case SG_POINTER_TEXT:
			::SetCursor(m_hcurIBeam);
			break;
		case SG_POINTER_SCROLL:
			::SetCursor(m_hcurMove);

			// Is the user pressing the middle mouse button and the control key?
			if((nFlags & MK_MBUTTON) && (nFlags & MK_CONTROL)){
				CSgraphDoc* pDoc = GetDocument();
				ASSERT_VALID(pDoc);

				float minX, maxX, minY, maxY;
				float dscroll;
				pDoc->graph[pDoc->selected_graph].GetPlotRange(minX, maxX, minY, maxY);

				scrollEnd = point;

				// Only scroll linear axes
				if(pDoc->graph[pDoc->selected_graph].x_scaleType == SCALE_LINEAR){
					dscroll = pDoc->graph[pDoc->selected_graph].ScreentoDataX(scrollEnd.x, extent) - 
						pDoc->graph[pDoc->selected_graph].ScreentoDataX(scrollStart.x, extent);

					// The choice of sign here determines if the mouse
					// moves the graph (with a fixed view) or move the
					// view (with a fixed graph).
					minX -= dscroll;
					maxX -= dscroll;
				}

				if(pDoc->graph[pDoc->selected_graph].y_scaleType == SCALE_LINEAR){
					dscroll = pDoc->graph[pDoc->selected_graph].ScreentoDataY(scrollEnd.y, extent) - 
						pDoc->graph[pDoc->selected_graph].ScreentoDataY(scrollStart.y, extent);

					// The choice of sign here determines if the mouse
					// moves the graph (with a fixed view) or move the
					// view (with a fixed graph).
					minY -= dscroll;
					maxY -= dscroll;
				}

				// Update the scolling reference points
				scrollStart = scrollEnd;

				// Update the current graph plot range
				pDoc->graph[pDoc->selected_graph].SetPlotRange(minX, maxX, minY, maxY,
					TRUE /* Override default scaling */);

				// Update the graph (even in the case of a log-log plot, where the scaling
				// is not allowed to change! Wastefull but simple...).
				ForceRedraw();
			}

			break;
		default:
			if((nFlags & MK_LBUTTON) && !(nFlags & MK_RBUTTON) && (isSelected != NULL)){
				isMoving = TRUE;
			
				// Grab control of the Mouse for the duration of the zoom.
				SetCapture();
			
				::SetCursor(m_hcurMove);

				// Couldn't find a way to show the text moving with the
				// cursor, so just move the selection box instead.
				// "Erase" the previous position
				isSelected->DrawSelect(pDC, extent);
				
				if(doubleBuffer)
					isSelected->DrawSelect(m_pDC, extent);

				// Draw the next position
				float tmp_x = isSelected->ScreentoX(point.x, extent), 
					  tmp_y = isSelected->ScreentoY(point.y, extent);

				float dx = tmp_x - move_x;
				float dy = tmp_y - move_y;
			
				move_x = isSelected->MoveX(dx);
				move_y = isSelected->MoveY(dy);
			
				// Make sure that the selected object remains on the 
				// screen at all times (otherwise a user could "lose" an
				// object of screen and not get it back)!
				if((move_x >= 0.0) && (move_x <= 1.0))
					//isSelected->x = move_x;
					isSelected->UpdateX(move_x);
			
				if((move_y >= 0.0) && (move_y <= 1.0))
					//isSelected->y = move_y;
					isSelected->UpdateY(move_y);

				move_x = tmp_x;
				move_y = tmp_y;

				isSelected->DrawSelect(pDC, extent);

				if(doubleBuffer)
					isSelected->DrawSelect(m_pDC, extent);
			}
			else
				::SetCursor(m_hcurCurrent);
	};

	ReleaseDC(pDC);

	CView::OnMouseMove(nFlags, point);
}

void CSgraphView::OnPlotDefaultScale() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if((pointerState == SG_POINTER_ZOOM) || 
	   (pointerState == SG_POINTER_ZOOM_START)){
		pointerState = SG_POINTER_FREE;
		::SetCursor(m_hcurCurrent);
	}

	pDoc->graph[pDoc->selected_graph].SetPlotRange();

	ForceRedraw();
}

void CSgraphView::OnPlotZoom() 
{
	pointerState = SG_POINTER_ZOOM_START;

	::SetCursor(m_hcurCross);
}

BOOL CSgraphView::SelectObject(CPoint point) 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDC *pDC = GetDC();
	SObject *tmpPtr;
	int i, status = FALSE;
	
	// Let the user select annonated text. If the user selects previously selected
	// text, tmpPtr will be NULL and status will be TRUE.
	tmpPtr = pDoc->SelectAnnotatedText(status, point, pDC, m_pDC, extent);

	// If any text has been selected, draw a selection box around it.
	if(tmpPtr){
		tmpPtr->DrawSelect(pDC, extent);

		if(doubleBuffer)
			tmpPtr->DrawSelect(m_pDC, extent);
	}

	// status will be true if we're already selected text
	for(i = 0;i < pDoc->num_graph;i++){
		if(pDoc->graph[i].Select(status, point, pDC, doubleBuffer, m_pDC, extent)){
			if(i != pDoc->selected_graph){
				// De-select the previously selected graph
				pDoc->graph[pDoc->selected_graph].DrawSelect(pDC, extent);
					
				if(doubleBuffer)
					// Draw selection marks on back buffer too!
					pDoc->graph[pDoc->selected_graph].DrawSelect(m_pDC, extent);

				// De-select any graph elements (like titles and lables) that
				// belong to the previously selected graph.
				pDoc->graph[pDoc->selected_graph].Select(TRUE, 
					point, pDC, doubleBuffer, m_pDC, extent);

				// Update the selected graph index
				pDoc->selected_graph = i;

				// Draw the selection indicator for the new graph
				pDoc->graph[pDoc->selected_graph].DrawSelect(pDC, extent);

				if(doubleBuffer)
					// Draw selection marks on back buffer too!
					pDoc->graph[pDoc->selected_graph].DrawSelect(m_pDC, extent);
			}
				
			// We've selected an object - so don't try to select any more
			status = TRUE;
		}
	}

	// Make the pDoc->selected_graph the selected graph 
	// (even if annotated text has been selected).
	pDoc->graph[pDoc->selected_graph].isSelected = TRUE;

	ReleaseDC(pDC);

	return status;
}

void CSgraphView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Don't do anything if the user is already holding down
	// another button
	if((nFlags & MK_RBUTTON) || (nFlags & MK_MBUTTON)){
		CView::OnLButtonDown(nFlags, point);
		return;
	}

	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	CStatusBar* pStatus = &pFrame->m_wndStatusBar;

	isSelected = NULL;

	switch(pointerState){
		case SG_POINTER_ZOOM_START:
			zoomStart = zoomEnd = point;

			// Grab control of the Mouse for the duration of the zoom.
			SetCapture();

			// Now that the user has clicked the left mouse button,
			// we can zoom.
			pointerState = SG_POINTER_ZOOM;
			break;
		case SG_POINTER_TEXT:
			// Put this case statement into its own scope
			{
				STextObject userText;
			
				userText.x = userText.ScreentoX(point.x, extent);
				userText.y = userText.ScreentoY(point.y, extent);

				// Has the user pressed OK and entered a non-zero length string?
				if(userText.ModalSet("Add a text lable") && 
				   (userText.Length() > 0)){
					CSgraphDoc* pDoc = GetDocument();
					ASSERT_VALID(pDoc);

					pDoc->annotate_text.Push(userText);

					// Don't just add the new text, but force a redraw to make sure
					// that the text is added to the double buffer.
					ForceRedraw();

					// The document has been modified
					pDoc->SetModifiedFlag();
				}

				pointerState = SG_POINTER_FREE;
				::SetCursor(m_hcurCurrent);
			}	
			break;
		default:
			if(SelectObject(point)){
				// Have we selected a non-graph object?
				// For now, we restrict the selected objects to text and axes.
				CSgraphDoc* pDoc = GetDocument();
				ASSERT_VALID(pDoc);
				
				for(int i= 0;i < pDoc->annotate_text.Length();i++)
					if(pDoc->annotate_text[i].isSelected)
						isSelected = &(pDoc->annotate_text[i]);

				if(pDoc->graph[pDoc->selected_graph].legend.isSelected)
					isSelected = &(pDoc->graph[pDoc->selected_graph].legend);

				if(pDoc->graph[pDoc->selected_graph].bottomAxisLable.isSelected)
					isSelected = &(pDoc->graph[pDoc->selected_graph].bottomAxisLable);
			
				if(pDoc->graph[pDoc->selected_graph].leftAxisLable.isSelected)
					isSelected = &(pDoc->graph[pDoc->selected_graph].leftAxisLable);
			
				if(pDoc->graph[pDoc->selected_graph].title.isSelected)
					isSelected = &(pDoc->graph[pDoc->selected_graph].title);
				
				if(nFlags & MK_SHIFT){
					// The user is selecting a graph to move
					isSelected = &(pDoc->graph[pDoc->selected_graph]);

					// Which element(s) of the graph should be moved?
					pDoc->graph[pDoc->selected_graph].PrepareMove();
				}
				else{
					// Was the user trying to move a graph or axis?
					if(isSelected == NULL){
						if(pStatus){
							// Provide helpfull info to users who are not familiar
							// with the interface.
							pStatus->SetPaneText(INFO_PANE, "Shift + Left button to move");
						}
					}
				}

				if(isSelected != NULL){
					move_x = isSelected->ScreentoX(point.x, extent);
					move_y = isSelected->ScreentoY(point.y, extent);
				}

				// Since the user may have selected a different graph, reset the 
				// animate plot dialog to reflect the status of this graph!
				if(capDlg->GetSafeHwnd())
					capDlg->PostMessage(WM_ANIMATE_UPDATE);

				// Redraw the status bar
				CString str;

				if(pStatus){
					float data_x = pDoc->graph[pDoc->selected_graph].
						ScreentoDataX(point.x, extent);
					float data_y = pDoc->graph[pDoc->selected_graph].
						ScreentoDataY(point.y, extent);

					str.Format("X = %f, Y = %f", data_x, data_y);

					pStatus->SetPaneText(XY_PANE, str);

					str.Format("G%d", pDoc->selected_graph);

					pStatus->SetPaneText(GRAPH_PANE, str);
					
					// By holding down the control key, the user can identify plots by
					// selecting a point with the mouse.
					if(nFlags & MK_CONTROL){
						// Which plot did the user click on (or near?)
						float f_x, f_y;
						int plot_id = pDoc->graph[pDoc->selected_graph].
							PlotHitTest(point.x, point.y, f_x, f_y, extent);

						if(plot_id >= 0){
							str.Format("Set %d: point (%g, %g)", 
								plot_id, f_x, f_y);
							pStatus->SetPaneText(INFO_PANE, str);
						}
					}
				}
			}
	};

	CView::OnLButtonDown(nFlags, point);
}

void CSgraphView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(pointerState == SG_POINTER_ZOOM /*isZooming*/){
		pointerState = SG_POINTER_FREE;

		//::SetCursor(m_hcurArrow);
		::SetCursor(m_hcurCurrent);
		
		// We're done zooming, so adjust the view
		zoomEnd = point;
		
		// Give up control of the Mouse now that we're finished zooming
		ReleaseCapture();

		if(zoomEnd != zoomStart){

			int left, right, top, bottom;
		
			if(zoomStart.x < zoomEnd.x){
				left = zoomStart.x;
				right = zoomEnd.x;
			}
			else{
				left = zoomEnd.x;
				right = zoomStart.x;
			}
		
			if(zoomStart.y < zoomEnd.y){
				top = zoomStart.y;
				bottom = zoomEnd.y;
			}
			else{
				top = zoomEnd.y;
				bottom = zoomStart.y;
			}
			
			// Make sure that the user has defined a rectangle with
			// non-zero volume
			if((top != bottom) && (left != right)){
				CSgraphDoc* pDoc = GetDocument();
				ASSERT_VALID(pDoc);

				pDoc->graph[pDoc->selected_graph].
					SetPlotRange(
						pDoc->graph[pDoc->selected_graph].
							ScreentoDataX(left, extent), 
						pDoc->graph[pDoc->selected_graph].
							ScreentoDataX(right, extent), 
						pDoc->graph[pDoc->selected_graph].
							ScreentoDataY(bottom, extent), 
						pDoc->graph[pDoc->selected_graph].
							ScreentoDataY(top, extent),
							TRUE /* Override default graph scaling */);

				ForceRedraw();
			}
		}
	}

	if(isMoving && isSelected){
		// Give up control of the Mouse now that we're finished moving
		ReleaseCapture();

		//::SetCursor(m_hcurArrow);
		::SetCursor(m_hcurCurrent);

		isMoving = FALSE;
		ForceRedraw();

		// The document has been modified
		CSgraphDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		pDoc->SetModifiedFlag();
	}

	CView::OnLButtonUp(nFlags, point);
}

void CSgraphView::OnMButtonDown(UINT nFlags, CPoint point) 
{
	// Don't do anything if the user is not holding down
	// the control key or if the mouse is in another mode (like zooming
	// or selecting).
	if(!(nFlags & MK_MBUTTON) || !(nFlags & MK_CONTROL) || (pointerState != SG_POINTER_FREE)){
		
		// Let users know that the interface has changed
		if(nFlags & MK_MBUTTON){
			CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
			CStatusBar* pStatus = &pFrame->m_wndStatusBar;

			// Display a message in the info pane
			if(pStatus){
				pStatus->SetPaneText(INFO_PANE, "Ctrl key + middle button to scroll");
			}
		}

		CView::OnMButtonDown(nFlags, point);
		return;
	}
	
	scrollStart = scrollEnd = point;

	// Grab control of the Mouse for the duration of the scroll.
	SetCapture();

	// Now that the user has clicked the left mouse button,
	// we can zoom.
	pointerState = SG_POINTER_SCROLL;

	::SetCursor(m_hcurMove);

	CView::OnMButtonDown(nFlags, point);
}

void CSgraphView::OnMButtonUp(UINT nFlags, CPoint point) 
{

	if(pointerState != SG_POINTER_FREE){
		// Restore the mouse and pointer state to the pre-zoom
		// configuration
		ReleaseCapture();

		// Now that the user has clicked the left mouse button,
		// we can zoom.
		pointerState = SG_POINTER_FREE;

		::SetCursor(m_hcurCurrent);
	}

	CView::OnMButtonUp(nFlags, point);
}

BOOL CSgraphView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	return TRUE;
	
	//return CView::OnSetCursor(pWnd, nHitTest, message);
}

void CSgraphView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Has any annotated text been selected?
	for(int i = 0;i < pDoc->annotate_text.Length();i++)
		if(pDoc->annotate_text[i].isSelected){
			pDoc->annotate_text[i].ModalSet("Edit text");

			ForceRedraw();
			CView::OnLButtonDblClk(nFlags, point);
			return;
		}

	// There is a better way to do this selection process, 
	// but for now test all of the elements of the selected graph.
	SGraph *graph = &(pDoc->graph[pDoc->selected_graph]);

	if(graph->legend.isSelected){
		pDoc->OnPlotLegend();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->title.isSelected){
		pDoc->OnPlotTitle();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->leftAxisLable.isSelected){
		pDoc->OnPlotYAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->bottomAxisLable.isSelected){
		pDoc->OnPlotXAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->bottomAxis.isSelected){
		OnGraphXAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->topAxis.isSelected){
		OnGraphXAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->leftAxis.isSelected){
		OnGraphYAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	if(graph->rightAxis.isSelected){
		OnGraphYAxis();
		CView::OnLButtonDblClk(nFlags, point);
		return;
	}

	// Has the graph itself been selected?
	CDC *pDC = GetDC();

	if(graph->SObject::Select(point, pDC, extent)){
		if(pDoc->OnPlotOptions() && (capDlg->GetSafeHwnd() == 0)){
			// Bring up the animate plot dialog
			capDlg->Create();
		}
	}

	ReleaseDC(pDC);
	CView::OnLButtonDblClk(nFlags, point);
}

void CSgraphView::OnPlotArrange() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CArrangeGraphDialog agDlg(pDoc->width, pDoc->height, 
							  pDoc->startX, pDoc->startY,
							  pDoc->verticalGap, pDoc->horizontalGap, 
							  pDoc->rows, pDoc->cols);

	if(agDlg.DoModal() == IDOK){
		int tmpRows, tmpCols;
		float tmpWidth, tmpHeight, tmpStartX, tmpStartY;
		float tmpVerticalGap, tmpHorizontalGap;

		// CArrangeGraphDialog performs its own error checking and
		// reporting, so just return if there is an error.
		if(!agDlg.GetData(tmpWidth, tmpHeight, tmpStartX, tmpStartY,
			tmpVerticalGap, tmpHorizontalGap, tmpRows, tmpCols))
			return;

		if(pDoc->rows*pDoc->cols > tmpRows*tmpCols){
			// Make sure that the user realizes that this will
			// throw away any current graphs
			if(pDoc->IsModified() && 
				(AfxMessageBox("Delete excess graph(s)?", MB_YESNO) == IDNO))
				return;
		}

		// Make sure values are proberly bounded
		if((tmpWidth < 0.0f) || (tmpWidth > 1.0f)){
			AfxMessageBox("\"Graph width\" is out of bounds"
				" (require 1.0 >= width > 0.0). Not changes made.");
			return;
		}
		
		pDoc->width = tmpWidth;

		if((tmpHeight < 0.0f) || (tmpHeight > 1.0f)){
			AfxMessageBox("\"Graph height\" is out of bounds"
				" (require 1.0 >= height > 0.0). Not changes made.");
			return;
		}
		
		pDoc->height = tmpHeight;

		if((tmpStartX < 0.0f) || (tmpStartX > 1.0f)){
			AfxMessageBox("\"Start at x\" is out of bounds"
				" (require 1.0 >= x > 0.0). Not changes made.");
			return;
		}

		pDoc->startX = tmpStartX;

		if((tmpStartY < 0.0f) || (tmpStartY > 1.0f)){
			AfxMessageBox("\"Start at y\" is out of bounds"
				" (require 1.0 >= y > 0.0). Not changes made.");
			return;
		}
		
		pDoc->startY = tmpStartY;

		// Indexing variables
		int i, j;

		if(agDlg.apply_to_all == FALSE){
			// Apply results to only the selected graph
			i = pDoc->selected_graph;

			pDoc->graph[i].x1 = pDoc->startX;
			pDoc->graph[i].x2 = pDoc->startX + pDoc->width;
			pDoc->graph[i].y1 = pDoc->startY;
			pDoc->graph[i].y2 = pDoc->startY + pDoc->height;
			
			pDoc->graph[i].x = pDoc->startX + 0.5f*pDoc->width;
			pDoc->graph[i].y = pDoc->startY + 0.5f*pDoc->height;

			pDoc->graph[i].SetBoundingBox();
				
			pDoc->graph[i].title.x = pDoc->graph[i].x;
			pDoc->graph[i].title.y = pDoc->graph[i].y2 + 0.1f;
				
			pDoc->graph[i].leftAxisLable.x = pDoc->graph[i].x1 - 0.1f;
			pDoc->graph[i].leftAxisLable.y = pDoc->graph[i].y;

			pDoc->graph[i].bottomAxisLable.x = pDoc->graph[i].x;
			pDoc->graph[i].bottomAxisLable.y = pDoc->graph[i].y1 - 0.1f;
				
			pDoc->graph[i].leftAxis.x = pDoc->graph[i].x1;
			pDoc->graph[i].leftAxis.y = pDoc->graph[i].y;
			pDoc->graph[i].leftAxis.line_length = pDoc->height;

			pDoc->graph[i].rightAxis.x = pDoc->graph[i].x2;
			pDoc->graph[i].rightAxis.y = pDoc->graph[i].y;
			pDoc->graph[i].rightAxis.line_length = pDoc->height;

			pDoc->graph[i].topAxis.x = pDoc->graph[i].x;
			pDoc->graph[i].topAxis.y = pDoc->graph[i].y2;
			pDoc->graph[i].topAxis.line_length = pDoc->width;

			pDoc->graph[i].bottomAxis.x = pDoc->graph[i].x;
			pDoc->graph[i].bottomAxis.y = pDoc->graph[i].y1;
			pDoc->graph[i].bottomAxis.line_length = pDoc->width;

			pDoc->graph[i].legend.x = pDoc->graph[i].x;
			pDoc->graph[i].legend.y = pDoc->graph[i].y;
	
			// Redraw
			pDoc->SetModifiedFlag();
			ForceRedraw();

			return;
		}
		
		if((tmpVerticalGap < 0.0f) || (tmpVerticalGap > 1.0)){
			AfxMessageBox("\"Vertical gap\" is out of bounds"
				" (require 1.0 >= gap > 0.0). Not changes made.");
			return;
		}

		pDoc->verticalGap = tmpVerticalGap;

		if((tmpHorizontalGap < 0.0f) || (tmpHorizontalGap > 1.0)){
			AfxMessageBox("\"Horizontal gap\" is out of bounds"
				" (require 1.0 >= gap > 0.0). Not changes made.");
			return;
		}

		pDoc->horizontalGap = tmpHorizontalGap;

		if((pDoc->rows != tmpRows) || (pDoc->cols != tmpCols)){
			SGraph* old_graph = pDoc->graph;

			pDoc->num_graph = tmpRows*tmpCols;
			pDoc->graph = new SGraph[pDoc->num_graph];

			// Avoid name space collision problems with the min and max macros
			// int min_num_graph = min(pDoc->num_graph, pDoc->rows*pDoc->cols);

			int min_num_graph = pDoc->rows*pDoc->cols;
			min_num_graph = (min_num_graph > pDoc->num_graph) ? pDoc->num_graph : min_num_graph;

			// Copy any existing graphs into the new graph array
			for(i = 0;i < min_num_graph;i++)
				pDoc->graph[i] = old_graph[i];

			// Select G0
			pDoc->selected_graph = 0;
			pDoc->graph[pDoc->selected_graph].isSelected = TRUE;

			// Update the new number of rows and columns
			pDoc->rows = tmpRows;
			pDoc->cols = tmpCols;
			
			delete [] old_graph;
		}

		for(i = 0;i < pDoc->rows;i++)
			for(j = 0;j < pDoc->cols;j++){
				int index = i*pDoc->cols + j;

				pDoc->graph[index].x1 = pDoc->startX + (pDoc->width + pDoc->horizontalGap)*j;
				pDoc->graph[index].x2 = pDoc->graph[index].x1 +
					pDoc->width;
				pDoc->graph[index].y1 = pDoc->startY + (pDoc->height + pDoc->verticalGap)*i;
				pDoc->graph[index].y2 = pDoc->graph[index].y1 +
					pDoc->height;

				pDoc->graph[index].x = 0.5f*(pDoc->graph[index].x2 - pDoc->graph[index].x1) +
					pDoc->graph[index].x1;
				pDoc->graph[index].y = 0.5f*(pDoc->graph[index].y2 - pDoc->graph[index].y1) +
					pDoc->graph[index].y1;

				pDoc->graph[index].SetBoundingBox();
				
				pDoc->graph[index].title.x = pDoc->graph[index].x;
				pDoc->graph[index].title.y = pDoc->graph[index].y2 + 0.1f;
				
				pDoc->graph[index].leftAxisLable.x = pDoc->graph[index].x1 - 0.1f;
				pDoc->graph[index].leftAxisLable.y = pDoc->graph[index].y;

				pDoc->graph[index].bottomAxisLable.x = pDoc->graph[index].x;
				pDoc->graph[index].bottomAxisLable.y = pDoc->graph[index].y1 - 0.1f;
				
				pDoc->graph[index].leftAxis.x = pDoc->graph[index].x1;
				pDoc->graph[index].leftAxis.y = pDoc->graph[index].y;
				pDoc->graph[index].leftAxis.line_length = pDoc->height;

				pDoc->graph[index].rightAxis.x = pDoc->graph[index].x2;
				pDoc->graph[index].rightAxis.y = pDoc->graph[index].y;
				pDoc->graph[index].rightAxis.line_length = pDoc->height;

				pDoc->graph[index].topAxis.x = pDoc->graph[index].x;
				pDoc->graph[index].topAxis.y = pDoc->graph[index].y2;
				pDoc->graph[index].topAxis.line_length = pDoc->width;

				pDoc->graph[index].bottomAxis.x = pDoc->graph[index].x;
				pDoc->graph[index].bottomAxis.y = pDoc->graph[index].y1;
				pDoc->graph[index].bottomAxis.line_length = pDoc->width;

				pDoc->graph[index].legend.x = pDoc->graph[index].x;
				pDoc->graph[index].legend.y = pDoc->graph[index].y;
			}
		
		// Redraw
		pDoc->SetModifiedFlag();
		ForceRedraw();
	}
}

void CSgraphView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	InitDefaults();

	// Make sure the screen gets refreshed (even if the window is 
	// overlapped).
	ForceRedraw();

	CView::OnUpdate(pSender, lHint, pHint);
}

void CSgraphView::OnEditCopy() 
{
	// Save the current view to the windows clipboard
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CMetaFileDC m_pMetaDC;
	CBitmap m_bitmap;
	CDC bitmapDC;

	// Get the current (screen) device context
	CDC *pDC = GetDC();

	// Prepare the bitmap representation
	bitmapDC.CreateCompatibleDC(pDC);
	m_bitmap.CreateCompatibleBitmap(pDC, extent.Width(), extent.Height());
	bitmapDC.SelectObject(&m_bitmap);

	// Prepare the metafile representation
	m_pMetaDC.CreateEnhanced(pDC, NULL, NULL, "prestoplot\0doc\0\0");
	m_pMetaDC.SetAttribDC(pDC->GetSafeHdc());

	// Draw to the device context.
	// The isPrinting flag disables the graph selection boxes
	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].Printing(TRUE);

	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].Printing(TRUE);

	BOOL doubleBufferStatus = doubleBuffer;
	doubleBuffer = FALSE;

	// Render the bitmap using an explicit background color
	render_background_color = TRUE;

	OnDraw(&bitmapDC);

	render_background_color = FALSE;

	// Render the metafile
	OnDraw(&m_pMetaDC);

	doubleBuffer = doubleBufferStatus;

	for(i = 0;i < pDoc->annotate_text.Length();i++)
		pDoc->annotate_text[i].Printing(FALSE);

	for(i = 0;i < pDoc->num_graph;i++)
		pDoc->graph[i].Printing(FALSE);

	m_pMetaDC.ReleaseAttribDC();

	//close meta file dc and prepare for clipboard;
	HENHMETAFILE hMF = m_pMetaDC.CloseEnhanced();

	//copy to clipboard
	if(OpenClipboard()){
		EmptyClipboard();
		::SetClipboardData(CF_BITMAP, m_bitmap.GetSafeHandle());
		::SetClipboardData(CF_ENHMETAFILE, hMF);
		CloseClipboard();
	}
	else{
		AfxMessageBox("Can't open the clipboard (it's already open in "
			"another application).");
	}

	// Woops! DON'T delete the metafile handle, since it is now owned
	// by the windows clipboard!
	// Clean up the metafile
	//::DeleteEnhMetaFile(hMF);

	// We no longer own the bitmap
	m_bitmap.Detach();

	ReleaseDC(pDC);

	::SetCursor(prevCursor);
}

UINT CSgraphView::GetClipFormat()
{
	// Note that the clipboard must be open before calling this function
	UINT clipFormat = 0;

	while(clipFormat = ::EnumClipboardFormats(clipFormat)){
		if(clipFormat == CF_TEXT)
			return CF_TEXT;

		if(clipFormat == CF_OEMTEXT)
			return CF_OEMTEXT;
	}

	return 0;
}

void CSgraphView::OnEditPaste() 
{
	UINT clipFormat;

	// Open the clipboard
	OpenClipboard();
	
	// Did we find an allowed import format?
	if((clipFormat = GetClipFormat()) != 0){
		HANDLE clipData = NULL;
		
		clipData = GetClipboardData(clipFormat);

		if(clipData == NULL){
			AfxMessageBox("Can't get valid clipboard handle!");
			CloseClipboard();
			return;
		}
		
		// Bring up a dialog to allow the user to select the import format
		CPasteDialog pDlg;
		
		if(pDlg.DoModal() != IDOK){
			CloseClipboard();
			return;
		}

		CSgraphDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		// Do we need to lock memory from the clipboard?
		GlobalLock(clipData);
		
		HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

		switch(pDoc->graph[pDoc->selected_graph].ImportPlot(clipData, pDlg.GetFormat())){
			case S_ERROR_NO_DATA:
				::SetCursor(prevCursor);
				AfxMessageBox("No data found in the clipboard");
				break;
			case S_ERROR_IO:
				::SetCursor(prevCursor);
				AfxMessageBox("IO error reading from the clipboard");
				break;
			case S_ERROR_IO_BUFFER:
				::SetCursor(prevCursor);
				AfxMessageBox("Buffer overflow (line too long) reading from the clipboard");
				break;
			case S_ERROR_MEM_ALLOC:
				::SetCursor(prevCursor);
				AfxMessageBox("Can't allocate memory to store data while "
					"reading from the clipboard");
				break;
			case S_ERROR_IO_NUM_COL:
				::SetCursor(prevCursor);
				AfxMessageBox("Missing columns in the clipboard data");
				break;
			case S_ERROR_IO_FLOAT:
				::SetCursor(prevCursor);
				// This error results from trying to store all possible
				// floating point number as type float (instead of type
				// double or even long double). We could use type double, 
				// but this would restrict the amount of data that could 
				// be stored in memory.
				AfxMessageBox("Can not convert numbers in the clipboard"
					" to float (4 bytes per value)");
				break;
			default: // No error

			// Make sure that we're not going to attempt to
			// compute the log of a number that is <= 0.
			if((pDoc->graph[pDoc->selected_graph].x_scaleType == SCALE_LOG) || 
			   (pDoc->graph[pDoc->selected_graph].y_scaleType == SCALE_LOG)){
							
				BOOL x_log_ok = TRUE, y_log_ok = TRUE;

				for(int i = 0;i < pDoc->graph[pDoc->selected_graph].plot.Length();i++){

					if(!pDoc->graph[pDoc->selected_graph].plot[i].canXLog)
						x_log_ok = FALSE;

					if(!pDoc->graph[pDoc->selected_graph].plot[i].canYLog)
						y_log_ok = FALSE;
				}
						
				// Tell the user what has happened.
				if((!x_log_ok && (pDoc->graph[pDoc->selected_graph].x_scaleType == SCALE_LOG)) || 
				   (!y_log_ok && (pDoc->graph[pDoc->selected_graph].y_scaleType == SCALE_LOG)))
					AfxMessageBox("Can't compute log(n<=0) in the clipboard data"
						"; Switching to a linear scale.");

				// Change the scale type(s) to allow all of the
				// data to be displayed
				if(!x_log_ok)
					pDoc->graph[pDoc->selected_graph].x_scaleType = SCALE_LINEAR;
						
				if(!y_log_ok)
					pDoc->graph[pDoc->selected_graph].y_scaleType = SCALE_LINEAR;
			}

			pDoc->graph[pDoc->selected_graph].SetPlotScaleType();
			pDoc->graph[pDoc->selected_graph].SetPlotRange();

			pDoc->SetModifiedFlag();

			// Update the view
			ForceRedraw();
		}

		::SetCursor(prevCursor);

		GlobalUnlock(clipData);
	}
	else{
		AfxMessageBox("Sorry -- only ASCII text can be pasted!");
	}

	// Close the clipboard
	CloseClipboard();
}

void CSgraphView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	// Open the clipboard
	OpenClipboard();

	// Does the clipboard contain a valid format?
	pCmdUI->Enable(GetClipFormat() != 0);

	// Close the clipboard
	CloseClipboard();
}

void CSgraphView::OnDataStat() 
{
	// Throw up a dialog to allow the user to select a plot(s)
	// to compute statistics for
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CPlotStatisticsDialog psDlg(pDoc);

	psDlg.DoModal();
}

void CSgraphView::OnViewCrosshair() 
{
	isCrossHair = !isCrossHair;
	
	// By setting the x value to a negative number, we
	// allow the code to test for a previously existing 
	// cross hair on the screen.
	lastCross.x = -1;

	ForceRedraw();
}

void CSgraphView::OnUpdateViewCrosshair(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(isCrossHair);
}

void CSgraphView::OnDataHist() 
{
	// Throw up a dialog to allow the user to select a plot(s)
	// to compute statistics for
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CHistogramDialog histDlg(pDoc);

	if(histDlg.DoModal() == IDOK){
		ForceRedraw();
		pDoc->SetModifiedFlag();
	}
}

// Construct a dialog box to allow the user to adjust X Axis
// properties.
void CSgraphView::OnGraphXAxis() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CAxisDialog axisDlg(pDoc, CAxisDialog::XAxis);

	axisDlg.DoModal();
}

// Construct a dialog box to allow the user to adjust Y Axis
// properties.
void CSgraphView::OnGraphYAxis() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CAxisDialog axisDlg(pDoc, CAxisDialog::YAxis);

	axisDlg.DoModal();
}

void CSgraphView::OnPlotScaling() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	float minX, minY, maxX, maxY;

	// Get the plot range of the currently selected graph to use as
	// defaults for the scaling dialog
	pDoc->graph[pDoc->selected_graph].GetPlotRange(minX, maxX, minY, maxY);

	CScaleDialog sDlg(maxX, minX, maxY, minY);
	
	if(sDlg.DoModal() == IDOK){

		maxX = sDlg.GetXMax();
		maxY = sDlg.GetYMax();
		minX = sDlg.GetXMin();
		minY = sDlg.GetYMin();

		switch(sDlg.applyTo){
			case CScaleDialog::APPLY_TO_ALL_AXES_CUR_GRAPH:
				if(sDlg.isDefault){
					pDoc->graph[pDoc->selected_graph].SetDefaultPlotRange();
					pDoc->graph[pDoc->selected_graph].SetPlotRange();
				}
				else{
					pDoc->graph[pDoc->selected_graph].
						SetDefaultPlotRange(minX, maxX, minY, maxY);
					pDoc->graph[pDoc->selected_graph].
						SetPlotRange(minX, maxX, minY, maxY);
				}
				break;
			case CScaleDialog::APPLY_TO_ALL_AXES_ALL_GRAPHS:
				if(sDlg.isDefault){
					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRange();
						pDoc->graph[i].SetPlotRange();
					}
				}
				else{
					// Check to make sure that we're not about to try and
					// take the log of a negative number
					if((minX <= 0.0f) || (maxX <= 0.0f) ||
					   (minY <= 0.0f) || (maxY <= 0.0f)){
						
						for(int i = 0;i < pDoc->num_graph;i++)
							if((pDoc->graph[i].x_scaleType == SCALE_LOG) || 
								(pDoc->graph[i].y_scaleType == SCALE_LOG)){
								AfxMessageBox("A negative axis range can not be used with\n\r"
									"a log scale.");
								return;
							}
					}

					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRange(minX, maxX, minY, maxY);
						pDoc->graph[i].SetPlotRange(minX, maxX, minY, maxY);
					}
				}
				break;
			case CScaleDialog::APPLY_TO_X_AXES_ALL_GRAPHS:
				if(sDlg.isDefault){
					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRangeX();
						pDoc->graph[i].SetPlotRangeX();
					}
				}
				else{
					// Check to make sure that we're not about to try and
					// take the log of a negative number
					if((minX <= 0.0f) || (maxX <= 0.0f)){
						
						for(int i = 0;i < pDoc->num_graph;i++)
							if(pDoc->graph[i].x_scaleType == SCALE_LOG){
								AfxMessageBox("A negative X axis range can not be used with\n\r"
									"a log scale.");
								return;
							}
					}

					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRangeX(minX, maxX);
						pDoc->graph[i].SetPlotRangeX(minX, maxX);
					}
				}
				break;
			case CScaleDialog::APPLY_TO_Y_AXES_ALL_GRAPHS:
				if(sDlg.isDefault){
					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRangeY();
						pDoc->graph[i].SetPlotRangeY();
					}
				}
				else{
					// Check to make sure that we're not about to try and
					// take the log of a negative number
					if((minY <= 0.0f) || (maxY <= 0.0f)){
						
						for(int i = 0;i < pDoc->num_graph;i++)
							if(pDoc->graph[i].y_scaleType == SCALE_LOG){
								AfxMessageBox("A negative Y axis range can not be used with\n\r"
									"a log scale.");
								return;
							}
					}

					for(int i = 0;i < pDoc->num_graph;i++){
						pDoc->graph[i].SetDefaultPlotRangeY(minY, maxY);
						pDoc->graph[i].SetPlotRangeY(minY, maxY);
					}
				}
				break;
		};

		// Update the view
		pDoc->SetModifiedFlag();
		ForceRedraw();
	}
}

void CSgraphView::OnFileDataExport() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->graph[pDoc->selected_graph].ExportPlot();
}

void CSgraphView::OnFileExportAudio() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CAudioDialog aDlg(pDoc);

	aDlg.DoModal();
}

void CSgraphView::OnDataDeriv() 
{
	// Throw up a dialog to allow the user to select a plot(s)
	// to compute statistics for
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CDerivativeDialog derivDlg(pDoc);

	if(derivDlg.DoModal() == IDOK){
		ForceRedraw();
		pDoc->SetModifiedFlag();
	}
}

void CSgraphView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// Add a hopefully non-gratuitus popup menu

	// First make sure that the user is not performing an operation
	// with the left mouse button (or even pressing another button).
	if((pointerState != SG_POINTER_FREE) || isMoving || (nFlags != MK_RBUTTON)){
		CView::OnRButtonDown(nFlags, point);
		return;
	}

	CMenu menu; 
	VERIFY(menu.LoadMenu(IDR_POPUP_MENU));

	CMenu* pPopup = menu.GetSubMenu(0); 

	ASSERT(pPopup != NULL); 
	
	CPoint screen_point = point;

	// Convert point to screen coordiantes
	ClientToScreen(&screen_point);

	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		screen_point.x, screen_point.y, 
		AfxGetMainWnd());
	
	CView::OnRButtonDown(nFlags, point);
}

void CSgraphView::OnPlotUserText() 
{
	pointerState = SG_POINTER_TEXT;
	::SetCursor(m_hcurIBeam);
}

void CSgraphView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	switch(nChar){
		// Allow the user to cancel a pipe read operations
		case 27: // The 'Esc' key
			if(pDoc->m_pipe && !pDoc->haltPipe && 
			  (AfxMessageBox("Halt reading from data pipe?", MB_YESNO) == IDYES))
				pDoc->haltPipe = TRUE;
			break;
		// Allow the user to use the arrow keys on the keyboard to
		// shift the x and y axis (for linear scales only).
		case 37: // Left arrow
			pDoc->graph[pDoc->selected_graph].ShiftX(-0.1f);
			ForceRedraw();
			break;
		case 38: // Up arrow
			pDoc->graph[pDoc->selected_graph].ShiftY(0.1f);
			ForceRedraw();
			break;
		case 39: // Right arrow
			pDoc->graph[pDoc->selected_graph].ShiftX(0.1f);
			ForceRedraw();
			break;
		case 40: // Down arrow
			pDoc->graph[pDoc->selected_graph].ShiftY(-0.1f);
			ForceRedraw();
			break;
	};
	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CSgraphView::OnEditClear() 
{
	if(isSelected){
		isSelected->OnDelete();
		
		// Before setting the isSelected pointer to NULL, we
		// should check to see if it points to user added text (and
		// delete that text from the annotated_text list).

		// The object is no longer selected
		isSelected = NULL;

		ForceRedraw();
	}
}

void CSgraphView::OnUpdateEditClear(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(isSelected != NULL);
}


void CSgraphView::OnViewDoublebuffer() 
{
	doubleBuffer = !doubleBuffer;
}

void CSgraphView::OnUpdateViewDoublebuffer(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(doubleBuffer);
}

void CSgraphView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);
	
	// If we lose focus while performing an operation with the
	// mouse, stop the operation!
	if(pointerState != SG_POINTER_FREE){
		pointerState = SG_POINTER_FREE;
		::SetCursor(m_hcurCurrent);
	}
}

void CSgraphView::OnPlotGraphColor() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CPrestoColor pcDlg(pDoc->graph[pDoc->selected_graph].background_color);
	
	if(pcDlg.DoModal() == IDOK){
		pDoc->graph[pDoc->selected_graph].background_color = pcDlg.GetColor();
		pDoc->SetModifiedFlag();
		ForceRedraw();
	}
}

void CSgraphView::OnPlotGraphSelect() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CSelectGraphDialog sgDlg(pDoc->selected_graph, pDoc->num_graph);

	if(sgDlg.DoModal() == IDOK){
		isSelected = NULL;
		CPoint null_point;

		null_point.x = -1;
		null_point.y = -1;

		// Deselect all of the objects
		SelectObject(null_point);

		for(int i = 0;i < pDoc->num_graph;i++)
			pDoc->graph[i].isSelected = FALSE;
		
		pDoc->selected_graph = sgDlg.current;

		pDoc->graph[pDoc->selected_graph].isSelected = TRUE;

		ForceRedraw();
	}
}

void CSgraphView::OnDataIntegrate() 
{
	// Throw up a dialog to allow the user to select a plot(s)
	// to compute statistics for
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CIntegrateDialog intergrateDlg(pDoc);

	intergrateDlg.DoModal();
}

void CSgraphView::OnAnimatePlot() 
{
	if(capDlg->GetSafeHwnd() == 0){
		capDlg->Create();
	}
}

void CSgraphView::OnUpdateAnimatePlot(CCmdUI* pCmdUI) 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->graph[pDoc->selected_graph].animate_plots);
}

LRESULT CSgraphView::OnAnimatePlotClose(WPARAM wParam, LPARAM lParam)
{
	if(capDlg->GetSafeHwnd()){
		capDlg->DestroyWindow();
	}

	return 0L;
}

LRESULT CSgraphView::OnAnimatePlotAdjust(WPARAM wParam, LPARAM lParam)
{
	if(capDlg->GetSafeHwnd()){
		CSgraphDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		
		if(pDoc->graph[pDoc->selected_graph].animate_plots){
			pDoc->graph[pDoc->selected_graph].curr_animate_plot = 
				capDlg->GetCurrPlot();

			ForceRedraw();
		}
	}

	return 0L;
}

void CSgraphView::OnViewRefresh() 
{
	ForceRedraw();
	//flushBuffer = TRUE;
	//PostMessage(WM_ERASEBKGND);
	//PostMessage(WM_PAINT);
}

UINT PipeWrapper(LPVOID lparam)
{
	((CSgraphDoc *)lparam)->ListenToPipe();

	return 0;
}

LRESULT CSgraphView::StartPipe(WPARAM wParam, LPARAM lParam)
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	m_hcurCurrent = m_hcurPipe;

	::SetCursor(m_hcurCurrent);

	AfxBeginThread(PipeWrapper, pDoc);

	return 0L;
}

void CSgraphView::StopPipe()
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	m_hcurCurrent = m_hcurArrow;

	::SetCursor(m_hcurArrow);
	
	// If the user has manually shut down the pipe, make sure that the
	// thread stops reading.
	pDoc->haltPipe = TRUE;

	flushBuffer = TRUE;

	// Note that we must use RedrawWindow() and NOT ForceRedraw() to update the
	// graph view. Why? Because if PrestoPlot is in the background and totally
	// obscured by other windows, the InvalidateRect() that gets called as part of
	// ForceRedraw() will not erase the background.
	RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_ERASENOW | RDW_ERASE);
}

void CSgraphView::OnPlotLines() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CGraphLineDialog glDlg(&(pDoc->graph[pDoc->selected_graph]));
	
	glDlg.DoModal();

	// Update the view regardless of the outcome of the line dialog
	ForceRedraw();
}

LRESULT CSgraphView::SetWYSIWYGAspectRatio(WPARAM wParam, LPARAM lParam)
{
	CPrintDialog printDlg(FALSE);	
	CDC printerDC;

	if(!AfxGetApp()->GetPrinterDeviceDefaults(&(printDlg.m_pd))){
		// No default printer installed!
		AfxMessageBox("Please install a default printer to use WYSIWYG mode");

		isWYSIWYG = FALSE;
		aspectWYSIWYG = 1.0f;

		return 0L;
	}

	// Creates a printer DC from the DEVMODE and DEVNAMES structures.
	if(printDlg.CreatePrinterDC() == NULL){
		AfxMessageBox("Unable to load printer dimensions for WYSIWYG mode");

		isWYSIWYG = FALSE;
		aspectWYSIWYG = 1.0f;

		return 0L;
	}

	printerDC.Attach(printDlg.GetPrinterDC());

	// Compute the printer aspect ratio
	int width = printerDC.GetDeviceCaps(HORZRES);
	int height = printerDC.GetDeviceCaps(VERTRES);

	if(width == 0){
		AfxMessageBox("Unable to set aspect ratio for WYSIWYG mode");
		
		isWYSIWYG = FALSE;
		aspectWYSIWYG = 1.0f;
	}
	else{
		aspectWYSIWYG = (float)(height)/width;
	}

	// Don't call GlobalFree here (despite the Windows documentation). I most
	// likely have a memory leak here, but calling GlobalFree on these member
	// variables of the print dialog cause a crash...
	//GlobalFree(printDlg.m_pd.hDevMode);
	//GlobalFree(printDlg.m_pd.hDevNames);

	// Set the frame aspect ratio
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
	if(pFrame){
		if(isWYSIWYG){
			pFrame->EnableWYSIWYG(aspectWYSIWYG);
		}
		else
			pFrame->DisableWYSIWYG();
	}

	return 0L;
}

void CSgraphView::OnViewWysiwyg() 
{
	isWYSIWYG = !isWYSIWYG;

	SetWYSIWYGAspectRatio();	
}

void CSgraphView::OnUpdateViewWysiwyg(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(isWYSIWYG);
}

void CSgraphView::OnFileExportAnimation() 
{
	CMovieDialog movieDlg(this);

	movieDlg.DoModal();
}

void CSgraphView::OnViewProperties() 
{
	CPropertiesDialog propDlg(SObject::lineScale, SObject::fontScale);

	BOOL force_redraw = FALSE;

	if(propDlg.DoModal() == IDOK){
		if(SObject::lineScale != propDlg.line_thickness_scale){
			if((SObject::lineScale == 1) && is_16_bit){
				if(AfxMessageBox("Please note that if lines are scaled by more "
					"than 1,\r\nall lines will appear solid. Continue?", 
					MB_YESNO) == IDNO){
					return;
				}
			}

			SObject::lineScale = propDlg.line_thickness_scale;
			
			// Save the global line scaling for later Serialization if needed
			CSgraphDoc* pDoc = GetDocument();
			ASSERT_VALID(pDoc);
			
			pDoc->doc_pref.SetLineScaling(SObject::lineScale);

			force_redraw = TRUE;
		}

		if(SObject::fontScale != propDlg.font_size_scale){

			SObject::fontScale = propDlg.font_size_scale;
			
			// Save the global line scaling for later Serialization if needed
			CSgraphDoc* pDoc = GetDocument();
			ASSERT_VALID(pDoc);
			
			pDoc->doc_pref.SetFontScaling(SObject::fontScale);

			force_redraw = TRUE;
		}

		if(force_redraw)
			ForceRedraw();
	}

}

void CSgraphView::OnEditCopyData() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->graph[pDoc->selected_graph].CopyPlot();
}


BOOL CSgraphView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	// To prevent accidental zooming, force the user to hold down the control
	// key to activate the wheel mouse zoom feature
	if(!(nFlags & MK_CONTROL)){
		CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;
		CStatusBar* pStatus = &pFrame->m_wndStatusBar;

		// Display a message in the info pane
		if(pStatus){
			pStatus->SetPaneText(INFO_PANE, "Hold down the Ctrl key + wheel mouse to zoom");
		}

		// Exit WITHOUT zooming!
		return CView::OnMouseWheel(nFlags, zDelta, pt);
	}

	// Allow the user to zoom the current graph using the wheel mouse
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// If we're zooming, we're not moving or selecting!
	if(isSelected || isMoving || (pointerState != SG_POINTER_FREE)){
		::SetCursor(m_hcurCurrent);
		pointerState = SG_POINTER_FREE;

		isSelected = NULL;
		isMoving = FALSE;

		ReleaseCapture();
	}

	float minX, maxX, minY, maxY;
	float dx, dy;
	pDoc->graph[pDoc->selected_graph].GetPlotRange(minX, maxX, minY, maxY);

	// zDelta is in increments of 120 (i.e. how far has the user rolled the wheel?).
	// dx and dy are set to 10% of the axis length times the number of wheel
	// "clicks".
	dx = (maxX - minX)*zDelta/1200.0f;
	dy = (maxY - minY)*zDelta/1200.0f;

	if(pDoc->graph[pDoc->selected_graph].x_scaleType == SCALE_LINEAR){
		// Only scale linear axes
		minX += dx;
		maxX -= dx;
	}

	if(pDoc->graph[pDoc->selected_graph].y_scaleType == SCALE_LINEAR){
		// Only scale linear axes
		minY += dy;
		maxY -= dy;
	}

	pDoc->graph[pDoc->selected_graph].SetPlotRange(minX, maxX, minY, maxY, 
		TRUE /* Override default graph scaling */);

	ForceRedraw();
	
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CSgraphView::OnDataFunc() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CEquationDialog eqDlg(pDoc, eqHistory);
	
	if(eqDlg.DoModal() == IDOK){
		ForceRedraw();
		pDoc->SetModifiedFlag();
	}
}

void CSgraphView::OnDataParaFunc() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CParametricFunctionDialog paraEqDlg(pDoc, paraEqHistory);
	
	if(paraEqDlg.DoModal() == IDOK){
		ForceRedraw();
		pDoc->SetModifiedFlag();
	}
}

void CSgraphView::OnDataPolarFunc() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CPolarEquationDialog polarEqDlg(pDoc, polarEqHistory);
	
	if(polarEqDlg.DoModal() == IDOK){
		ForceRedraw();
		pDoc->SetModifiedFlag();
	}
}

void CSgraphView::OnDataNonLin() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CNonLinFitDialog nlFitDlg(this);
	
	if(nlFitDlg.DoModal() == IDOK){
		ForceRedraw();

		pDoc->SetModifiedFlag();
	}
}

void CSgraphView::OnPageColor() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CPrestoColor cpDlg(pDoc->background_color);

	if(cpDlg.DoModal() == IDOK){
		pDoc->background_color = cpDlg.GetColor();

		ForceRedraw();

		pDoc->SetModifiedFlag();
	}
}


void CSgraphView::OnFileExportImage() 
{
	// Open a file dialog to let the user specify a filename and file format
	CFileDialog fDlg(FALSE /* Save As*/);

	fDlg.m_ofn.lpstrTitle = "Export Image";
	fDlg.m_ofn.lpstrFilter = "Bitmap files (*.bmp)\0*.bmp\0"
							 "PNG files (*.png)\0*.png\0\0";
	fDlg.m_ofn.Flags |= OFN_OVERWRITEPROMPT;

	if(fDlg.DoModal() == IDOK){
		// What kind of file are we writing?
		switch(fDlg.m_ofn.nFilterIndex){
			case 1:
				ExportImageBMP(LPCSTR(fDlg.GetFileName()));
				break;
			case 2:
				ExportImagePNG(LPCSTR(fDlg.GetFileName()));
				break;
			default:
				AfxMessageBox("Unknown file format!");
				break;
		}
	}
	
}

void CSgraphView::OnPageOverlap() 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(pDoc->num_graph < 2){
		AfxMessageBox("Can't overlap less than two graphs!");
		return;
	}

	COverlapGraphDialog ogDlg(pDoc->num_graph);

	if(ogDlg.DoModal() == IDOK){
		pDoc->graph[ogDlg.topGraph].Overlap(pDoc->graph[ogDlg.bottomGraph]);

		// Update the view
		ForceRedraw();
	}
}

void CSgraphView::OnUpdatePageOverlap(CCmdUI* pCmdUI) 
{
	CSgraphDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->num_graph > 1);
}
