//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
#include "stdafx.h"
#include "slegend.h"
#include "color.h"

/////////////////////////////////////////////////////////////////////////////
//	*** SLegend ***
/////////////////////////////////////////////////////////////////////////////
SLegend::SLegend(list<SPlot> &m_plot)
{
	plot = &m_plot;

	LegendCushion = 4;

	isDrawn = FALSE;
	isBoundingBox = FALSE;

	x = 0.5;
	y = 0.5;

	fillColor = NO_COLOR;
	edgeColor = 0x00000000; // Black

	fontColor = 0x00000000; // Black

	// The default font
	font.lfHeight = DefaultLegendFontSize;

	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_NORMAL;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, DefaultFont );

}

void SLegend::Serialize(CArchive& ar)
{
	SObject::Serialize(ar);

	SerializeFont(font, ar);

	if(ar.IsStoring()){
		ar << isBoundingBox;
		ar << fillColor;
		ar << edgeColor;
		ar << fontColor;
	}
	else{
		ar >> isBoundingBox;
		ar >> fillColor;

		if(version > 1.22)
			ar >> edgeColor;
		else
			edgeColor = 0x00000000; // Black

		ar >> fontColor;
	}
}

void SLegend::Draw(CDC* pDC, CRect &extent)
{
	if(!isDrawn)
		return;

	// A DWORD buffer for drawing thick lines
	DWORD* dashBuffer = NULL;

	// Set the background drawing mode
	int save_bkg_mode = pDC->SetBkMode(TRANSPARENT);
	
	CBrush legend_brush;
	LOGBRUSH log_brush;

	// Set the symbol brush
	log_brush.lbColor = fillColor;

	if(log_brush.lbColor == NO_COLOR)
		log_brush.lbStyle = BS_HOLLOW;
	else
		log_brush.lbStyle = BS_SOLID;

	legend_brush.CreateBrushIndirect(&log_brush);

	// Make the current brush the point brush
	CBrush *oldbrush = pDC->SelectObject(&legend_brush); 

	// The legend box is a rectangle that is just wide enough
	// to fit the longest legend (text and symbol).
	// The symbol width is given by DefaultLegendSymbolWidth.
	int symbol_width = XtoScreen(DefaultLegendSymbolWidth, extent) - 
		XtoScreen(0.0, extent);
	CSize legend_size;
	int i;

	legend_size.cx = 0;
	legend_size.cy = 0;

	for(i = 0;i < plot->Length();i++)
		if((*plot)[i].isDrawn && ((*plot)[i].legend.Length() > 0)){
			CSize text_size = (*plot)[i].legend.GetExtent(pDC);

			if(text_size.cx > legend_size.cx)
				legend_size.cx = text_size.cx;

			legend_size.cy += text_size.cy + LegendCushion;
		}

	legend_size.cy += LegendCushion;
	legend_size.cx += symbol_width + 3*LegendCushion;

	// Scale the legend box to half width and half height
	legend_size.cx /= 2;
	legend_size.cy /= 2;

	// The center of the legend box (in screen coordiantes)
	int screen_x = XtoScreen(x, extent), 
		screen_y = YtoScreen(y, extent);

	if(isBoundingBox){
		CPen legendpen(PS_SOLID, 1*lineScale, edgeColor);
		CPen *oldpen = pDC->SelectObject(&legendpen); 

		// Draw a bounding box
		pDC->Rectangle(screen_x - legend_size.cx, 
			screen_y + legend_size.cy, screen_x + legend_size.cx,
			screen_y - legend_size.cy);

		pDC->SelectObject(oldpen); 
	}

	// Location of symbol (if present)
	int symbol_x = screen_x + legend_size.cx - symbol_width/2 - LegendCushion;

	// Start of line (if present)
	int line_x = screen_x + legend_size.cx - symbol_width - LegendCushion;

	int curr_y = screen_y - legend_size.cy;

	for(i = 0;i < plot->Length();i++){

		// Grab a reference to the current plot (for fast access).
		SPlot &plotRef = (*plot)[i];

		if(plotRef.isDrawn && (plotRef.legend.Length() > 0)){
			CSize text_size = plotRef.legend.GetExtent(pDC);
			
			int curr_x = screen_x - legend_size.cx + 
				LegendCushion + text_size.cx/2;

			curr_y += LegendCushion + text_size.cy/2;

			// Draw the text
			plotRef.legend.x = ScreentoX(curr_x, extent);
			plotRef.legend.y = ScreentoY(curr_y, extent);
			plotRef.legend.Draw(pDC, extent);
			
			// Draw the line (if present)
			if(plotRef.lineType != SPlot::LINE_NONE){
				CPen *plotpen = NULL;

				if((plotRef.lineType != SPlot::LINE_SOLID) && 
				   (plotRef.lineSize*lineScale != 1) && (!is_16_bit)){
					// Create the pen for drawing the lines. A few hoops must be jumped 
					// through in order to render thick, non-solid lines.
					LOGBRUSH penBrush;

					penBrush.lbStyle = BS_SOLID;
					penBrush.lbColor = plotRef.lineColor;

					int dashSize;
					
					if(dashBuffer)
						delete [] dashBuffer;

					dashBuffer = plotRef.GetLineStyle(plotRef.lineType, dashSize);

					// Use the extented form of the CPen constructor to enable the use
					// of the thick dashed lines.
					plotpen = new CPen(PS_GEOMETRIC | PS_ENDCAP_SQUARE | PS_JOIN_BEVEL | PS_USERSTYLE, 
						plotRef.lineSize*lineScale, &penBrush, dashSize, dashBuffer);
				}
				else{
					// Create the pen for drawing the lines. Use a plain old CPen for this
					// task (as it is quite fast!).
					plotpen = new CPen(plotRef.GetLineStyle(plotRef.lineType), 
						plotRef.lineSize*lineScale, plotRef.lineColor);
				}

				// Make the current pen the state pen
				CPen *oldplotpen = pDC->SelectObject(plotpen); 
				
				pDC->MoveTo(line_x, curr_y);
				pDC->LineTo(line_x + symbol_width, curr_y);

				pDC->SelectObject(oldplotpen); 

				if(plotpen)
					delete plotpen;
			}

			// Draw the symbol (if present)
			if((*plot)[i].pointType != SPlot::POINT_NONE){
				CPen plotpen(PS_SOLID, 1*lineScale, plotRef.lineColor);

				// Make the current pen the state pen
				CPen *oldplotpen = pDC->SelectObject(&plotpen); 

				// Create the brush for drawing the symbols
				CBrush pointbrush;
				pointbrush.CreateBrushIndirect(&(plotRef.brush));

				// Make the current brush the point brush
				CBrush *oldbrush = pDC->SelectObject(&pointbrush); 
				
				CPoint pointUpTriangle[3];

				switch((*plot)[i].pointType){
					case SPlot::POINT_CIRCLE:
						pDC->Ellipse(symbol_x - plotRef.pointSize, curr_y - plotRef.pointSize,
							 symbol_x + plotRef.pointSize + 1, curr_y + plotRef.pointSize + 1);
						break;
					case SPlot::POINT_SQUARE:
						pDC->Rectangle(symbol_x - plotRef.pointSize, curr_y - plotRef.pointSize,
							 symbol_x + plotRef.pointSize + 1, curr_y + plotRef.pointSize + 1);
						break;
					case SPlot::POINT_TRIANGLE:
						// The vertcies of the triangle (counter clockwise).
						pointUpTriangle[0].x = symbol_x;
						pointUpTriangle[0].y = (long)(curr_y - plotRef.pointSize);
					
						pointUpTriangle[1].x = (long)(symbol_x - plotRef.pointSize*0.866025f);
						pointUpTriangle[1].y = (long)(curr_y + plotRef.pointSize*0.5f);

						pointUpTriangle[2].x = (long)(symbol_x + plotRef.pointSize*0.866025f);
						pointUpTriangle[2].y = (long)(curr_y + plotRef.pointSize*0.5f);

						pDC->Polygon(pointUpTriangle, 3);
						break;
					case SPlot::POINT_DOT:
						// Draw the smallest circle we can
						pDC->Ellipse(symbol_x - 1, curr_y - 1, symbol_x + 1, curr_y + 1);
						break;
					case SPlot::POINT_HISTOGRAM_X:
					case SPlot::POINT_HISTOGRAM_Y:
						// Center the historgram "bar" with respect to the text
						pDC->Rectangle(line_x, curr_y - text_size.cy/2,
							 line_x + symbol_width + 1, curr_y + text_size.cy/2);
						break;
					case SPlot::POINT_VECTOR:
						break;
				};

				pDC->SelectObject(oldplotpen);
				pDC->SelectObject(oldbrush);
			}

			curr_y += text_size.cy/2;
		}
	}

	// Restore GDI state
	pDC->SetBkMode(save_bkg_mode);
	pDC->SelectObject(oldbrush); 

	if(dashBuffer)
		delete [] dashBuffer;

	if(isSelected && !isPrinting)
		DrawSelect(pDC, extent);
}

void SLegend::DrawSelect(CDC* pDC, CRect &extent)
{
	// Draw a dotted selection boxe around the text string
	// Select ROP2 mode
	int ropOld = pDC->SetROP2(R2_XORPEN);

	// Note - since these lines ONLY appear on the screen,
	// they do not have to be scaled by SObject::lineScale!
	CPen linepen(PS_DOT, 1, COLORREF (0) /* Black */);

	// Make the current pen the state pen
	CPen *oldpen = pDC->SelectObject(&linepen); 

	SelectRect(pDC, extent).Draw(pDC);

	// Restore the previous pen
	pDC->SelectObject(oldpen); 
	pDC->SetROP2(ropOld);
}

CSelectRect SLegend::SelectRect(CDC* pDC, CRect &extent)
{
	// Set the font and pen
	CFont tmp_font;
	tmp_font.CreateFontIndirect(&font);

	CFont* save_font = pDC->SelectObject(&tmp_font);

	CPen legendpen(PS_SOLID, 1*lineScale, COLORREF(0) /* Black */);
	CPen *oldpen = pDC->SelectObject(&legendpen); 

	// The legend box is a rectangle that is just wide enough
	// to fit the longest legend (text and symbol).
	// The symbol width is given by DefaultLegendSymbolWidth.
	int symbol_width = XtoScreen(DefaultLegendSymbolWidth, extent) - 
		XtoScreen(0.0, extent);
	CSize legend_size;
	int i;

	legend_size.cx = 0;
	legend_size.cy = 0;

	for(i = 0;i < plot->Length();i++)
		if((*plot)[i].isDrawn && ((*plot)[i].legend.Length() > 0)){
			//CSize text_size = pDC->GetTextExtent((*plot)[i].legend);
			CSize text_size = (*plot)[i].legend.GetExtent(pDC);
			if(text_size.cx > legend_size.cx)
				legend_size.cx = text_size.cx;

			legend_size.cy += text_size.cy + LegendCushion;
		}

	legend_size.cy += LegendCushion;
	legend_size.cx += symbol_width + 3*LegendCushion;

	// The bounding box
	CSelectRect select_rect;

	select_rect.Size(legend_size);
	select_rect.X(XtoScreen(x, extent));
	select_rect.Y(YtoScreen(y, extent));

	// Restore GDI state
	pDC->SelectObject(save_font);
	pDC->SelectObject(oldpen);

	return select_rect;
}

void SLegend::PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent)
{
	SObject::PreparePrinting(pDC, scaleMul, scaleDiv, extent);

	font.lfHeight = -MulDiv(-font.lfHeight, printScaleMul, printScaleDiv);
	LegendCushion = MulDiv(LegendCushion, printScaleMul, printScaleDiv);
}

void SLegend::CleanUpPrinting(CDC* pDC, CRect &extent)
{
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleDiv, printScaleMul);
	LegendCushion = MulDiv(LegendCushion, printScaleDiv, printScaleMul);

	SObject::CleanUpPrinting(pDC, extent);
}

void SLegend::OnDelete()
{
	// If the legend is deleted, don't throw away the legend entries,
	// just toggle the isDrawn and isSelected flags.
	isDrawn = FALSE;

	isSelected = FALSE;
}