//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

#include "stdafx.h"
#include "stextobject.h"
#include "TextEditDialog.h"


/////////////////////////////////////////////////////////////////////////////
//	*** STextObject ***
/////////////////////////////////////////////////////////////////////////////
STextObject::STextObject(CString m_text, float m_x, float m_y, 
						 int orientation)
{
	x = m_x;
	y = m_y;
	
	fontColor = 0x000000; // Black

	// The default font
	font.lfHeight = DefaultTextFontSize;

	font.lfWidth = 0;
	font.lfEscapement = orientation;
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_NORMAL;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, DefaultFont );

	SetText(m_text);
}

STextObject::STextObject()
{
	x = 0.0f;
	y = 0.0f;
	
	fontColor = 0x000000; // Black

	// The default font
	font.lfHeight = DefaultTextFontSize;

	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_NORMAL;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, DefaultFont );
}


void STextObject::PreparePrinting(CDC* pDC, long scaleMul, long scaleDiv, CRect &extent)
{
	SObject::PreparePrinting(pDC, scaleMul, scaleDiv, extent);
	
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleMul, printScaleDiv);

	label_text.SetPrintScaling(printScaleMul, printScaleDiv);
}

void STextObject::CleanUpPrinting(CDC* pDC, CRect &extent)
{	
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleDiv, printScaleMul);

	label_text.SetPrintScaling(1, 1);

	SObject::CleanUpPrinting(pDC, extent);
}

void STextObject::Draw(CDC* pDC, CRect &extent)
{	
	// Don't bother drawing zero length strings!
	if(!isDrawn || (text.GetLength() == 0))
		return;

	label_text.Draw(pDC, XtoScreen(x, extent), YtoScreen(y, extent));

	if(isSelected && !isPrinting)
		DrawSelect(pDC, extent);
}

CSelectRect STextObject::SelectRect(CDC* pDC, CRect &extent)
{
	// Empty string have no length
	if(text.GetLength() == 0)
		return CSelectRect();

	CSelectRect select_rect;

	select_rect.X(XtoScreen(x, extent));
 	select_rect.Y(YtoScreen(y, extent));
	select_rect.Angle(0.1*font.lfOrientation*TWOPI/360.0);

	CRect tmpRect = label_text.GetExtent(pDC);

	select_rect.SizeX(tmpRect.Width());
	select_rect.SizeY(tmpRect.Height());

	return select_rect;
}

void STextObject::DrawSelect(CDC* pDC, CRect &extent)
{
	// Draw a dotted selection box around the text string
	// Select ROP2 mode

	// Note that the mode must be set to R2_XORPEN, not R2_NOT,
	// to get a dotted rectangle (R2_NOT yeilds a solid rectangle
	// regardless of the pen type).
	int ropOld = pDC->SetROP2(R2_XORPEN);

	// Note - since these lines ONLY appear on the screen,
	// they do not have to be scaled by SObject::lineScale!
	CPen linepen(PS_DOT, 1, COLORREF (0) /* Black */);

	// Make the current pen the state pen
	CPen *oldpen = pDC->SelectObject(&linepen); 

	SelectRect(pDC, extent).Draw(pDC);

	// Restore the previous pen
	pDC->SelectObject(oldpen); 
	pDC->SetROP2(ropOld);
}

void STextObject::OnDelete()
{
	// If the users requesting a delete, empty the text string
	SetText(_T(""));
}

BOOL STextObject::ModalSet(const CString &title)
{
	CTextEditDialog teDlg(title, text, font, fontColor);

	if(teDlg.DoModal() == IDOK){
		font = teDlg.GetFont();
		fontColor = teDlg.GetFontColor();
		SetText(teDlg.GetText());

		return TRUE;
	}

	return FALSE;
}

void STextObject::Serialize(CArchive& ar)
{
	SObject::Serialize(ar);

	SerializeFont(font, ar);

	if(ar.IsStoring()){
		ar << fontColor;
		ar << text;
	}
	else{
		ar >> fontColor;
		ar >> text;

		// Parse the text object
		label_text.SetLabel(text, font, fontColor);
	}
}