//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.

// Version control for PrestoPlot

#ifndef __PRESTO_VERSION
#define __PRESTO_VERSION

//	************************ Version Control/ Change Log ********************
//	Version 1.02:
//		- Fixed selection bug for annotated text
//		- Added plot integration feature
//	Version 1.03:
//		- Added plot animation and user defined plot infomation strings
//	Version 1.04
//		- Added ability to read from anonymous pipe (see PrestoLink 
//		helper API and PipeIO.cpp for more info.
//		- Fixed bug in CArrangeGraphDialog and changed behavior of arrange
//		graph function. Now, whenever possible, data is preserved (as 
//		opposed to the original method wich deleted all graphs).
//		- Added titles to file dialogs to reflect their function (Export,
//		Import, etc.).
//		- Fixed bug in error bar dialog (both x and y error bars can now be
//		controlled independantly).
//		- Fixed error bar printing bug (error bar hats were not being scaled
//		correctly).
//		- Added a version number to SgraphDoc. This breaks all older binary
//		saved files!!! It was an unfortunate oversight on my part but needed
//		to move the rows and cols variables from SgraphView to SgraphDoc.
//	Version 1.05
//		- Incremented version number to fix bug in reading error bar data
//		from a saved file (the y error bar data was saved twice - neglecting
//		the x error bar data altogether). Also added error bar properties to 
//		the SPlot::Serialize function.
//		- Added feature to allow import of text values from the clipboard. 
//		This enables data to be easily sent from both spreadsheet and 
//		word processors. Both file and memory IO are now encapsulated by the
//		IOBuffer class which provides a consistent interface for all IO 
//		operations.
//		- Fixed bug that caused crash when plots were deleted while the
//		animation dialog was visable.
//		- Added window size to the status bar when sizing the window.
//	Version 1.06
//		- Added a LaTeX like text engine.
//		- Fixed bug that prevented setting axis tick label font color.
//	Version 1.07
//		- Fixed charset problem in font mapping that only allowed western
//		fonts to be displayed.
//		- Fixed bug in arrange graph dialog (that switched the horizontal and
//		vertical gap sizes).
//		- Added a normalize option to the histogram dialog.
//		- Fixed axis scale bug (newly imported plots didn't get their axis
//		types set).
//		- Fixed bug allocating font resources (windows 95/98/ME only). Caused
//		computer to run out of system (GDI) resources.
//		- Fixed bug pasting an image to the clipboard (windows 95/98/ME only).
//	Version 1.08
//		- Improved the LaTeX parse engine (i.e. curly brace matching, \frac,
//		\sum, \int).
//		- Fixed bug in selecting previously selected text outside a graph
//		border.
//	Version 1.09
//		- Modified the text edit dialog to use a RichEdit Control for improved
//		scroll bar functionality.
//		- Added user defined lines.
//		- Fixed bug printing LaTeX $\frac{}{}$ commands.
//		- Fixed bug printing legends.
//	Version 1.10
//		- Fixed bug in arrange graph dialog.
//		- Fixed text offset bug in class TextElem.
//		- Fixed graph legend spacing bug.
//		- Fixed \leftarrow and \leftrightarrow symbols (they had been 
//		accidently switched).
//		- Fixed set options bug while importing data. Graph strings, like the
//		title and axes titles, are only set if they have not been previously
//		assigned a value.
//		- Added a WYSIWYG mode to PrestoPlot (which makes the screen
//		dimensions match the selected printer dimensions).
//		- Fixed bug in histogram generation (last bin was being omitted).
//		- Added a test for OS which is used to determine 16 bit (i.e. 95, 98 and ME) 
//		or 32 bit (i.e. NT and 00) screen coordinates. Previously assumed 16 bit screen
//		coordinates. See the "about" dialog for current mode.
//		- Added an option to export graph animations as an AVI file.
//	Version 1.11
//		- Modified the file open and save dialogs to always use the current
//		  directory.
//		- Updated world scaling dialog to allow per graph control of 
//		  the scaling.
//		- Fixed tick direction bug in TickMark dialog.
//		- Added a properties dialog with the option to scale all line
//		  thicknesses.
//	Version 1.12
//		- Moved graph dimensions and intergraph spacing to the Document class.
//		- Added the ability to produce dotted and dashed lines with a thickness
//		  greater than 1 (for 32 bit OS's only).
//		- Fixed bug in axis dialog that allowed the user to enter a minor 
//		  tick space equal to zero.
//		- Added a FP_PRECISION number and now test user input in the world
//		  scaling dialog for correct behavior will lower and upper limits 
//		  when using log axis scales.
//	Version 1.13
//		- Added a font scaling option to the properties dialog (not currently
//		  enabled though).
//		- Added the ability to copy (multiple) data sets to the clipboard.
//		- Changed "fudge factor" in SAxisObject::Draw() to fix tick drawing 
//		  bug (fudge factor was previously to large).
//		- Added an overloaded "=" operator to SLegend to avoid shallow copy
//		  semantics (we actually don't want to copy the member pointer of legend
//		  at all!). This fixed a bug that prevented legends from being added to
//		  a graph after the arrange graphs features had been used (which also
//		  crashed the debug but not the release versions of the code).
//		- Added a test to prevent the user from specifying a negative axis range
//		  with a log scale.
//		- Fixed the axis lables to remove a trailing decimal point for the case
//		  of zero precision.
//		- Fixed bug in LaTeX parser that accidently ignored the characters 
//		  ",.;:[]<>+-=/*" as token separators.
//		- Rationalized the behavior of the plot legend when deleting legends.
//		- Fixed line scaling bug that prevented user added lines and plot lines from
//		  rendering correctly under certain circumstances.
//		- Improved error handling when determining font characteristics
//		  (only a factor when sending output to certain printers).
//		- Fixed bug in histogram dialog (scroll bars were accidently deactivated).
//		- Fixed bug in histogram creation (destination graph and new plots
//		  are tested for compatible axes -- i.e. linear or log scale).
//		- Fixed another bug in histogram creation - fixed bin width
//		  histograms were not being normalized!.
//	Version 1.14
//		- Added the ability to import multiple files via the commandline 
//		  (including files with the wildcard).
//		- Added the ability to plot arbitrary one dimensional functions.
//		- Added the ability to curve fit arbitrary one dimensional functions.		
//		- Added an option to the histogram dialog to align the bin center
//		  at the data point (the previous default behavior). If this is
//		  unchecked, the left edge of the bin is aligned with the data point.
//		- Added "|" to the list of token separators for the LaTeX parser.
//		- Fixed string validation to allow the use of engineering notation
//		  to specify some integers (i.e. 1e3 == 1000).
//		- Fixed round off error that caused an offset in major and minor ticks
//		  (tick mark position is now accumulated as double instead of float).
//		- Added default graph title and axis labels.
//		- Improved font memory management.
//	Version 1.15
//		- Added the ability to specify background page color.
//		- Added exit and save & exit functionality to the PrestoLink API.
//		- Fixed a bug that attempted to read too many string into a CListBox
//		  (the maximum is ~MAX_LIST_BOX_ENTRIES, i.e. 3600).
//		- Added polar and parametric plots to the plotting functionality.
//		- Fixed bug in "Apply to All Axes, All Graphs" for the tick mark
//		  and tick lable dialog boxes.
//		- Revamped the non-linear curve fitting engine to use discrete derivatives 
//		  if analytic derivatives can not be computed.
//		- Fixed bug in plot axis determination.
//		- Added a "save plot" feature to the non-linear curve fitting box.
//	Version 1.16
//		- Added the capability to plot vector fields(!).
//		- Holding down Control + Left mouse button displays the plot name of the
//		  closest point.
//		- Added the ability to move and re-scale graphs. Hold Shift + Left mouse
//		  button.
//		- Modified the Arrange Graph dialog to allow arrangement of the 
//		  selected graph.
//	Version 1.17
//		- Fixed bug in drawing graph legends with line thicknesses greater than
//		  1 (for non-solid lines).
//		- Switched to CreatePointFontIndirect (from CreateFontIndirect) to get 
//		  better consistancy when embedding rotated fonts in metfiles.
//		- Fixed text selection bug (selected text was not moveable).
//		- Fixed a graph redraw bug that prevented the display from being erased
//		  if PrestoPlot is totally obscured by other windows.
//		- Added the \perp and \parallel symbols to the LaTeX dictionary.
//		- Cleanup up plot selection with the addition of a PlotListBox class. 
//		  this simplifies the creation of dialog boxes that need to select one
//		  or more plots.
//		- Improved clipboard copy to save both a metafile AND a bitmap to the
//		  clipboard. 
//		- Added selective use of GDI path objects to improve the rendering of
//		  thick dashed and dotted lines.
//
//	Version 1.18
//		- Added axis grid lines.
//		- Modified the wheel mouse zooming feature. The user must now hold down the
//		  control key (Ctrl) while adjusting the wheel. This was done to prevent accidental
//		  zooming.
//		- Modified the middle mouse scrolling behviour. The control key must
//		  be held down to scroll graph.
//		- Added scientific notation to the tick label options.
//		- Fixed a bug in the StringToFloat function that returned numbers that were
//		  outside the range of a float (but within the range of double).
//		- Enabled GDI path rendering for 16 bit operating systems to improve the
//		  consistency of non-solid line types.
//		- Added some "real-time" documentation. Information is displayed on the status
//		  bar to help users understand the mouse and keyboard interface to PrestoPlot.
//		- Improved the WYSIWYG-ness of PrestoPlot by (a) making sure that the relative 
//		  sizes of text characters are conserved when printing in WYSIWYG mode and (b) 
//		  setting the frame aspect ratio to be consistent with the desired client 
//		  dimensions (in WYSIWYG mode).
//
//	Version 1.19
//		- Fixed bug in printing of tick labels. Roundoff allowed labels like 10x10^n to appear
//		  (as opposed to the correct form, 1x10^{n+1}).
//		- Added a button to the tool bar to control WYSIWYG display. This is to
//		  highlight the improvements in PrestoPlots WYSIWYG printing.
//		- Added sticky graph scaling (as requested my many users).
//		- Replaced "Default Scaling" with "Restore Scaling" on the menu bar.
//		- Replaced "World Scaling" with "Define Scaling" on the menu bar.
//		- Replaced the "World Scaling" accelerator (Ctrl + W) with Ctrl +D to match
//		  the new menu label.
//		- Fixed bug that prevented the correct printing of background color.
//		- Added file export for BMP and PNG images. The PNG export code uses 
//		  the zlib and libpng libraries. 
//		- Added functions to the PrestoLink API to export graphs as BMP or PNG
//		  files.
//
//	Version 1.2
//		- Fixed bug in setting log axis scaling. Inactive plots that couldn't
//		  support log scaling were preventing the change from linear to log 
//		  scaling.
//		- Fixed bug in "sticky" scaling (problem switching from log to linear 
//		  plots).
//		- Fixed another bug in "sticky" scaling (default axis ranges were not
//		  set after loading a saved file).
//		- Fixed round off error in scientific format axis labels.
//		- Fixed axis scaling bug that prevent proper scaing in a graph
//		  contined a plot with no points.
//		- Modified the HitTest function to return the (x,y) coordinates of
//		  the closest point when the user presses the left mouse button
//		  while holding the ctrl-key.
//		- Added "ln" as a synonym for "log".
//		- Added "log10" for log base 10.
//
//	Version 1.21
//		- Added an option for user defined axis color (useful for white on
//		  black slides and colorful Powerpoint presentations).
//		- Added command line options to set the axis type:
//			-logx  ... Set the x axis to a log scale
//			-logy  ... Set the y axis to a log scale
//			-logxy ... Set the x and y axes to a log scale
//
//	Version 1.22
//		- Added history lists for the equation and fitting dialogs.
//		- Fixed bug in ASCII file import (trailing white space could
//		  derail the parser).
//		- Fixed bug in non-linear curve fitting that prevented the
//		  real-time display of the fitted equation.
//		- Fixed print preview bug that prevented the correct display
//		  of graph background color.
//
//	Version 1.23
//		- Fixed bug in command line import using wildcards (the full path
//		  was not preserved).
//		- Added the ability to import audio WAV files (both mono and stero
//		  files are supported).
//		- Added the ability to export audio files (to WAV files and the 
//		  sound card).
//		- Added persistant import filter selection (i.e. xy, xydy, etc.).
//		- Fixed bug in graph legend that displayed legends for deactivated 
//		  plots.
//		- Added the option to specify the color of border of the legend 
//		  bounding box.
//		- Added the ability to overlap two graphs (usefull for making plots
//		  with a second x or y axis).

#define		SGRAPH_VERSION		(1.23f)

#endif // __PRESTO_VERSION